@@ -4,10 +4,12 @@
 
 (use gauche.test)
 (use scheme.list)
-(use srfi-13)
+(use srfi.13)
 (use gauche.uvector)
 
-(test-start "file.util")
+(test-start "file.*")
+
+(test-section "file.util")
 (use file.util)
 (cond-expand
  [gauche.os.windows
@@ -49,6 +51,7 @@
 (test* "build-path" (n "foo") (build-path "" "foo"))
 (test* "build-path" (n "foo/bar") (build-path "" "foo" "" "bar"))
 (test* "build-path" (n "foo") (build-path "" "foo" ""))
+(test* "build-path" (n "./a/./b") (build-path #f "a" #f "b"))
 
 (test* "decompose-path" '("/a/b/c" "d" "e")
        (values->list (decompose-path "/a/b/c/d.e")))
@@ -783,4 +786,23 @@
     (cleanup))
   )
 
+;;=====================================================================
+(test-section "file.event")
+(use file.event)
+(test-module 'file.event)
+(test-module 'file.event.common)        ;loaded via file.event
+
+(define-module file-event-generic-test
+  (use gauche.test)
+  (use file.event.generic)
+  (test-module 'file.event.generic))
+
+(define-module file-event-specific-test
+  (use gauche.test)
+  ;; Those submodules are already loaded via file.event
+  (cond-expand
+   [gauche.sys.inotify (test-module 'file.event.inotify)]
+   [gauche.sys.kqueue (test-module 'file.event.kqueue)]
+   [else]))
+
 (test-end)
