@@ -2,7 +2,7 @@
 ;;; peg.scm - Parser Expression Grammar Parser
 ;;;
 ;;;   Copyright (c) 2006 Rui Ueyama (rui314@gmail.com)
-;;;   Copyright (c) 2008-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2008-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,7 +35,7 @@
 (define-module parser.peg
   (use scheme.list)
   (use scheme.charset)
-  (use srfi-13)
+  (use srfi.13)
   (use gauche.collection)
   (use gauche.generator)
   (use gauche.lazy)
