@@ -33,7 +33,7 @@
 
 (use gauche.test)
 (use scheme.list)
-(use srfi-13)
+(use srfi.13)
 (use gauche.generator)
 (use gauche.lazy)
 
@@ -1123,22 +1123,19 @@
                    ("Country"   . "US"))))
   )
 
-(cond-expand
- [gauche.ces.utf8
-  (let1 data `(("[\"\\u03bb\"]" #("\x3bb;"))
-               ("[\"\\ud800\"]" ,(test-error <json-parse-error>))
-               ("[\"\\ud867\\ude3d\\u03bb\"]" #("\x29e3d;\x3bb;"))
-               ("[\"\\ude3d\\ud867\"]" ,(test-error <json-parse-error>))
-               ("[\"\\uf020\\u03bb\"]"  #("\xf020;\x3bb;")))
-    (dolist [d data]
-      (test* (format "unicode escape reading (~s)" (car d))
-             (cadr d)
-             (parse-json-string (car d)))
-      (when (vector? (cadr data))
-        (test* (format "unicode escape writing (~s)" (cadr d))
-               (car d)
-               (construct-json-string (cadr d))))))]
- [else])
+(let1 data `(("[\"\\u03bb\"]" #("\x3bb;"))
+             ("[\"\\ud800\"]" ,(test-error <json-parse-error>))
+             ("[\"\\ud867\\ude3d\\u03bb\"]" #("\x29e3d;\x3bb;"))
+             ("[\"\\ude3d\\ud867\"]" ,(test-error <json-parse-error>))
+             ("[\"\\uf020\\u03bb\"]"  #("\xf020;\x3bb;")))
+  (dolist [d data]
+    (test* (format "unicode escape reading (~s)" (car d))
+           (cadr d)
+           (parse-json-string (car d)))
+    (when (vector? (cadr data))
+      (test* (format "unicode escape writing (~s)" (cadr d))
+             (car d)
+             (construct-json-string (cadr d))))))
 
 (let ()
   (define (t obj)
