@@ -36,6 +36,7 @@
 #define GAUCHE_TLS_H
 
 #include <gauche.h>
+#include <gauche/net.h>
 
 #if defined(EXTTLS_EXPORTS)
 #define LIBGAUCHE_EXT_BODY
@@ -52,27 +53,6 @@
     "/etc/pki/tls/certs/ca-budle.crt",    /* fedora (compat) */         \
     "/usr/local/etc/openssl/cert.pem"     /* osx homebrew openssl */
 
-#if defined(GAUCHE_USE_AXTLS)
-#  if defined(GAUCHE_WINDOWS)
-#    include <ws2tcpip.h>
-#  endif /*GAUCHE_WINDOWS*/
-#include "axTLS/ssl/ssl.h"
-#endif  /* GAUCHE_USE_MBEDTLS */
-
-#if defined(GAUCHE_USE_MBEDTLS)
-#include <mbedtls/version.h>
-#include <mbedtls/ssl.h>
-#include <mbedtls/ctr_drbg.h>
-#include <mbedtls/entropy.h>
-
-#ifdef HAVE_MBEDTLS_NET_SOCKETS_H
-#include <mbedtls/net_sockets.h>
-#elif  HAVE_MBEDTLS_NET_H
-#include <mbedtls/net.h>
-#endif
-
-#endif  /*GAUCHE_USE_MBEDTLS*/
-
 /* This is a 'success' code of AxTLS.  mbedtls x509 routine returns 0
    on success, which happens to be the same as SSL_OK.
    We just need this in case we're configured with mbedtls and without axtls.
@@ -112,19 +92,19 @@ typedef struct ScmTLSRec ScmTLS;
 
 struct ScmTLSRec {
     SCM_HEADER;
-    ScmObj sock;                /* <socket> */
     ScmObj in_port;
     ScmObj out_port;
 
     ScmObj (*connect)(ScmTLS*, const char*, const char*, int);
-    ScmObj (*connectSock)(ScmTLS*, int);
     ScmObj (*bind)(ScmTLS*, const char*, const char*, int);
     ScmObj (*accept)(ScmTLS*);
-    ScmObj (*acceptSock)(ScmTLS*, int);
     ScmObj (*read)(ScmTLS*);
     ScmObj (*write)(ScmTLS*, ScmObj);
     ScmObj (*close)(ScmTLS*);
-    ScmObj (*loadObject)(ScmTLS*, ScmObj, const char*, const char*);
+    u_long (*poll)(ScmTLS*, u_long, ScmTimeSpec*);
+    ScmObj (*loadCertificate)(ScmTLS*, const char*);
+    ScmObj (*loadPrivateKey)(ScmTLS*, const char*, const char*);
+    ScmObj (*getConnectionAddress)(ScmTLS*, int);
     void   (*finalize)(ScmObj, void*);
 };
 
@@ -132,36 +112,40 @@ SCM_CLASS_DECL(Scm_TLSClass);
 #if defined(GAUCHE_USE_AXTLS)
 SCM_CLASS_DECL(Scm_AxTLSClass);
 #endif /*GAUCHE_USE_AXTLS*/
-#if defined(GAUCHE_USE_MBEDTLS)
-SCM_CLASS_DECL(Scm_MbedTLSClass);
-#endif /*GAUCHE_USE_MBEDTLS*/
 
 #define SCM_CLASS_TLS   (&Scm_TLSClass)
 #define SCM_TLS(obj)    ((ScmTLS*)obj)
 #define SCM_TLSP(obj)   SCM_ISA(obj, SCM_CLASS_TLS)
 
 extern ScmObj Scm_MakeTLS(ScmObj);
-extern ScmObj Scm_TLSDestroy(ScmTLS* t);
-extern ScmObj Scm_TLSLoadObject(ScmTLS* t, ScmObj obj_type,
-                                const char *filename,
-                                const char *password);
+extern ScmObj Scm_TLSLoadCertificate(ScmTLS *t, const char *path);
+extern ScmObj Scm_TLSLoadPrivateKey(ScmTLS *t, const char *path,
+                                    const char *password);
 extern ScmObj Scm_TLSConnect(ScmTLS *t,
                              const char *host,
                              const char *port, /* number or service name */
                              ScmObj proto);
-extern ScmObj Scm_TLSConnectWithSocket(ScmTLS* t, ScmObj sock, int fd);
+
 extern ScmObj Scm_TLSBind(ScmTLS *t,
                           const char *ip,
                           const char *port, /* numeric or service name */
-                          int proto);
+                          ScmObj proto);
 extern ScmObj Scm_TLSAccept(ScmTLS *t); /* returns connected <tls> */
-extern ScmObj Scm_TLSAcceptWithSocket(ScmTLS* t, ScmObj sock, int fd);
 extern ScmObj Scm_TLSClose(ScmTLS* t);
-extern ScmObj Scm_TLSSocket(ScmTLS *t);
 
 extern int    Scm_TLSSystemCABundleAvailable(void);
 
+extern ScmObj Scm_TLSGetConnectionAddress(ScmTLS *t, int who);
+enum {
+    TLS_SELF_ADDRESS = 0,
+    TLS_PEER_ADDRESS = 1
+};
 
+extern u_long Scm_TLSPoll(ScmTLS *t, u_long rwflags, ScmTimeSpec *timeout);
+enum {
+    TLS_POLL_READ  = 1,
+    TLS_POLL_WRITE = 2
+};
 
 /*
    KZ: presumably due to block sizes imposed by the crypto algorithms
@@ -179,6 +163,7 @@ extern ScmObj Scm_TLSOutputPort(ScmTLS* t);
 extern ScmObj Scm_TLSInputPortSet(ScmTLS* t, ScmObj port);
 extern ScmObj Scm_TLSOutputPortSet(ScmTLS* t, ScmObj port);
 
+
 SCM_DECL_END
 
 #endif /*GAUCHE_TLS_H */
