@@ -38,32 +38,32 @@
   (use gauche.connection)
   (use gauche.net)
   (use util.match)
-  (export <tls> make-tls tls-destroy tls-connect tls-accept tls-close
-          tls-load-object tls-read tls-write
+  (export <tls> make-tls tls-connect
+          tls-bind tls-accept tls-poll tls-close
+          tls-load-certificate tls-load-private-key
+          tls-read tls-write
           tls-input-port tls-output-port
           tls-ca-bundle-path
           default-tls-class
 
+          ;; connection interface
+          connection-self-address connection-peer-address
+          connection-input-port connection-output-port
+          connection-shutdown connection-close
+
+          ;; deprecated interface
           SSL_SERVER_VERIFY_LATER SSL_CLIENT_AUTHENTICATION
           SSL_DISPLAY_BYTES SSL_DISPLAY_STATES SSL_DISPLAY_CERTS
           SSL_DISPLAY_RSA SSL_CONNECT_IN_PARTS SSL_NO_DEFAULT_KEY
           SSL_OBJ_X509_CERT SSL_OBJ_X509_CACERT SSL_OBJ_RSA_KEY
           SSL_OBJ_PKCS8 SSL_OBJ_PKCS12
-
-          ;; connection interface
-          connection-self-address connection-peer-address
-          connection-input-port connection-output-port
-          connection-shutdown connection-close)
+          tls-load-object tls-destroy
+)
   )
 (select-module rfc.tls)
 
-(export-if-defined <ax-tls>)
-
 ;; The initialization of default-tls-class depends on the availability
 ;; of classes and tls-ca-bundle-path.
-;;
-;; If tls-ca-bundle-path is #f, we favor <ax-tls>, for <mbed-tls> doensn't work
-;; without ca-bundle.  Othwerise, we favor <mbed-tls>.
 
 (without-precompiling
  (cond-expand
@@ -75,18 +75,12 @@
      (if (%tls-system-ca-bundle-available?)
        (tls-ca-bundle-path 'system)
        (tls-ca-bundle-path #f)))
-   (if (and (not (tls-ca-bundle-path))
-            (global-variable-bound? (find-module 'rfc.tls) '<ax-tls>))
-     ;; if system ca-bundle isn't available, we give up mbed-tls.
-     (default-tls-class (delay <ax-tls>))
-     (default-tls-class (delay <mbed-tls>)))
+   (default-tls-class (delay <mbed-tls>))
    ]
   [else
-   ;; For the backward compatibility, the default configuration with only
-   ;; axtls assumes tls-ca-bundle-path is #f.
    (when (eq? (tls-ca-bundle-path) 'check)
      (tls-ca-bundle-path #f))
-   (default-tls-class (delay <ax-tls>))]))
+   (default-tls-class #f)]))
 
 (inline-stub
  (declcode
@@ -94,6 +88,58 @@
 
  (declare-stub-type <tls> "ScmTLS*")
 
+ (define-cproc make-tls (:rest initargs) Scm_MakeTLS)
+ (define-cproc tls-load-certificate (tls::<tls> filename::<const-cstring>)
+   Scm_TLSLoadCertificate)
+ (define-cproc tls-load-private-key (tls::<tls>
+                                     filename::<const-cstring>
+                                     password::<const-cstring>?)
+   Scm_TLSLoadPrivateKey)
+ (define-cproc %tls-connect (tls::<tls>
+                             host::<const-cstring>
+                             port::<const-cstring>
+                             proto)
+   Scm_TLSConnect)
+ (define-cproc %tls-bind (tls::<tls>
+                          ip::<const-cstring>?
+                          port::<const-cstring>
+                          proto)
+   Scm_TLSBind)
+ (define-cproc %tls-accept (tls::<tls>) Scm_TLSAccept)
+ (define-cproc %tls-close (tls::<tls>) Scm_TLSClose)
+ (define-cproc tls-read (tls::<tls>) Scm_TLSRead)
+ (define-cproc tls-write (tls::<tls> msg) Scm_TLSWrite)
+ (define-cproc tls-input-port (tls::<tls>) Scm_TLSInputPort)
+ (define-cproc tls-output-port (tls::<tls>) Scm_TLSOutputPort)
+ (define-cproc tls-poll (tls::<tls> rwflags::<list> :optional (timeout #f))
+   (let* ([ts::ScmTimeSpec]
+          [pts::ScmTimeSpec* (Scm_GetTimeSpec timeout (& ts))]
+          [iflags::u_long 0])
+     (for-each (lambda (f)
+                 (cond
+                  [(SCM_EQ f 'read) (logior= iflags TLS_POLL_READ)]
+                  [(SCM_EQ f 'write) (logior= iflags TLS_POLL_WRITE)]
+                  [else
+                   (Scm_Error "List of 'read and 'write expected, but got: %S"
+                              rwflags)]))
+               rwflags)
+     (let* ([r::u_long (Scm_TLSPoll tls iflags pts)]
+            [result SCM_NIL])
+       (when (logand r TLS_POLL_READ)
+         (set! result (Scm_Cons 'read result)))
+       (when (logand r TLS_POLL_WRITE)
+         (set! result (Scm_Cons 'write result)))
+       (return result))))
+
+ ;; internal
+ (define-cproc %tls-input-port-set! (tls::<tls> port) Scm_TLSInputPortSet)
+ (define-cproc %tls-output-port-set! (tls::<tls> port) Scm_TLSOutputPortSet)
+ (define-cproc %tls-get-self-address (tls::<tls>)
+   (return (Scm_TLSGetConnectionAddress tls TLS_SELF_ADDRESS)))
+ (define-cproc %tls-get-peer-address (tls::<tls>)
+   (return (Scm_TLSGetConnectionAddress tls TLS_PEER_ADDRESS)))
+
+ ;; DEPRECATED APIs
  (define-enum SSL_SERVER_VERIFY_LATER)
  (define-enum SSL_CLIENT_AUTHENTICATION)
  (define-enum SSL_NO_DEFAULT_KEY)
@@ -107,55 +153,54 @@
  (define-enum SSL_OBJ_RSA_KEY)
  (define-enum SSL_OBJ_PKCS8)
  (define-enum SSL_OBJ_PKCS12)
-
- (define-cproc make-tls (:rest initargs) Scm_MakeTLS)
- (define-cproc tls-load-object (tls::<tls> obj-type filename::<const-cstring>
-                                           :optional (password::<const-cstring>? #f)) Scm_TLSLoadObject)
- (define-cproc %tls-destroy (tls::<tls>) Scm_TLSDestroy)
- (define-cproc %tls-connect (tls::<tls>
-                             host::<const-cstring>
-                             port::<const-cstring>
-                             proto)
-   Scm_TLSConnect)
- (define-cproc %tls-connect-with-socket (tls::<tls> sock fd::<long>)
-   Scm_TLSConnectWithSocket)
- (define-cproc %tls-accept-with-socket (tls::<tls> sock fd::<long>)
-   Scm_TLSAcceptWithSocket)
- (define-cproc %tls-accept (tls::<tls>) Scm_TLSAccept)
- (define-cproc %tls-close (tls::<tls>) Scm_TLSClose)
- (define-cproc tls-read (tls::<tls>) Scm_TLSRead)
- (define-cproc tls-write (tls::<tls> msg) Scm_TLSWrite)
- (define-cproc tls-input-port (tls::<tls>) Scm_TLSInputPort)
- (define-cproc tls-output-port (tls::<tls>) Scm_TLSOutputPort)
- (define-cproc tls-socket (tls::<tls>) Scm_TLSSocket)
- ;; internal
- (define-cproc tls-input-port-set! (tls::<tls> port) Scm_TLSInputPortSet)
- (define-cproc tls-output-port-set! (tls::<tls> port) Scm_TLSOutputPortSet)
+ (define-cproc tls-load-object (tls::<tls>
+                                obj-type::<fixnum>
+                                filename::<const-cstring>
+                                :optional (password::<const-cstring>? #f))
+   ::<void>
+   (Scm_Warn "tls-load-object is deprecated.  Use tls-load-certificate or \
+              tls-load-private-key.\n")
+   (case obj-type
+     [(SSL_OBJ_X509_CERT SSL_OBJ_X509_CACERT)
+      (Scm_TLSLoadCertificate tls filename)]
+     [(SSL_OBJ_RSA_KEY SSL_OBJ_PKCS8 SSL_OBJ_PKCS12)
+      (Scm_TLSLoadPrivateKey tls filename password)]
+     [else (Scm_Error "Invalid obj-type: %d" obj-type)]))
 
  (declcode "void Scm_Init_tls(ScmModule *);")
  (initcode "Scm_Init_tls(Scm_CurrentModule());")
  )
 
 ;; API
-(define (tls-connect tls . args)
-  (match args
-    [((? (cut is-a? <> <socket>) sock))
-     (%tls-connect-with-socket tls sock (socket-fd sock))]
-    [(host port proto)
-     (%tls-connect tls host port proto)]
-    [_ (error "Invalid arguments:" (cons tls args))])
-  (tls-input-port-set! tls (make-tls-input-port tls))
-  (tls-output-port-set! tls (make-tls-output-port tls))
+(define (tls-connect tls host port :optional (proto 'tcp))
+  (assume-type port (</> <string> <integer>))
+  (let1 p (if (integer? port)
+            (if (or (inexact? port) (negative? port))
+              (error "Nonnegative exact integer or string expected, but got:"
+                     port)
+              (x->string port))
+            port)
+    (%tls-connect tls host p proto)
+    (%tls-input-port-set! tls (make-tls-input-port tls))
+    (%tls-output-port-set! tls (make-tls-output-port tls)))
   tls)
 
 ;; API
-(define (tls-accept tls :optional (sock #f))
-  (rlet1 new-tls
-      (if sock
-        (%tls-accept-with-socket tls sock (socket-fd sock))
-        (%tls-accept tls))
-    (tls-input-port-set! new-tls (make-tls-input-port new-tls))
-    (tls-output-port-set! new-tls (make-tls-output-port new-tls))))
+(define (tls-bind tls host port :optional (proto 'tcp))
+  (assume-type port (</> <string> <integer>))
+  (let1 p (if (integer? port)
+            (if (or (inexact? port) (negative? port))
+              (error "Nonnegative exact integer or string expected, but got:"
+                     port)
+              (x->string port))
+            port)
+    (%tls-bind tls host p proto)))
+
+;; API
+(define (tls-accept tls)
+  (rlet1 new-tls (%tls-accept tls)
+    (%tls-input-port-set! new-tls (make-tls-input-port new-tls))
+    (%tls-output-port-set! new-tls (make-tls-output-port new-tls))))
 
 ;; API
 (define (tls-close t)
@@ -163,15 +208,7 @@
     (close-input-port (tls-input-port t)))
   (when (output-port? (tls-output-port t))
     (close-output-port (tls-output-port t)))
-  (%tls-close t)
-  (when (is-a? (tls-socket t) <socket>)
-    (socket-shutdown (tls-socket t))))
-
-;; API
-(define (tls-destroy t)
-  (%tls-destroy t)
-  (when (is-a? (tls-socket t) <socket>)
-    (socket-close (tls-socket t))))
+  (%tls-close t))
 
 ;; Internal
 (define-cproc %tls-system-ca-bundle-available? () ::<boolean>
@@ -181,25 +218,30 @@
   (rlet1 ip (make <virtual-input-port>)
     (set! (~ ip'getb)
           (let ((buf #f) (pos 0) (size 0))
-            (^[]
-              (unless buf
-                (set! buf (tls-read tls))
-                (set! size (string-size buf))
-                (set! pos 0))
-              (rlet1 r (string-byte-ref buf pos)
-                (set! pos (+ pos 1))
-                (when (= pos size) (set! buf #f))))))))
+            (rec (reader)
+              (if buf
+                (rlet1 r (string-byte-ref buf pos)
+                  (set! pos (+ pos 1))
+                  (when (= pos size) (set! buf #f)))
+                (let1 data (tls-read tls)
+                  (if (eof-object? data)
+                    data
+                    (begin
+                      (set! buf data)
+                      (set! size (string-size buf))
+                      (set! pos 0)
+                      (reader))))))))))
 
 (define (make-tls-output-port tls)
   (rlet1 op (make <virtual-output-port>)
     (set! (~ op'puts) (^[msg] (tls-write tls msg)))
     (set! (~ op'putb) (^[b] (tls-write tls (make-byte-string 1 b))))))
 
-;; Connection interface
-(define-method connection-self-address ((s <tls>))
-  (socket-getsockname (tls-socket s)))
-(define-method connection-peer-address ((s <tls>))
-  (socket-getpeername (tls-socket s)))
+;; Deprecated
+(define (tls-destroy tls) (tls-close tls))
+
+(define-method connection-self-address ((s <tls>)) (%tls-get-self-address s))
+(define-method connection-peer-address ((s <tls>)) (%tls-get-peer-address s))
 (define-method connection-input-port ((s <tls>))
   (tls-input-port s))
 (define-method connection-output-port ((s <tls>))
