@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.uvector - uniform vectors
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -53,6 +53,8 @@
           read-bytevector read-bytevector!
           referencer
 
+          ;; string <-> uvector operations are now in libvec.scm
+          ;; we reexport them for the compatibility.
           string->s8vector string->s8vector!
           string->u8vector string->u8vector!
           string->s32vector string->s32vector!
@@ -60,7 +62,8 @@
           s8vector->string u8vector->string
           s32vector->string u32vector->string
 
-          uvector-alias uvector-binary-search uvector-class-element-size
+          uvector-alias uvector-segment/shared
+          uvector-binary-search uvector-class-element-size
           uvector-copy uvector-copy! uvector-ref uvector-set! uvector-size
           uvector->list uvector->vector uvector-swap-bytes uvector-swap-bytes!
 
@@ -81,7 +84,7 @@
 ;; it is expected to belong gauche.uvector.
 (autoload gauche.vport open-output-uvector get-output-uvector)
 
-;; referenced from srfi-160 make-TAGvector-generator
+;; referenced from SRFI-160 make-TAGvector-generator
 (autoload gauche.generator uvector->generator)
 
 (inline-stub
@@ -105,6 +108,19 @@
  (define-cproc uvector-alias
    (klass::<class> v::<uvector> :optional (start::<fixnum> 0) (end::<fixnum> -1))
    Scm_UVectorAlias)
+
+ (define-cproc uvector-segment/shared (v::<uvector> n::<fixnum>)
+   (unless (> n 0)
+     (Scm_Error "Positive exact integer required, but got: %d" n))
+   (let* ([len::ScmSmallInt (SCM_UVECTOR_SIZE v)]
+          [i::ScmSmallInt 0]
+          [h SCM_NIL] [t SCM_NIL])
+     (while (< i len)
+       (let* ([vv (Scm_UVectorAlias (Scm_ClassOf (SCM_OBJ v))
+                                    v i (?: (> (+ i n) len) len (+ i n)))])
+         (SCM_APPEND1 h t vv))
+       (set! i (+ i n)))
+     (return h)))
  )
 
 ;; byte swapping
@@ -129,9 +145,9 @@
 ;; uvector-size
 (inline-stub
  (define-cproc uvector-size (v::<uvector>
-                             :optional (start::<int> 0) (end::<int> -1))
-   ::<int>
-   (let* ([len::int (SCM_UVECTOR_SIZE v)])
+                             :optional (start::<fixnum> 0) (end::<fixnum> -1))
+   ::<fixnum>
+   (let* ([len::ScmSmallInt (SCM_UVECTOR_SIZE v)])
      (SCM_CHECK_START_END start end len)
      (return (* (- end start)
                 (Scm_UVectorElementSize (Scm_ClassOf (SCM_OBJ v)))))))
@@ -285,7 +301,7 @@
  )
 
 ;; search
-;; rounding can be #f, 'floor or 'ceiling  (srfi-114 also uses symbols
+;; rounding can be #f, 'floor or 'ceiling  (SRFI-114 also uses symbols
 ;; for rounding.  we don't use 'round and 'truncate, though, for
 ;; it doesn't make much sense.)
 (inline-stub
@@ -447,7 +463,7 @@
 
 ;; copy
 (inline-stub
- (define-cproc uvector-copy! (dest::<uvector> dstart::<int> src::<uvector>
+ (define-cproc uvector-copy! (dest::<uvector> dstart::<fixnum> src::<uvector>
                               :optional (sstart::<fixnum> 0)
                                         (send::<fixnum> -1))
    ::<void>
@@ -463,257 +479,6 @@
               size)))
  )
 
-;; String operations
-(inline-stub
- ;; A common operation to extract range of char* from the input string S.
- ;; START and END may be adjusted.
- ;; SP and EP are const char* variable that gets start and end pointers.
- (define-cise-stmt with-input-string-pointers
-   [(_ (s start end sp ep) . body)
-    (let ([sb (gensym)] [size (gensym)] [len (gensym)] [ss (gensym)])
-      `(let* ([,sb :: (const ScmStringBody*) (SCM_STRING_BODY ,s)]
-              [,size :: ScmSize (SCM_STRING_BODY_SIZE ,sb)]
-              [,len :: ScmSize (SCM_STRING_BODY_LENGTH ,sb)]
-              [,ss :: (const char*) (SCM_STRING_BODY_START ,sb)])
-         (SCM_CHECK_START_END ,start ,end (cast int ,len))
-         (let* ([,sp :: (const char*)
-                     (?: (== ,start 0)
-                         ,ss
-                         (Scm_StringBodyPosition ,sb ,start))]
-                [,ep :: (const char*)
-                     (?: (== ,end ,len)
-                         (+ ,ss ,size)
-                         (Scm_StringBodyPosition ,sb ,end))])
-           ,@body)))])
-
- (define-cfn string->bytevector (klass::ScmClass*
-                                 s::ScmString*
-                                 start::ScmSmallInt
-                                 end::ScmSmallInt
-                                 immutable::int)
-   :static
-   (with-input-string-pointers (s start end sp ep)
-     (let* ([buf::char* NULL])
-       (if immutable
-         (set! buf (cast char* sp))  ; Eek! drop const qualifier
-         (begin
-           (set! buf (SCM_NEW_ATOMIC2 (char*) (- ep sp)))
-           (memcpy buf sp (- ep sp))))
-       (return (Scm_MakeUVectorFull klass (cast ScmSmallInt (- ep sp)) buf
-                                    immutable NULL)))))
-
- (define-cproc string->s8vector
-   (s::<string>
-    :optional (start::<fixnum> 0) (end::<fixnum> -1) (immutable?::<boolean> #f))
-   (return (string->bytevector SCM_CLASS_S8VECTOR s start end immutable?)))
-
- (define-cproc string->u8vector
-   (s::<string>
-    :optional (start::<fixnum> 0) (end::<fixnum> -1) (immutable?::<boolean> #f))
-   (return (string->bytevector SCM_CLASS_U8VECTOR s start end immutable?)))
-
- (define-cfn string->bytevector! (v::ScmUVector*
-                                  tstart::ScmSmallInt
-                                  s::ScmString*
-                                  start::ScmSmallInt
-                                  end::ScmSmallInt)
-   :static
-   (let* ([tlen::ScmSmallInt (SCM_UVECTOR_SIZE v)])
-     (when (and (>= tstart 0) (< tstart tlen))
-       (SCM_UVECTOR_CHECK_MUTABLE v)
-       (with-input-string-pointers (s start end sp ep)
-         (let* ([buf::(char*) (+ (cast char* (SCM_UVECTOR_ELEMENTS v)) tstart)])
-           (if (> (- tlen tstart) (- ep sp))
-             (memcpy buf sp (- ep sp))
-             (memcpy buf sp (- tlen tstart))))))
-     (return (SCM_OBJ v))))
-
- (define-cproc string->s8vector! (v::<s8vector>
-                                  tstart::<fixnum>
-                                  s::<string>
-                                  :optional (start::<fixnum> 0)
-                                  (end::<fixnum> -1))
-   (return (string->bytevector! (SCM_UVECTOR v) tstart s start end)))
-
- (define-cproc string->u8vector! (v::<u8vector>
-                                  tstart::<fixnum>
-                                  s::<string>
-                                  :optional (start::<fixnum> 0)
-                                  (end::<fixnum> -1))
-   (return (string->bytevector! (SCM_UVECTOR v) tstart s start end)))
-
- (define-cfn bytevector->string (v::ScmUVector*
-                                 start::ScmSmallInt
-                                 end::ScmSmallInt
-                                 term)
-   :static
-   (let* ([len::ScmSmallInt (SCM_UVECTOR_SIZE v)])
-     ;; We automatically avoid copying the string contents when the
-     ;; following conditions are met:
-     ;; * The source vector is immutable
-     ;; * The owner of source vector is NULL (If there's an owner such as
-     ;;   mmap handle, it isn't desirable if a string points to the memory
-     ;;   without keeping ownership info.)
-     ;; * The resulting string is not a small fraction of a large vector.
-     ;;   If so, we may waste space by retaining large chunk of memory
-     ;;   most of which won't be ever used.  Here we use some heuristics:
-     ;;   - If the source vector is not small (>= 256)
-     ;;   - and the string covers only a fraction (1/5) or less,
-     ;;   - then we copy the content.
-     ;; NB: We may add a flag that force the content to be shared, for
-     ;; the programs that really want to avoid allocation.
-     (SCM_CHECK_START_END start end len)
-     (let* ([flags::int (?: (and (SCM_UVECTOR_IMMUTABLE_P v)
-                                 (== (-> v owner) NULL)
-                                 (not (and (>= len 256)
-                                           (<= (- end start) (/ len 5)))))
-                            0
-                            SCM_STRING_COPYING)])
-       (when (SCM_INTP term)
-         (let* ([terminator::u_char (logand #xff (SCM_INT_VALUE term))]
-                [i::ScmSmallInt])
-           (for [(set! i start) (< i end) (post++ i)]
-             (when (== terminator
-                       (aref (cast u_char* (SCM_UVECTOR_ELEMENTS v)) i))
-               (set! end i)
-               (break)))))
-       (return (Scm_MakeString (+ (cast char* (SCM_UVECTOR_ELEMENTS v)) start)
-                               (- end start) -1 flags)))))
-
- (define-cproc s8vector->string (v::<s8vector>
-                                 :optional (start::<fixnum> 0)
-                                           (end::<fixnum> -1)
-                                           (terminator #f))
-   (return (bytevector->string (SCM_UVECTOR v) start end terminator)))
-
- (define-cproc u8vector->string (v::<u8vector>
-                                 :optional (start::<fixnum> 0)
-                                           (end::<fixnum> -1)
-                                           (terminator #f))
-   (return (bytevector->string (SCM_UVECTOR v) start end terminator)))
-
- (define-cfn string->wordvector (klass::ScmClass* s::ScmString*
-                                 start::ScmSmallInt end::ScmSmallInt
-                                 endian::ScmObj)
-   :static
-   (unless (SCM_SYMBOLP endian)
-     (set! endian (Scm_DefaultEndian)))
-   (with-input-string-pointers (s start end sp ep)
-     (let* ([v (Scm_MakeUVector klass (- end start) NULL)]
-            [eltp::uint32_t* (cast uint32_t* (SCM_UVECTOR_ELEMENTS v))]
-            [i::ScmSmallInt 0]
-            [do_swap::int (SWAP_REQUIRED endian)])
-       (for [() (< sp ep) (post++ i)]
-         (let* ([ch::ScmChar])
-           (SCM_CHAR_GET sp ch)
-           (if do_swap
-             (let* ([v::swap_u32_t])
-               (set! (ref v val) (cast uint32_t ch))
-               (SWAP_4 v)
-               (set! (aref eltp i) (ref v val)))
-             (set! (aref eltp i) (cast uint32_t ch)))
-           (+= sp (SCM_CHAR_NBYTES ch))))
-       (return v))))
-
- (define-cproc string->s32vector (s::<string>
-                                  :optional (start::<fixnum> 0)
-                                            (end::<fixnum> -1)
-                                            endian)
-   (return (string->wordvector SCM_CLASS_S32VECTOR s start end endian)))
-
- (define-cproc string->u32vector (s::<string>
-                                  :optional (start::<fixnum> 0)
-                                            (end::<fixnum> -1)
-                                            endian)
-   (return (string->wordvector SCM_CLASS_U32VECTOR s start end endian)))
-
- (define-cfn string->wordvector! (v::ScmUVector*
-                                  tstart::ScmSmallInt
-                                  s::ScmString*
-                                  start::ScmSmallInt
-                                  end::ScmSmallInt
-                                  endian::ScmObj)
-   :static
-   (unless (SCM_SYMBOLP endian)
-     (set! endian (Scm_DefaultEndian)))
-   (let* ([tlen::ScmSmallInt (SCM_UVECTOR_SIZE v)])
-     (when (and (>= tstart 0) (< tstart tlen))
-       (SCM_UVECTOR_CHECK_MUTABLE v)
-       (with-input-string-pointers (s start end sp ep)
-         (let* ([buf::uint32_t* (cast uint32_t* (SCM_UVECTOR_ELEMENTS v))]
-                [i::ScmSmallInt tstart]
-                [do_swap::int (SWAP_REQUIRED endian)])
-           (for [() (and (< sp ep) (< i tlen)) (post++ i)]
-             (let* ([ch::ScmChar])
-               (SCM_CHAR_GET sp ch)
-               (if do_swap
-                 (let* ([v::swap_u32_t])
-                   (set! (ref v val) (cast uint32_t ch))
-                   (SWAP_4 v)
-                   (set! (aref buf i) (ref v val)))
-                 (set! (aref buf i) (cast uint32_t ch)))
-               (+= sp (SCM_CHAR_NBYTES ch)))))))
-     (return (SCM_OBJ v))))
-
- (define-cproc string->s32vector! (v::<s32vector>
-                                   tstart::<fixnum>
-                                   s::<string>
-                                   :optional (start::<fixnum> 0)
-                                             (end::<fixnum> -1)
-                                             endian)
-   (return (string->wordvector! (SCM_UVECTOR v) tstart s start end endian)))
-
- (define-cproc string->u32vector! (v::<u32vector>
-                                   tstart::<fixnum>
-                                   s::<string>
-                                   :optional (start::<fixnum> 0)
-                                             (end::<fixnum> -1)
-                                             endian)
-   (return (string->wordvector! (SCM_UVECTOR v) tstart s start end endian)))
-
- (define-cfn wordvector->string (v::ScmUVector*
-                                 start::ScmSmallInt
-                                 end::ScmSmallInt
-                                 term
-                                 endian)
-   :static
-   (unless (SCM_SYMBOLP endian)
-     (set! endian (Scm_DefaultEndian)))
-   (let* ([len::ScmSmallInt (SCM_UVECTOR_SIZE v)]
-          [s (Scm_MakeOutputStringPort FALSE)]
-          [do_swap::int (SWAP_REQUIRED endian)])
-     (SCM_CHECK_START_END start end len)
-     (let* ([eltp::int32_t* (cast int32_t* (SCM_UVECTOR_ELEMENTS v))])
-       (while (< start end)
-         (let* ([ch::ScmChar]
-                [val::uint32_t (aref eltp (post++ start))])
-           (if do_swap
-             (let* ([v::swap_u32_t])
-               (set! (ref v val) val)
-               (SWAP_4 v)
-               (set! ch (cast ScmChar (ref v val))))
-             (set! ch (cast ScmChar val)))
-           (when (and (SCM_INTP term)
-                      (== (SCM_INT_VALUE term) ch))
-             (break))
-           (Scm_PutcUnsafe ch (SCM_PORT s)))))
-     (return (Scm_GetOutputStringUnsafe (SCM_PORT s) 0))))
-
- (define-cproc s32vector->string (v::<s32vector>
-                                  :optional (start::<fixnum> 0)
-                                            (end::<fixnum> -1)
-                                            (terminator #f)
-                                            endian)
-   (return (wordvector->string (SCM_UVECTOR v) start end terminator endian)))
-
- (define-cproc u32vector->string (v::<u32vector>
-                                  :optional (start::<fixnum> 0)
-                                            (end::<fixnum> -1)
-                                            (terminator #f)
-                                            endian)
-   (return (wordvector->string (SCM_UVECTOR v) start end terminator endian)))
- )
-
 ;; for the bakcward compatibility
 (define read-block! read-uvector!)
 (define write-block write-uvector)
@@ -728,7 +493,7 @@
     (get-output-uvector p :shared #t)))
 
 ;;-------------------------------------------------------------
-;; Element range check (srfi-160)
+;; Element range check (SRFI-160)
 ;;
 
 (define-cproc u8? (v) ::<boolean>
