@@ -10,7 +10,7 @@
 (use gauche.sequence)
 (use gauche.generator)
 (use scheme.list)
-(use srfi-13)
+(use srfi.13)
 (use file.util)
 (test-module 'gauche.vport)
 
@@ -133,6 +133,27 @@
                              buf)))))
          (read-block 10 p)))
 
+;; regression tests for peek consuming EOF
+(test* "peek-char EOF regression" (make-list 3 (eof-object))
+       (let* ((state #f)
+              (p (make <virtual-input-port>
+                   :getc (lambda ()
+                           (set! state (not state))
+                           (if state (eof-object) #\a)))))
+         (list (peek-char p)
+               (peek-byte p)
+               (read-char p))))
+
+(test* "peek-byte EOF regression" (make-list 3 (eof-object))
+       (let* ((state #f)
+              (p (make <virtual-input-port>
+                   :getc (lambda ()
+                           (set! state (not state))
+                           (if state (eof-object) #\a)))))
+         (list (peek-byte p)
+               (peek-char p)
+               (read-byte p))))
+
 ;;-----------------------------------------------------------
 (test-section "virtual-output-port")
 
@@ -248,7 +269,7 @@
 
 (let ()
   (define (test-biport file size)
-    (let* ((ifile #"~(sys-dirname (current-load-path))/~file")
+    (let* ((ifile (build-path 'cld file))
            (src (open-input-file ifile))
            (p (apply make <buffered-input-port>
                      :fill  (lambda (buf) (read-block! buf src))
@@ -302,12 +323,22 @@
            (list a b s c)))
   )
 
+;; buffered input port and buffering mode
+(let ()
+  (define p (make <buffered-input-port>
+              :fill (^[buf] (u8vector-set! buf 0 #x20) 1)
+              :ready (^[] #f)))
+  (set! (port-buffering p) :none)
+  (test* "buffered-input-port premature return" '#u8(#x20)
+         (read-uvector <u8vector> 20 p))
+  )
+
 ;;-----------------------------------------------------------
 (test-section "buffered-output-port")
 
 (let ()
   (define (test-boport file size)
-    (let* ((ifile #"~(sys-dirname (current-load-path))/~file")
+    (let* ((ifile (build-path 'cld file))
            (src  (file->string ifile))
            (sink (open-output-string))
            (closed? #f)
@@ -572,36 +603,33 @@
            finished)))
 
 ;;-----------------------------------------------------------
-(test-section "srfi-181 and 192")
+(test-section "SRFI-181 and 192")
 
 (define-module srfi-181-tests
   (use gauche.test)
   (use gauche.uvector)
   (use scheme.list)
-  (use srfi-13)
-  (use srfi-64)
-  (use srfi-181)
-  (use srfi-192)
+  (use srfi.13)
+  (use srfi.64)
+  (use srfi.181)
+  (use srfi.192)
   (use scheme.base :only (bytevector
                           bytevector-copy
                           bytevector-u8-ref
                           flush-output-port))
 
-  (test-module 'srfi-181)
-  (test-module 'srfi-192)
-
-  (define-syntax import (syntax-rules () ((_ . _) #f)))
-
-  (include "../../test/include/srfi-181-192-tests.scm")
+  (test-module 'srfi.181)
+  (test-module 'srfi.192)
+  (test-include-r7 "../../test/include/srfi-181-192-tests.scm")
   )
 
-;; Auxiliary srfi-181 tests
+;; Auxiliary SRFI-181 tests
 
 ;; i/o-invalid-position-error
 (define-module srfi-181-tests-2
   (use gauche.test)
-  (use srfi-181)
-  (use srfi-192)
+  (use srfi.181)
+  (use srfi.192)
   (define p (make-custom-binary-input-port
              'invalid-position-error-test
              (^ _ 1)
@@ -622,7 +650,7 @@
   (use gauche.vport)
   (use gauche.charconv) ; chaconv is tested before vport
   (use gauche.uvector)  ; uvector is tested before vport
-  (use srfi-181)
+  (use srfi.181)
 
   (define (import-test codec external-data internal-string handling)
     (let1 native-codec (~ (native-transcoder)'codec)
@@ -665,38 +693,62 @@
     (import-test codec external-data internal-string handling)
     (export-test codec internal-string external-data handling))
 
-  (cond-expand
-   [gauche.ces.utf8
-    (roundtrip-test (make-codec 'latin1)
-                    '#u8(#x41 #xc2 #x42)
-                    "A\u00c2B" 'replace)
-    (roundtrip-test (make-codec 'utf-16be)
-                    '#u8(#x00 #x41 #x00 #xc2 #x00 #x42)
-                    "A\u00c2B" 'replace)
-    (roundtrip-test (make-codec 'utf-16le)
-                    '#u8(#x41 #x00 #xc2 #x00 #x42 #x00)
-                    "A\u00c2B" 'replace)
-    (import-test (make-codec 'utf-16)
-                 '#u8(#xfe #xff #x00 #x41 #x00 #xc2 #x00 #x42)
-                 "A\u00c2B" 'replace)
-    (import-test (make-codec 'utf-16)
-                 '#u8(#xff #xfe #x41 #x00 #xc2 #x00 #x42 #x00)
-                 "A\u00c2B" 'replace)
-    (export-test (make-codec 'utf-16)
-                 "A\u00c2B"
-                 (test-one-of
-                  '#u8(#xff #xfe #x41 #x00 #xc2 #x00 #x42 #x00)
-                  '#u8(#xfe #xff #x00 #x41 #x00 #xc2 #x00 #x42))
-                 'replace)
+  (roundtrip-test (make-codec 'latin1)
+                  '#u8(#x41 #xc2 #x42)
+                  "A\u00c2B" 'replace)
+  (roundtrip-test (make-codec 'utf-16be)
+                  '#u8(#x00 #x41 #x00 #xc2 #x00 #x42)
+                  "A\u00c2B" 'replace)
+  (roundtrip-test (make-codec 'utf-16le)
+                  '#u8(#x41 #x00 #xc2 #x00 #x42 #x00)
+                  "A\u00c2B" 'replace)
+  (import-test (make-codec 'utf-16)
+               '#u8(#xfe #xff #x00 #x41 #x00 #xc2 #x00 #x42)
+               "A\u00c2B" 'replace)
+  (import-test (make-codec 'utf-16)
+               '#u8(#xff #xfe #x41 #x00 #xc2 #x00 #x42 #x00)
+               "A\u00c2B" 'replace)
+  (export-test (make-codec 'utf-16)
+               "A\u00c2B"
+               (test-one-of
+                '#u8(#xff #xfe #x41 #x00 #xc2 #x00 #x42 #x00)
+                '#u8(#xfe #xff #x00 #x41 #x00 #xc2 #x00 #x42))
+               'replace)
 
-    (import-test (make-codec 'utf-16be)
-                 '#u8(#xdc #x00)
-                 (test-error <io-decoding-error>) 'raise)
-    (export-test (make-codec 'ascii)
-                 "A\u3000"
-                 (test-error <io-encoding-error>) 'raise)
-    ]
-   [else])
+  (import-test (make-codec 'utf-16be)
+               '#u8(#xdc #x00)
+               (test-error <io-decoding-error>) 'raise)
+  (export-test (make-codec 'ascii)
+               "A\u3000"
+               (test-error <io-encoding-error>) 'raise)
   )
 
+;;-----------------------------------------------------------
+(test-section "Base64 with uvector")
+
+(use rfc.base64)
+
+(test* "base64 encode from uvector"
+       '("" "AA==" "//79/Pv6")
+       (list (base64-encode-bytevector '#u8())
+             (base64-encode-bytevector '#u8(0))
+             (base64-encode-bytevector '#u8(255 254 253 252 251 250))))
+
+(test* "base64 decode to uvector"
+       '(#u8() #u8(0) #u8(255 254 253 252 251 250))
+       (list (base64-decode-bytevector "")
+             (base64-decode-bytevector "AA==")
+             (base64-decode-bytevector "//79/Pv6")))
+
+(test* "base64 encode from uvector (digits)"
+       "__79+_z6"
+       (base64-encode-bytevector '#u8(255 254 253 251 252 250)
+                                 :digits "+_"))
+
+(test* "base64 decode from uvector (digits)"
+       '#u8(255 254 253 251 252 250)
+       (base64-decode-bytevector "__79+_z6"
+                                 :digits #(#\+ #\_)))
+
+
 (test-end)
