@@ -1,7 +1,7 @@
 /*
  * gauche/vector.h - Vector API
  *
- *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -40,11 +40,11 @@
 
 struct ScmVectorRec {
     SCM_HEADER;
-#if GAUCHE_API_VERSION >= 1000
+#if GAUCHE_API_VERSION >= 98
     ScmWord size_flags;
-#else  /* GAUCHE_API_VERSION < 1000 */
+#else  /* GAUCHE_API_VERSION < 98 */
     ScmWord size;
-#endif /* GAUCHE_API_VERSION < 1000 */
+#endif /* GAUCHE_API_VERSION < 98 */
     ScmObj elements[1];
 };
 
@@ -57,7 +57,7 @@ SCM_CLASS_DECL(Scm_VectorClass);
 
 /* SCM_VECTOR_SIZE_SLOT_INITIALIZER is used in cgen-generated code  */
 
-#if GAUCHE_API_VERSION >= 1000
+#if GAUCHE_API_VERSION >= 98
 #define SCM_VECTOR_SIZE(obj)        (SCM_VECTOR(obj)->size_flags >> 1)
 #define SCM_VECTOR_IMMUTABLE_P(obj) (SCM_VECTOR(obj)->size_flags & 1)
 #define SCM_VECTOR_IMMUTABLE_SET(obj, flag)     \
@@ -65,19 +65,19 @@ SCM_CLASS_DECL(Scm_VectorClass);
      ? (SCM_UVECTOR(obj)->size_flags |= 1)      \
      : (SCM_UVECTOR(obj)->size_flags &= ~1))
 #define SCM_VECTOR_SIZE_SLOT_INITIALIZER(len, imm) \
-    SCM_OBJ(((len)<<1)|(imm?1:0))
-#else  /* GAUCHE_API_VERSION < 1000 */
+    SCM_OBJ(SCM_WORD(((len)<<1)|((imm)?1:0)))
+#else  /* GAUCHE_API_VERSION < 98 */
 #define SCM_VECTOR_SIZE(obj)        (SCM_VECTOR(obj)->size)
 #define SCM_VECTOR_IMMUTABLE_P(obj) (!SCM_VECTORP(obj)) /* always FALSE, but need to use obj to avoid unused variable warning */
 #define SCM_VECTOR_IMMUTABLE_SET(obj, flag)  /*empty*/
 #define SCM_VECTOR_SIZE_SLOT_INITIALIZER(len, imm)  SCM_OBJ(len)
-#endif /* GAUCHE_API_VERSION < 1000 */
+#endif /* GAUCHE_API_VERSION < 98 */
 
 #define SCM_VECTOR_CHECK_MUTABLE(obj)           \
   do { if (SCM_VECTOR_IMMUTABLE_P(obj)) {       \
     Scm_Error("vector is immutable: %S", obj);  \
   }} while (0)
-    
+
 SCM_EXTERN ScmObj Scm_MakeVector(ScmSmallInt size, ScmObj fill);
 SCM_EXTERN ScmObj Scm_VectorRef(ScmVector *vec, ScmSmallInt i, ScmObj fallback);
 SCM_EXTERN ScmObj Scm_VectorSet(ScmVector *vec, ScmSmallInt i, ScmObj obj);
@@ -153,7 +153,8 @@ typedef enum {
     SCM_UVECTOR_C64,
     SCM_UVECTOR_C128,
     SCM_UVECTOR_RESERVED2,      /* reserved for c256 */
-    SCM_UVECTOR_INVALID = -1
+    SCM_UVECTOR_GENERIC = -1,   /* can appear in inline asm of uvector-ref */
+    SCM_UVECTOR_INVALID = -2
 } ScmUVectorType;
 
 #define SCM_UVECTOR_SUBTYPE_P(obj, type) \
@@ -362,7 +363,8 @@ SCM_EXTERN ScmObj Scm_MakeC128VectorFromArrayShared(ScmSmallInt size,
                                                    ScmDoubleComplex array[]);
 
 
-/* For the backward compatibility */
+/* For the backward compatibility.  Use ScmUVector for the new code
+   (that's why we don't define ScmC**Vector, for they are new additions. */
 typedef ScmUVector ScmS8Vector;
 typedef ScmUVector ScmU8Vector;
 typedef ScmUVector ScmS16Vector;
@@ -375,7 +377,14 @@ typedef ScmUVector ScmF16Vector;
 typedef ScmUVector ScmF32Vector;
 typedef ScmUVector ScmF64Vector;
 
-/* 
+/*
+ * View memory as an uvector
+ */
+ScmObj Scm_MakeViewUVector(ScmMemoryRegion *mem, ScmClass *klass,
+                           ScmSmallInt len, ScmSmallInt offset,
+                           int immutable);
+
+/*
  * String/uvector common utility
  */
 
@@ -418,7 +427,7 @@ SCM_EXTERN int    Scm_Bit2Int(ScmObj bit);
 SCM_EXTERN ScmObj Scm_Bit2Bool(ScmObj bit);
 SCM_EXTERN ScmObj Scm_MakeBitvector(ScmSmallInt size, ScmObj init);
 SCM_EXTERN ScmObj Scm_ListToBitvector(ScmObj lis);
-SCM_EXTERN ScmObj Scm_BitvectorCopy(ScmBitvector *v, 
+SCM_EXTERN ScmObj Scm_BitvectorCopy(ScmBitvector *v,
                                     ScmSmallInt start,
                                     ScmSmallInt end);
 SCM_EXTERN ScmObj Scm_BitvectorCopyX(ScmBitvector *dest, ScmSmallInt dstart,
