@@ -1,7 +1,7 @@
 ;;;
 ;;; file/util.scm - filesystem utility functions
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -39,7 +39,7 @@
 #!no-fold-case
 
 (define-module file.util
-  (use srfi-13)
+  (use srfi.13)
   (use util.match)
   (export current-directory directory-list directory-list2 directory-fold
           home-directory temporary-directory
@@ -229,15 +229,24 @@
 (define create-directory* make-directory*)
 
 ;; rm -rf
-(define (remove-directory* dir)
+(define (remove-directory* dir :key (if-does-not-exist :error))
   (define (rec d)
     (receive (dirs files)
         (directory-list2 d :add-path? #t :children? #t :follow-link? #f)
       (for-each rec dirs)
       (for-each sys-rmdir dirs)
       (for-each sys-unlink files)))
-  (rec dir)
-  (sys-rmdir dir))
+  (case if-does-not-exist
+    [(:error #f)]
+    [else (error ":if-does-not-exist must be either :error or #f, bot got:"
+                 if-does-not-exist)])
+  (if (file-exists? dir)
+    (if (file-is-directory? dir)
+      (begin (rec dir) (sys-rmdir dir) #t)
+      (error "Not a directory:" dir))
+    (if if-does-not-exist
+      (error "No such directory:" dir)
+      #f)))
 
 (define delete-directory* remove-directory*)
 
@@ -399,6 +408,18 @@
 
 (define (build-path base-path . components)
   (define path-separator-string (string (path-separator)))
+  (define real-base-path
+    (case base-path
+      [(cwd) (current-directory)]
+      [(cld) (if-let1 clp (current-load-path)
+               (sys-dirname clp)
+               (current-directory))]
+      [(#f) "."]
+      [else (unless (string? base-path)
+              (error "base-path must be a string, #f, \
+                      or a symbol 'cwd or 'cld, \
+                      but got:" base-path))
+            base-path]))
   (define (rec base components)
     (if (null? components)
       base
@@ -406,6 +427,7 @@
           (match (car components)
             ['up   ".."]
             ['same "."]
+            ['#f "."]
             [(? absolute-path? p)
              (error "can't append absolute path after other path" p)]
             [p (sys-normalize-pathname p)])
@@ -415,7 +437,7 @@
                    [else
                     (string-append base path-separator-string component)])
           (cdr components)))))
-  (rec (sys-normalize-pathname base-path) components))
+  (rec (sys-normalize-pathname real-base-path) components))
 
 (define (expand-path path)
   (sys-normalize-pathname path :expand #t))
