@@ -1,7 +1,7 @@
 ;;;
 ;;; process.scm - process interface
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -40,7 +40,7 @@
   (use gauche.generator)
   (use gauche.connection)
   (use scheme.charset)
-  (use srfi-13)
+  (use srfi.13)
   (export <process> <process-abnormal-exit>
           run-process do-process do-process!
           process? process-alive? process-pid
@@ -71,14 +71,13 @@
           ))
 (select-module gauche.process)
 
-;; Delay-load gauche.charconv
-(autoload gauche.charconv
-          wrap-with-input-conversion wrap-with-output-conversion)
-(autoload gauche.uvector
-          write-uvector)
+;; Delay-load to avoid circular dependency
+(autoload gauche.charconv wrap-with-input-conversion wrap-with-output-conversion)
+(autoload gauche.uvector write-uvector)
+(autoload gauche.threads make-thread thread-start!)
 
 ;; Avoid build dependency issue
-(autoload srfi-19
+(autoload srfi.19
           make-time add-duration time>?)
 
 (define-class <process> ()
@@ -144,16 +143,19 @@
 
 ;; A typical use case---run process synchronously, returns #t for
 ;; success, #f for failure.
-(define (do-process command . args)
-  (let* ([eflag (case (get-keyword :on-abnormal-exit args #f)
-                  [(#f) #f]
-                  [(:error) #t]
-                  [else => (cut error
-                                "Value for on-abnormal-exit argument \
-                                 must be either #f or :error, but got:" <>)])]
-         [p (apply run-process command (delete-keyword :on-abnormal-exit args))])
-    (process-wait p #f eflag)
-    (zero? (process-exit-status p))))
+(define (do-process command :key (on-abnormal-exit #f) :allow-other-keys args)
+  (let* ([raise-flag (ecase on-abnormal-exit
+                       [(#f :exit-code) #f]
+                       [(:error) #t])]
+         [p (apply run-process command args)])
+    (process-wait p #f raise-flag)
+    (let1 status (process-exit-status p)
+      (cond [(zero? status) #t]
+            [(eq? on-abnormal-exit :exit-code)
+             (if (sys-wait-exited? status)
+               (sys-wait-exit-status status)
+               (%check-normal-exit p))] ;this raises <process-abnormal-exit>
+            [else #f]))))
 
 ;; Similar to do-process, but raise an error on abnormal exit.
 ;; This use case is typical, so we add a separate API.
@@ -426,33 +428,16 @@
   (values iomap toclose ipipes opipes tmpfiles))
 
 ;; Set up source of << and <<< redirection, used inside %setup-iomap.
-;; We don't want to depend on gauche.threads at compile time, so we split
-;; this part to be evaluated at runtime.
-;; Returns updated iomap, toclose, and tmpfiles.
-(without-precompiling
- (define (%setup-<< fd do-file dir arg iomap toclose tmpfiles)
-   (define (write-arg o)
-     (unwind-protect
-         (cond [(eq? dir '<<<) (write arg o)]
-               [(string? arg) (display arg o)]
-               [else (write-uvector arg o)])
-       (close-output-port o)))
-   (cond-expand
-    [gauche.sys.threads
-     (receive (in out) (sys-pipe)
-       (thread-start! (make-thread (cut write-arg out)))
-       (values (acons fd in iomap) (cons in toclose) tmpfiles))]
-    [else
-     ;; If we can't use threads, we fall back to using temporary
-     ;; files.  It has some drawbacks: (1) If we 'exec'-ing
-     ;; (not forking), we don't have chance to remove them, and
-     ;; (2) On Windows we won't be able to remove temp file unless
-     ;; we wait the child process to complete.  We'll do our best
-     ;; anyway.
-     (receive (out nam) (sys-mkstemp (%temp-path-prefix))
-       (write-arg out)
-       (do-file '< fd nam)
-       (values iomap toclose (cons nam tmpfiles)))])))
+(define (%setup-<< fd do-file dir arg iomap toclose tmpfiles)
+  (define (write-arg o)
+    (unwind-protect
+        (cond [(eq? dir '<<<) (write arg o)]
+              [(string? arg) (display arg o)]
+              [else (write-uvector arg o)])
+      (close-output-port o)))
+  (receive (in out) (sys-pipe)
+    (thread-start! (make-thread (cut write-arg out)))
+    (values (acons fd in iomap) (cons in toclose) tmpfiles)))
 
 (define (%ensure-mask mask)
   (cond
@@ -717,9 +702,11 @@
                                              [gauche.os.windows 'windows]
                                              [else 'posix])))
   (ecase flavor
+   ;; Windows flavors is supported in src/libsys.scm.  See the comment in it.
    [(windows)
-    ;; This is supported in src/libsys.scm.  See the comment in it.
-    (%sys-escape-windows-command-line str)]
+    (%sys-escape-windows-command-line str #f)]
+   [(windows-batchfile)
+    (%sys-escape-windows-command-line str #t)]
    [(posix)
     ;; We follow standard unix shell convention: if STR contains special
     ;; chars, we quote the entire STR by single-quotes.  If STR contains
