@@ -1,7 +1,7 @@
 ;;;
 ;;; tr.scm - transliterate characters
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,7 +35,7 @@
 
 (define-module text.tr
   (use scheme.list)
-  (use srfi-13)
+  (use srfi.13)
   (use gauche.generator)
   (export tr transliterate string-tr string-transliterate
           build-transliterator)
