@@ -1,7 +1,7 @@
 ;;;
 ;;; Array operations.  This is superset of SRFI-25.
 ;;;
-;;;  Copyright (c) 2002-2020  Shiro Kawai  <shiro@acm.org>
+;;;  Copyright (c) 2002-2024  Shiro Kawai  <shiro@acm.org>
 ;;;  Copyright(C) 2004      by Alex Shinn  (foof@synthcode.com)
 ;;;
 ;;;  Permission to use, copy, modify, distribute this software and
@@ -18,8 +18,7 @@
 ;; map n-dimensional indices to an index of the backing storage.
 
 (define-module gauche.array
-  (use srfi-1)
-  (use srfi-11)
+  (use scheme.list)
   (use gauche.uvector)
   (use gauche.collection)
   (use gauche.sequence)
@@ -43,14 +42,14 @@
           u64array s64array f16array f32array f64array
           array-concatenate array-transpose array-rotate-90
           array-flip array-flip!
-          identity-array array-inverse determinant determinant! 
-          array-mul array-expt
-          array-div-left array-div-right 
+          identity-array array-inverse determinant determinant!
+          array-mul array-vector-mul vector-array-mul array-expt
+          array-div-left array-div-right
           array-add-elements array-add-elements!
           array-sub-elements array-sub-elements!
           array-negate-elements array-negate-elements!
           array-mul-elements array-mul-elements!
-          array-div-elements array-div-elements! 
+          array-div-elements array-div-elements!
           array-reciprocate-elements array-reciprocate-elements!
           pretty-print-array
           ))
@@ -58,10 +57,11 @@
 
 (autoload "gauche/matrix"
   array-concatenate array-transpose array-rotate-90 array-flip array-flip!
-  identity-array array-inverse determinant determinant! array-mul array-expt
-  array-div-left array-div-right 
+  identity-array array-inverse determinant determinant!
+  array-mul array-vector-mul vector-array-mul array-expt
+  array-div-left array-div-right
   array-add-elements array-add-elements!
-  array-sub-elements array-sub-elements! 
+  array-sub-elements array-sub-elements!
   array-negate-elements array-negate-elements!
   array-mul-elements array-mul-elements!
   array-div-elements array-div-elements!
@@ -85,15 +85,10 @@
   (next-method)
   (let ((name (class-name class)))
     (when name
-      (define-method write-object ((self class) port)
-        (format port "#,(~A ~S" name (array->list (array-shape self)))
-        (array-for-each (cut format port " ~S" <>) self)
-        (format port ")"))
       (define-reader-ctor name
         (^[sh . inits]
           (list-fill-array! (make-array-internal class (apply shape sh)) inits))))))
 
-
 (define-class <array-base> ()
   ((start-vector    :init-keyword :start-vector :getter start-vector-of)
    (end-vector      :init-keyword :end-vector   :getter end-vector-of)
@@ -112,6 +107,12 @@
     (set! (slot-ref self 'getter) (^[index] (get store index)))
     (set! (slot-ref self 'setter) (^[index value] (set store index value)))))
 
+(define-method write-object ((self <array-base>) port)
+  (format port "#,(~A ~S" (class-name (class-of self))
+          (array->list (array-shape self)))
+  (array-for-each (cut format port " ~S" <>) self)
+  (format port ")"))
+
 (define-class <array> (<array-base>)
   ()
   :backing-storage-creator make-vector
@@ -703,4 +704,3 @@
       (^[ind] (add! (array-ref ar ind)))
       (make-vector (array-rank ar)))
     (get)))
-
