@@ -1,7 +1,7 @@
 /*
  * bignum.c - multiple precision exact integer arithmetic
  *
- *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -80,6 +80,7 @@ char *alloca ();
 #include <limits.h>
 #define LIBGAUCHE_BODY
 #include "gauche.h"
+#include "gauche/priv/configP.h"
 #include "gauche/priv/arith.h"
 #include "gauche/bits.h"
 #include "gauche/bits_inline.h"
@@ -814,13 +815,19 @@ static ScmBignum *bignum_mul_si(const ScmBignum *bx, long y)
         br->sign = -br->sign;
         return br;
     }
-    /* TODO: optimize for 2^n case !*/
     ScmBignum *br;
     br = make_bignum(bx->size + 1); /* TODO: more accurate estimation */
-    u_long yabs = (y<0)? -y:y;
-    br->sign = bx->sign;
-    bignum_mul_word(br, bx, yabs, 0);
-    if (y<0) br->sign = -br->sign;
+    if (y & (y-1)) {
+        /* general case */
+        u_long yabs = (y<0)? -y:y;
+        br->sign = bx->sign;
+        bignum_mul_word(br, bx, yabs, 0);
+        if (y<0) br->sign = -br->sign;
+    } else {
+        /* fast path when y is a power of two (note that y != 0). */
+        int n = Scm__LowestBitNumber((u_long)y);
+        bignum_lshift(br, bx, n);
+    }
     return br;
 }
 
