@@ -1,7 +1,7 @@
 ;;;
 ;;; compile-1.scm - The compiler: Pass 1
 ;;;
-;;;   Copyright (c) 2004-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -193,6 +193,43 @@
                  proc inliner)]
       )))
 
+;; This procedure is set in the inliner slot of procedure created by
+;; make-case-lambda.  If the call site knows the number of arguments,
+;; we can skip the dispatching.  Returns IForm if inlined, or #<undef>
+;; if not.
+;; NB: The expansion after procedure selection is almost dupe of
+;; pass1/expand-inliner, but we can't consolidate them because the order
+;; of applying pass1 differ: pass1/expand-inliner runs pass1 on its arguments
+;; by itself, while pass1/make-case-lambda-inliner already gets pass1-ed
+;; arguments.  Call for refactoring.
+(define (pass1/make-case-lambda-inliner clambda-proc)
+  (and-let1 info (%case-lambda-info clambda-proc)
+    (let* ([min-args (get-keyword :min-reqargs info)]
+           [max-index (get-keyword :max-optargs info)]
+           [dispatch (get-keyword :dispatch-vector info)])
+      (^[src arg-iforms]
+        (let1 nargs (length arg-iforms)
+          (or (and-let* ([ (<= min-args nargs) ]
+                         [index (- nargs min-args)]
+                         [index (if (< index max-index) index (- max-index 1))]
+                         [proc (vector-ref dispatch index)]
+                         [inliner (%procedure-inliner proc)])
+                (cond
+                 [(integer? inliner)
+                  (let ([opt?  (slot-ref proc 'optional)])
+                    (if (and (argcount-ok? args (slot-ref proc 'required) opt?)
+                             (<= nargs MAX_LITERAL_ARG_COUNT))
+                      ($asm src (if opt? `(,inliner ,nargs) `(,inliner))
+                            arg-iforms)))]
+                 [(vector? inliner)
+                  (expand-inlined-procedure src (unpack-iform inliner)
+                                            arg-iforms)]
+                 [(procedure? inliner)
+                  (inliner src arg-iforms)]
+                 ;; NB: No compiler macros can appear here.
+                 [else #f]))
+              (undefined)))))))
+
 ;; Returns #t iff exp is the form (with-module module VARIABLE)
 ;; We need to check the global value of with-module, for it might
 ;; be redefined.  We assume this function is called infrequently,
@@ -250,6 +287,10 @@
 ;; expressions, we re-evaluate <init-expr> and replace the frame entry with
 ;;    (<name> . <lvar>)
 (define (pass1/body-rec exprs mframe vframe cenv)
+  (define (dupe-check var mframe vframe)
+    (when (or (and (pair? mframe) (assq var mframe))
+              (and (pair? vframe) (assq var vframe)))
+      (error "Duplicate internal definitions of" var)))
   (match exprs
     [(((op . args) . src) . rest)
      (or (and-let* ([ (or (not vframe) (not (assq op vframe))) ]
@@ -279,6 +320,7 @@
                             (error "define without expression is not allowed in R7RS" (caar exprs))
                             `(,var :rec ,(undefined) . ,src))]
                          [_ (error "malformed internal define:" (caar exprs))])
+               (dupe-check (car def) mframe vframe)
                (if (not mframe)
                  (let* ([cenv (cenv-extend cenv '() SYNTAX)]
                         [mframe (car (cenv-frames cenv))]
@@ -291,6 +333,7 @@
             [(global-identifier=? head define-syntax.) ; internal syntax definition
              (match args
                [(name trans-spec)
+                (dupe-check name mframe vframe)
                 (if (not mframe)
                  (let* ([cenv (cenv-extend cenv `((,name)) SYNTAX)]
                         [mframe (car (cenv-frames cenv))]
@@ -378,7 +421,7 @@
 ;; Pass1 utilities
 ;;
 
-;; get symbol or id, and returns identiier.
+;; get symbol or id, and returns identifier.
 (define (ensure-identifier sym-or-id cenv)
   (if (symbol? sym-or-id)
     (make-identifier sym-or-id (cenv-module cenv) (cenv-frames cenv))
@@ -445,38 +488,48 @@
        (%insert-syntax-binding m ',(car formals)
                                (make-syntax ',(car formals) m ,name)))))
 
+;; Pre-resolved global identifiers.
+;; With er-macro-transformer, this is done implicitly within rename procedure.
+;; But for the forms expanded by the compiler, we know exactly which identifiers
+;; should be reffered, so we directly refer to them.
 (define (global-id id) (make-identifier id (find-module 'gauche) '()))
 (define (global-id% id) (make-identifier id (find-module 'gauche.internal) '()))
 
-(define define.         (global-id 'define))
-(define define-inline.  (global-id 'define-inline))
-(define define-syntax.  (global-id 'define-syntax))
-(define lambda.         (global-id 'lambda))
-(define r5rs-define.    (make-identifier 'define (find-module 'null) '()))
-(define r5rs-lambda.    (make-identifier 'lambda (find-module 'null) '()))
-(define setter.         (global-id 'setter))
-(define lazy.           (global-id 'lazy))
-(define eager.          (global-id 'eager))
-(define values.         (global-id 'values))
-(define begin.          (global-id 'begin))
-(define let.            (global-id 'let))
-(define include.        (global-id 'include))
-(define include-ci.     (global-id 'include-ci))
-(define else.           (global-id 'else))
-(define =>.             (global-id '=>))
-(define current-module. (global-id 'current-module))
-(define with-module.    (global-id 'with-module))
-(define quasiquote.     (global-id 'quasiquote))
-(define unquote.        (global-id 'unquote))
-(define unquote-splicing. (global-id 'unquote-splicing))
-(define let-optionals*. (global-id 'let-optionals*))
-(define let-keywords*.  (global-id 'let-keywords*))
-
-(define make-case-lambda. (global-id% 'make-case-lambda))
+(define %expression-name-mark-key.     (global-id% '%expression-name-mark-key))
 (define %make-er-transformer.          (global-id% '%make-er-transformer))
 (define %make-er-transformer/toplevel. (global-id% '%make-er-transformer/toplevel))
 (define %with-inline-transformer.      (global-id% '%with-inline-transformer))
 
+(define =>.               (global-id '=>))
+(define apply.            (global-id 'apply))
+(define begin.            (global-id 'begin))
+(define current-module.   (global-id 'current-module))
+(define define-inline.    (global-id 'define-inline))
+(define define-syntax.    (global-id 'define-syntax))
+(define define.           (global-id 'define))
+(define eager.            (global-id 'eager))
+(define else.             (global-id 'else))
+(define error.            (global-id 'error))
+(define include-ci.       (global-id 'include-ci))
+(define include.          (global-id 'include))
+(define lambda.           (global-id 'lambda))
+(define lazy.             (global-id 'lazy))
+(define let-keywords*.    (global-id 'let-keywords*))
+(define let-optionals*.   (global-id 'let-optionals*))
+(define let.              (global-id 'let))
+(define make-case-lambda. (global-id% 'make-case-lambda))
+(define null?.            (global-id 'null?))
+(define quasiquote.       (global-id 'quasiquote))
+(define r5rs-define.      (make-identifier 'define (find-module 'null) '()))
+(define r5rs-lambda.      (make-identifier 'lambda (find-module 'null) '()))
+(define receive.          (global-id 'receive))
+(define setter.           (global-id 'setter))
+(define unless.           (global-id 'unless))
+(define unquote-splicing. (global-id 'unquote-splicing))
+(define unquote.          (global-id 'unquote))
+(define values.           (global-id 'values))
+(define with-module.      (global-id 'with-module))
+
 ;; Returns an IForm for (values) - useful for define-pass1-syntax that does
 ;; compile-time things and returns nothing.  The delay trick is to create
 ;; iform only once.  The module to call pass1 doesn't matter, for we directly
@@ -556,13 +609,26 @@
          (unless (vm-compiler-flag-is-set? SCM_COMPILE_LEGACY_DEFINE)
            (%insert-binding module (unwrap-syntax name)
                             (%uninitialized) '(fresh)))
-         ($define oform flags id (pass1 expr cenv))))]
+         ($define oform flags id
+                  (%wrap-as-named-expression oform (pass1 expr cenv) id))))]
     [_ (error "syntax-error:" oform)]))
 
 (define (%rename-toplevel-identifier! identifier)
   (slot-set! identifier 'name (gensym #"~(identifier->symbol identifier)."))
   identifier)
 
+;; Wrap IFORM with $dynenv to carry ID as the name of the expression.
+;; Effectively it is the same as
+;;   (with-continuation-mark '#:expression-name 'id <body>)
+;; During evaluation of IFORM, the name can be retrieved from the
+;; continuation mark.  Note that the name is meaningful
+;; only when an expression is in the tail context, so
+;; call-with-immediate-continuation-mark should be used.
+(define (%wrap-as-named-expression src iform id)
+  ($dynenv src (list ($call src ($gref %expression-name-mark-key.) '())
+                     ($const (unwrap-syntax id)))
+           iform))
+
 ;; Inlinable procedure.
 ;;   Inlinable procedure has both properties of a macro and a procedure.
 ;;   It is a bit tricky since the inliner information has to exist
@@ -607,13 +673,13 @@
 
 (define (pass1/define-inline form name expr cenv)
   (let1 iform (pass1 expr (cenv-add-name cenv (variable-name name)))
-    (receive (closure closed) (pass1/check-inlinable-lambda iform)
+    (receive (closures closed) (pass1/check-inlinable-lambda iform)
       (cond
-       [(and (not closure) (not closed)) ; too complex to inline
+       [(and (null? closures) (null? closed)) ; too complex to inline
         (pass1/make-inlinable-binding form name iform cenv)]
        [(null? closed)               ; no closed env
-        (pass1/mark-closure-inlinable! closure name cenv)
-        (pass1/make-inlinable-binding form name closure cenv)]
+        (ifor-each (cut pass1/mark-closure-inlinable! <> name cenv) closures)
+        (pass1/make-inlinable-binding form name iform cenv)]
        [else ; inlinable lambda has closed env.
         ;; See the comment in subst-lvars above on the transformation.
         ;; closed :: [(lvar . init-iform)]
@@ -622,35 +688,48 @@
         (receive (gvars subs)
             (pass1/define-inline-classify-env name closed cenv)
           (let1 defs (pass1/define-inline-gen-closed-env gvars cenv)
-            ($lambda-body-set! closure
-                               (subst-lvars ($lambda-body closure) subs))
-            (pass1/mark-closure-inlinable! closure name cenv)
+            (ifor-each
+             (^[closure]
+               ($lambda-body-set! closure
+                                  (subst-lvars ($lambda-body closure) subs))
+               (pass1/mark-closure-inlinable! closure name cenv))
+             closures)
             ($seq `(,@defs
-                     ,(pass1/make-inlinable-binding form name closure cenv)))))]
+                     ,(pass1/make-inlinable-binding form name iform cenv)))))]
        ))))
 
-;; If IFORM generates a closure with local environment, returns
-;; the closure itself ($lambda node) and the environment
-;; ((lvar . init) ...).
+;; See if IFORM eventaully generate a closure (or closures).  It is the
+;; cases when the final expression in IFORM becomes $LAMBDA node or $CLAMBDA
+;; node.
+;; The closure(s) may close local environment, if the form is something like
+;; ($let ... ($lambda ...)).
+;; Returns two values:
+;;    A list of closures:   ($lambda-node ...)
+;;    Alist of closed variables: ((var . init) ...)
+
 ;; Typical case is ($let ... ($lambda ...)).  In such case this
 ;; procedure effectively strips $let nodes.
 ;; If IFORM has more complicated structure, we just return (values #f #f)
 ;; to give up inlining.
 (define (pass1/check-inlinable-lambda iform)
-  (cond [(has-tag? iform $LAMBDA) (values iform '())]
+  (cond [(has-tag? iform $LAMBDA) (values `(,iform) '())]
+        [(has-tag? iform $CLAMBDA)
+         ;; NB: At pass1, $clambda-closures are all $LAMBDA node.
+         ;; (Beware that we might have $LREF or $GREF in the later passes.)
+         (values ($clambda-closures iform) '())]
         [(has-tag? iform $LET)
-         (receive (closure closed)
+         (receive (closures closed)
              (pass1/check-inlinable-lambda ($let-body iform))
-           (if (and (not closure) (not closed))
-             (values #f #f) ; giveup
+           (if (and (null? closures) (null? closed))
+             (values '() '()) ; giveup
              (let loop ([lvars (reverse ($let-lvars iform))]
                         [inits (reverse ($let-inits iform))]
                         [closed closed])
                (if (null? lvars)
-                 (values closure closed)
+                 (values closures closed)
                  (loop (cdr lvars) (cdr inits)
                        (acons (car lvars) (car inits) closed))))))]
-        [else (values #f #f)]))
+        [else (values '() '())]))
 
 (define (pass1/define-inline-classify-env name lv&inits cenv)
   (define gvars '())
@@ -683,7 +762,7 @@
          ;; Gauche's compiler optimizes it to refer to the singleton instance.
          [dummy-proc (^ _ name)]
          [packed (pack-iform closure)])
-    ($lambda-flag-set! closure packed)
+    ($lambda-inliner-set! closure packed)
     (when name
       ;; record inliner function for compiler.  this is used only when
       ;; the procedure needs to be inlined in the same compiler unit.
@@ -713,6 +792,7 @@
 
 (define (with-source-info obj src)
   (or (and-let* ([ (pair? obj) ]
+                 [ (pair? src) ]
                  [si (debug-source-info src)])
         (extended-cons (car obj) (cdr obj) `((source-info . ,si))))
       obj))
@@ -766,8 +846,9 @@
      (let* ([cenv (cenv-add-name cenv (variable-name name))]
             [xformer (pass1/eval-macro-rhs 'define-hybrid-syntax
                                            macro-expr cenv)]
-            [body (pass1/call expr ($gref %with-inline-transformer.)
-                              (list expr xformer) cenv)])
+            [body ($call expr ($gref %with-inline-transformer.)
+                         (list (pass1 expr cenv)
+                               ($const xformer)))])
        (pass1/make-inlinable-binding form name body cenv))]
     [_ (error "syntax-error: define-hybrid-syntax")]))
 
@@ -1268,10 +1349,9 @@
                     (error "Invalid formal parameter:" (car xs)))
                   (loop (cdr xs) (cons (car xs) ys))]
                  [else (error "Invalid formal parameter:" formals)]))
-       (pass1/vanilla-lambda (add-arg-info form formals #f)
+       (pass1/vanilla-lambda (add-arg-info form formals)
                              (if rest (append reqs (list rest)) reqs)
-                             (length reqs)
-                             (if rest 1 0)
+                             (length reqs) (if rest 1 0) #f
                              body cenv))]
     [_ (error "syntax-error: malformed lambda:" form)]))
 
@@ -1280,39 +1360,40 @@
     [(_ formals . body)
      (receive (args formals types nreqs nopts kargs)
          (parse-extended-lambda-args formals)
-       (let1 typeinfo (and types
-                           (pass1-for-type-annotations types args cenv))
+       (let1 type (and types
+                       (construct-procedure-type
+                        (pass1-for-type-annotations types args cenv)
+                        (positive? nopts)
+                        '*))
          (if (null? kargs)
-           (pass1/vanilla-lambda (add-arg-info form formals typeinfo)
-                                 args nreqs nopts body cenv)
+           (pass1/vanilla-lambda (add-arg-info form formals)
+                                 args nreqs nopts type body cenv)
            ;; Convert extended lambda into vanilla lambda
            (let1 restarg (gensym "rest")
-             (pass1/vanilla-lambda (add-arg-info form formals typeinfo)
+             (pass1/vanilla-lambda (add-arg-info form formals)
                                    (append args (list restarg))
-                                   nreqs 1
+                                   nreqs 1 type
                                    (pass1/extended-lambda-body form cenv restarg
                                                                kargs body)
                                    cenv)))))]
     [_ (error "syntax-error: malformed lambda:" form)]))
 
-;; Add formals list as 'arg-info, and argument type list to 'arg-types,
-;; in the source form.  They're retrieved by compiled-code-attach-source-info.
-;; NB: For arg-types, We need to retrieve <type> from names.  To be done.
-(define (add-arg-info form formals arg-types)
+;; Add formals list as 'arg-info to the source form.
+;; They're retrieved by compiled-code-attach-source-info.
+(define (add-arg-info form formals)
   (rlet1 xform (if (extended-pair? form)
                  form
                  (extended-cons (car form) (cdr form)))
-    (pair-attribute-set! xform 'arg-info formals)
-    (when arg-types
-      (pair-attribute-set! xform 'arg-types arg-types))))
+    (pair-attribute-set! xform 'arg-info formals)))
 
-(define (pass1/vanilla-lambda form formals nreqs nopts body cenv) ; R7RS lambda
+(define (pass1/vanilla-lambda form formals nreqs nopts type body cenv) ; R7RS lambda
   (let* ([lvars (imap make-lvar+ formals)]
-         [intform ($lambda form (cenv-exp-name cenv) nreqs nopts lvars #f #f)]
+         [lnode ($lambda form (cenv-exp-name cenv) nreqs nopts lvars #f '())]
          [newenv (cenv-extend/proc cenv (%map-cons formals lvars)
-                                   LEXICAL intform)])
-    (vector-set! intform 6 (pass1/body body newenv))
-    intform))
+                                   LEXICAL lnode)])
+    ($lambda-type-set! lnode type)
+    ($lambda-body-set! lnode (pass1/body body newenv))
+    lnode))
 
 (define-pass1-syntax (receive form cenv) :gauche
   (match form
@@ -1332,7 +1413,7 @@
 ;;   <args> is the list of requried args, plus a restarg if any.
 ;;   <formals> is a copy of input sans type annotation.
 ;;   <types> is a list of types for required args.  If there's a restarg,
-;;      the lenght of <types> is one shorter than the length of <args>.
+;;      the length of <types> is one shorter than the length of <args>.
 ;;      Can be #f if there's no type annotation.
 ;;   <nreqs> is the number of required arguments.
 ;;   <nopts> is 0 if no optional arg, 1 if not.
@@ -1422,9 +1503,9 @@
         `((,let-optionals*. ,garg ,(append binds rest)
            ,@(if (and (not r) (null? ks) (not a))
                ;; TODO: better error message!
-               `((unless (null? ,rest)
-                   (error "too many arguments for" ',form))
-                 (let () ,@(expand-key ks rest a)))
+               `((,unless. (,null?. ,rest)
+                   (,error. "too many arguments for" ',form))
+                 (,let. () ,@(expand-key ks rest a)))
                (expand-key ks rest a)))))))
   (define (expand-key ks garg a)
     (if (null? ks)
@@ -1449,7 +1530,7 @@
 
   (parse-kargs cenv kargs '() '() #f #f))
 
-;; case-lambda (srfi-16)
+;; case-lambda (SRFI-16)
 ;;   we recognize it here so that we can do aggressive inlining.
 (define-pass1-syntax (case-lambda form cenv) :gauche
   (match form
@@ -1457,23 +1538,34 @@
      (pass1 `(,lambda. ,formals ,@body) cenv)]
     [(_) (error "syntax-error: malformed case-lambda:" form)]
     [(_ (formals . body) ...)
-     (receive (min-req max-req) (find-argcount-minmax formals)
-       (pass1 `(,make-case-lambda.
-                ,min-req ,max-req ',formals
-                (list ,@(map (^(f b) `(,lambda. ,f ,@b)) formals body))
-                ',(cenv-exp-name cenv))
-              cenv))]
+     (let1 closures (map (^[f b] (pass1 `(,lambda. ,f ,@b) cenv)) formals body)
+       ($clambda form (cenv-exp-name cenv) closures
+                 (compute-clambda-argcounts closures)))]
     [_ (error "syntax-error: malformed case-lambda:" form)]))
 
-(define (find-argcount-minmax formals)
-  (define (length. xs k)
-    (if (pair? xs) (length. (cdr xs) (+ k 1)) k))
-  (let loop ([formals formals] [min-req #f] [max-req 0])
-    (if (null? formals)
-      (values min-req max-req)
-      (let1 k (length. (car formals) 0)
-        (loop (cdr formals) (if min-req (min min-req k) k) (max max-req k))))))
+;; This is called when constructing and unpacking CLAMBDA nodes.
+(define (compute-clambda-argcounts lambda-nodes)
+  (map (^l (cons ($lambda-reqargs l) ($lambda-optarg l))) lambda-nodes))
 
+;; Called during optimization.  Returns a $LAMBDA node (or, in later passes,
+;; $LREF or $GREF that points to a closure) that will be used to apply
+;; to the given ARGS.   ARGS is used only to find the number of arguments;
+;; the elements doesn't matter.  Can return #f if there's no applicable body.
+(define (select-clambda-body clambda args)
+  (let1 nargs (length args)
+    (let loop ([closures ($clambda-closures clambda)]
+               [argcounts ($clambda-argcounts clambda)])
+      (match argcounts
+        [() #f]
+        [((nreq . nopt) . argcounts)
+         (if (or (and (= nopt 0) (= nreq nargs))
+                 (and (> nopt 0) (>= nreq nargs)))
+           (car closures)
+           (loop (cdr closures) argcounts))]))))
+
+;;
+;; LET
+;;
 (define-pass1-syntax (let form cenv) :null
   (match form
     [(_ () body ...)
@@ -1509,7 +1601,7 @@
        (let* ([env1 (cenv-extend cenv `((,name . ,lvar)) LEXICAL)]
               [env2 (cenv-extend/name env1 (%map-cons var args) LEXICAL name)]
               [lmda ($lambda form name (length args) 0 args
-                             (pass1/body body env2))])
+                             (pass1/body body env2) '())])
          (lvar-initval-set! lvar lmda)
          ($let form 'rec
                (list lvar)
@@ -1593,7 +1685,7 @@
 (define-pass1-syntax (set! form cenv) :null
   (match form
     [(_ (op . args) expr)
-     ;; srfi-17.  We recurse to pass1 on expanded form, for (setter op) might
+     ;; SRFI-17.  We recurse to pass1 on expanded form, for (setter op) might
      ;; have a chance of optimization.
      (pass1 (with-original-source `((,setter. ,op) ,@args ,expr) form) cenv)]
     [(_ name expr)
@@ -1619,11 +1711,25 @@
                     (list (pass1 `(,lambda. () ,expr) cenv)))]
     [_ (error "syntax-error: malformed lazy:" form)]))
 
+;; (delay expr) == (lazy (receive vals expr (apply eager vals)))
 (define-pass1-syntax (delay form cenv) :null
   (match form
-    [(_ expr) (pass1 `(,lazy. (,eager. ,expr)) cenv)]
+    [(_ expr) (let1 vals (gensym "vals")
+                (pass1 `(,lazy. (,receive. ,vals ,expr
+                                   (,apply. ,eager. ,vals)))
+                       cenv))]
     [_ (error "syntax-error: malformed delay:" form)]))
 
+;; Control flow ..............................................
+
+(define-pass1-syntax (with-continuation-mark form cenv) :gauche
+  (match form
+    [(_ key val expr)
+     ($dynenv form `(,(pass1 key cenv)
+                     ,(pass1 val cenv))
+              (pass1 expr cenv))]
+    [_ (error "syntax-error: malformed with-continuation-mark:" form)]))
+
 ;; Module related ............................................
 
 (define-pass1-syntax (define-module form cenv) :gauche
@@ -1779,7 +1885,7 @@
     (unless (string? filename)
       (error "include requires literal string, but got:" filename))
     (let1 iport (pass1/open-include-file filename (cenv-source-path cenv))
-      (port-case-fold-set! iport case-fold?)
+      (set! (port-case-fold iport) case-fold?)
       (pass1/report-include iport #t)
       (unwind-protect
           ;; This could be written simpler using port->sexp-list, but it would
@@ -1801,14 +1907,14 @@
     ;; find-load-file returns either (<found-path> <rest-of-search-paths>)
     ;; or (<pseudo-path> <rest-of-search-paths> <thunk-to-open-content>)
     ;; see libeval.scm for the details.
-    (if-let1 path&rest (find-load-file path search-paths
-                                       (cons "" *load-suffixes*)
-                                       :allow-archive #t
-                                       :relative-dot-path #t)
-      (if (pair? (cddr path&rest)) ; archive hook is in effect.
-        ((caddr path&rest) (car path&rest))
-        (open-input-file (car path&rest) :encoding #t))
-      (error "include file is not readable: " path))))
+    (match (find-load-file path search-paths
+                           (cons "" *load-suffixes*)
+                           :allow-archive #t
+                           :relative-dot-path #t)
+      [(pseudo-path rest open-content)     ;archive hook is in effect
+       (open-content pseudo-path)]
+      [(found-path rest) (open-input-file found-path :encoding #t)]
+      [_ (error "Can't find the file to include: " path)])))
 
 ;; Report including.
 (define (pass1/report-include iport open?)
@@ -1816,49 +1922,6 @@
     (format (current-error-port) ";;~a including ~s\n"
             (if open? "Begin" "End") (port-name iport))))
 
-;; Class stuff ........................................
-
-;; KLUDGES.  They should be implemented as macros, but the
-;; current compiler doesn't preserve macro definitions.
-;; These syntax handler merely expands the given form to
-;; the call to internal procedures of objlib.scm, which
-;; returns the macro expanded form.
-
-(define-pass1-syntax (define-generic form cenv) :gauche
-  (match form
-    [(_ name . opts)
-     (check-toplevel form cenv)
-     (pass1 (with-module gauche.object (%expand-define-generic name opts)) cenv)]
-    [_ (error "syntax-error: malformed define-generic:" form)]))
-
-(define-pass1-syntax (define-method form cenv) :gauche
-  (define (parse name rest quals)
-    (match rest
-      [((? keyword? q) . rest)
-       (parse name rest (cons q quals))]
-      [(specs . body)
-       (pass1 (with-module gauche.object
-                (%expand-define-method name quals specs body))
-              cenv)]
-      [_ (error "syntax-error: malformed define-method (empty body):" form)]))
-  ;; Should we limit define-method only at the toplevel?  Doing so
-  ;; is consistent with toplevel define and define-syntax.  Allowing
-  ;; define-method in non-toplevel is rather CL-ish and not like Scheme.
-  ;; (check-toplevel form cenv)
-  (match form
-    [(_ name . rest)
-     (parse name rest '())]
-    [_ (error "syntax-error: malformed define-method:" form)]))
-
-(define-pass1-syntax (define-class form cenv) :gauche
-  (match form
-    [(_ name supers slots . options)
-     (check-toplevel form cenv)
-     (pass1 (with-module gauche.object
-              (%expand-define-class name supers slots options))
-            cenv)]
-    [_ (error "syntax-error: malformed define-class:" form)]))
-
 ;; Black magic ........................................
 
 (define-pass1-syntax (eval-when form cenv) :gauche
