@@ -1,7 +1,7 @@
 ;;;
 ;;; compile-2.scm - The compiler: Pass 2
 ;;;
-;;;   Copyright (c) 2004-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -200,9 +200,24 @@
                (and (= (lvar-ref-count lv) 0)
                     (lvar-immutable? lv)
                     (has-tag? (car inits) $LAMBDA)
-                    (eq? ($lambda-flag (car inits)) 'used)))
+                    ($lambda-used? (car inits))))
              ;; This lambda node has already been inlinded, so we can skip.
              (loop (cdr lvars) (cdr inits) new-lvars new-inits)]
+            [(has-tag? (car inits) $CLAMBDA)
+             ;; We lift component closures of case-lamdba into local bindings,
+             ;; for easier closure optimization.  The lifted local bindings
+             ;; are referred by the $CLAMBDA constructure, we force this
+             ;; $LET type to letrec, if it is not.
+             (receive (lifted-lvars lifted-lambdas)
+                 (pass2/lift-clambda-closures (car inits) (car lvars))
+               (when (eq? ($let-type iform) 'let)
+                 ($let-type-set! iform 'rec))
+               (loop (cdr lvars) (cdr inits)
+                     (append (cons (car lvars) lifted-lvars) new-lvars)
+                     (append (cons (pass2/rec (car inits) penv #f)
+                                   (imap (cut pass2/rec <> penv #f)
+                                         lifted-lambdas))
+                             new-inits)))]
             [else
              (loop (cdr lvars) (cdr inits)
                    (cons (car lvars) new-lvars)
@@ -215,6 +230,24 @@
       (for-each pass2/optimize-closure lvars inits)
       (pass2/shrink-let-frame iform lvars obody))))
 
+;; Called from pass2/$LET, when a local variable LVAR is bound to
+;; $CLAMBDA node.  IFORM is the $CLAMBDA node.
+;; In this case, we lift its component closures to local varaibles,
+;; which allows furthre optimization.
+;; IForm is modified to refer to the LREFS instead of $LAMBDAs.
+;; Returns a list of newly introduced lvars and $LAMBDA nodes.
+;; NB: LVAR is only used to create names for generated lvars.
+(define (pass2/lift-clambda-closures iform lvar)
+  (define prefix (symbol-append (lvar-name lvar) '-))
+  (define orig-lambdas ($clambda-closures iform))
+  (let loop ([i 0] [lvars '()] [lambdas orig-lambdas])
+    (if (null? lambdas)
+      (let1 lvars (reverse lvars)
+        ($clambda-closures-set! iform (imap $lref lvars))
+        (values lvars orig-lambdas))
+      (let1 new-lvar (make-lvar (symbol-append prefix i) (car lambdas))
+        (loop (+ i 1) (cons new-lvar lvars) (cdr lambdas))))))
+
 (define (pass2/shrink-let-frame iform lvars obody)
   (pass2/intermediate-lref-removal lvars obody)
   (receive (new-lvars new-inits removed-inits)
@@ -286,6 +319,13 @@
 ;; form that is no longer used---that means the lambda form is inlined
 ;; elsewhere, and its body has been modified to suit the inlined environment,
 ;; so we can no longer compile the $lambda node safely.
+
+;; NB: By eliminating unused init expressions that contains LREF/LSET, the
+;; lvar reference count may change.  However, traversing entire tree could
+;; be costly, so we only do adjustment for some trival cases---pass3 runs
+;; 'reset-lvars' which correctly sets lref count after pass2-3 optimizations.
+;; This is kludge, still, and it may turn out that we should do proper
+;; refcount adjustment here.  We'll see.
 (define (pass2/remove-unused-lvars lvars type)
   (let loop ([lvars lvars] [rl '()] [ri '()] [rr '()])
     (cond [(null? lvars)
@@ -300,6 +340,11 @@
                      (cond [($lref? init)
                             (lvar-ref--! ($lref-lvar init))
                             rr]
+                           [(has-tag? init $CLAMBDA)
+                            (ifor-each (lambda (c)
+                                         (when ($lref? c)
+                                           (lvar-ref--! ($lref-lvar c))))
+                                       ($clambda-closures init))]
                            [(transparent? init) rr]
                            [else (cons init rr)]))))]
           [else
@@ -401,7 +446,7 @@
     (let loop ([env env])
       (cond [(null? env) #t]
             [(eq? (car env) lambda-node) #t]
-            [(eq? ($lambda-flag (car env)) 'dissolved)
+            [($lambda-dissolved? (car env))
              (loop (cdr env))] ;; skip dissolved (inlined) lambdas
             [else #f])))
   (let loop ([call&envs call&envs]
@@ -446,7 +491,7 @@
     (lvar-ref--! lvar)
     ($call-flag-set! call 'embed)
     ($call-proc-set! call lambda-node)
-    ($lambda-flag-set! lambda-node 'dissolved)
+    ($lambda-dissolved-set! lambda-node)
     ($lambda-body-set! lambda-node ($label ($lambda-src lambda-node) #f
                                            ($lambda-body lambda-node)))
     (unless (null? rec-calls)
@@ -474,7 +519,7 @@
         ($seq-body-set! call-node (list inlined)))))
 
   (lvar-ref-count-set! lvar 0)
-  ($lambda-flag-set! lambda-node 'dissolved)
+  ($lambda-dissolved-set! lambda-node)
   (let loop ([calls calls])
     (cond [(null? (cdr calls))
            (inline-it (car calls) lambda-node)
@@ -493,7 +538,13 @@
                                       (cons iform penv) #t))
   iform)
 
-(define (pass2/$CLAMBDA iform penv tail?) iform)
+(define (pass2/$CLAMBDA iform penv tail?)
+  (let loop ([cs ($clambda-closures iform)] [r '()])
+    (if (null? cs)
+      (begin
+        ($clambda-closures-set! iform (reverse r))
+        iform)
+      (loop (cdr cs) (cons (pass2/rec (car cs) iform #f) r)))))
 
 (define (pass2/$LABEL iform penv tail?)
   ;; $LABEL's body should already be processed by pass2, so we don't need
@@ -517,7 +568,7 @@
 ;; Call optimization
 ;;   We try to inline the call whenever possible.
 ;;
-;;   1. If proc is $LAMBDA, we turn the whole struct into $LET.
+;;   1. If proc is $LAMBDA or $CLAMBDA, we turn the whole struct into $LET.
 ;;
 ;;        ($call ($lambda .. (LVar ...) Body) Arg ...)
 ;;         => ($let (LVar ...) (Arg ...) Body)
@@ -548,77 +599,72 @@
 ;;        as 'rec'.
 
 (define (pass2/$CALL iform penv tail?)
-  (cond
-   [($call-flag iform) iform] ;; this node has already been visited.
-   [else
-    ;; scan OP first to give an opportunity of variable renaming
-    ($call-proc-set! iform (pass2/rec ($call-proc iform) penv #f))
+  (define (try-inline iform penv tail?) ;iform is $CALL node, proc already pass2'ed.
     (let ([proc ($call-proc iform)]
           [args ($call-args iform)])
       (cond
-       [(vm-compiler-flag-noinline-locals?)
-        ($call-args-set! iform (imap (cut pass2/rec <> penv #f) args))
-        iform]
-       [(has-tag? proc $LAMBDA) ;; ((lambda (...) ...) arg ...)
+       [(has-tag? proc $LAMBDA)  ; ((lambda (...) ...) arg ...)
         (pass2/rec (expand-inlined-procedure ($*-src iform) proc args)
                    penv tail?)]
+       [(has-tag? proc $CLAMBDA) ; ((case-lambda ...) arg ...)
+        (and-let1 body (select-clambda-body proc args)
+          ($call-proc-set! iform body))
+        iform]
        [(and ($lref? proc)
-             (pass2/head-lref proc penv tail?))
-        => (^[result]
+             (lvar-const-value ($lref-lvar proc)))
+        => (^[initval]
              (cond
-              [(vector? result)
-               ;; Directly inlinable case.  NB: this only happens if the $LREF
-               ;; node is the lvar's single reference, so we know the inlined
-               ;; procedure is never called recursively.  Thus we can safely
-               ;; traverse the inlined body without going into infinite loop.
-               ;;
-               ;; We directly embed the iform (result), which is a lambda expr
-               ;; bound to PROC.  The lambda iform may not be scanned yet by
-               ;; pass2/$LET, though.  We mark the node 'used, so that
-               ;; pass2/$LET can skip processing it.
-               ($lambda-flag-set! result 'used)
-               (pass2/rec (expand-inlined-procedure ($*-src iform) result args)
-                          penv tail?)]
-              [else
-               ;; We need more info to decide optimizing this node.  For now,
-               ;; we mark the call node by the returned flag and push it
-               ;; to the $LAMBDA node.
-               (let1 lambda-node (lvar-initval ($lref-lvar proc))
-                 ($call-flag-set! iform result)
-                 ($lambda-calls-set! lambda-node
-                                     (acons iform penv
-                                            ($lambda-calls lambda-node)))
-                 ($call-args-set! iform (imap (cut pass2/rec <> penv #f) args))
-                 iform)]))]
-       [else
-        ($call-args-set! iform (imap (cut pass2/rec <> penv #f) args))
-        iform]))]))
+              [(has-tag? initval $LAMBDA)
+               ;; Calling $LREF bound constantly to $LAMBDA.
+               ;; If this is the only reference of $LAMBDA and
+               ;; not self-recursive, we can inline it.
+               (if-let1 call-flag (cond [(pass2/self-recursing? initval penv)
+                                    (if tail? 'tail-rec 'rec)]
+                                   [(= (lvar-ref-count ($lref-lvar proc)) 1)
+                                    #f]
+                                   [else 'local])
+                 (begin
+                   ($call-flag-set! iform call-flag)
+                   ($lambda-calls-set! initval
+                                       (acons iform penv
+                                              ($lambda-calls initval)))
+                   (pass2-args iform args penv))
+                 (begin
+                   (lvar-ref--! ($lref-lvar proc))
+                   ($lambda-used-set! initval) ; let pass2/$LET skip this
+                   ($call-proc-set! iform initval)
+                   (try-inline iform penv tail?)))]
+              [else (pass2-args iform args penv)]))]
+       [else (pass2-args iform args penv)])))
+  (define (pass2-args iform args penv)
+    ($call-args-set! iform (imap (cut pass2/rec <> penv #f) args))
+    iform)
 
-;; Check if IFORM ($LREF node) can be a target of procedure-call optimization.
-;;   - If IFORM is not statically bound to $LAMBDA node,
-;;     returns #f.
-;;   - If the $LAMBDA node that can be directly inlined, returns
-;;     the $LAMBDA node.
-;;   - If the call is self-recursing, returns 'tail-rec or 'rec, depending
-;;     on whether this call is tail call or not.
-;;   - Otherwise, returns 'local.
-
-(define (pass2/head-lref iform penv tail?)
-  (and-let* ([lvar ($lref-lvar iform)]
-             [initval (lvar-const-value lvar)]
-             [ (has-tag? initval $LAMBDA) ])
-    (cond
-     [(pass2/self-recursing? initval penv) (if tail? 'tail-rec 'rec)]
-     [(and (= (lvar-ref-count lvar) 1)
-           (lvar-immutable? lvar))
-      ;; we can inline this lambda directly.
-      (lvar-ref--! lvar)
-      initval]
-     [else 'local])))
+  (if ($call-flag iform)
+    iform ;; this node has already been visited.
+    (begin
+      ;; scan OP first to give an opportunity of variable renaming
+      ($call-proc-set! iform (pass2/rec ($call-proc iform) penv #f))
+      (if (vm-compiler-flag-noinline-locals?)
+        (pass2-args iform ($call-args iform) penv) ; don't try to inline.
+        (try-inline iform penv tail?)))))
 
 (define (pass2/self-recursing? node penv)
   (find (cut eq? node <>) penv))
 
+;;
+;; DYNENV
+;;
+(define (pass2/$DYNENV iform penv tail?)
+  (let ([key (pass2/rec (car ($dynenv-kvs iform)) penv #f)]
+        [val (pass2/rec (cadr ($dynenv-kvs iform)) penv #f)]
+        ;; We always evaluate EXPR as tail expr.  EXTEND-DENV instruction
+        ;; pushes extra continuation.
+        [expr (pass2/rec ($dynenv-body iform) penv #t)])
+    ($dynenv-kvs-set! iform `(,key ,val))
+    ($dynenv-body-set! iform expr)
+    iform))
+
 ;; Recognize the pattern like:
 ;;   ($let ([args ($LIST x y z ...)])
 ;;      ...
