@@ -1,7 +1,7 @@
 ;;;
 ;;; compile-3.scm - The compiler: Pass 3
 ;;;
-;;;   Copyright (c) 2004-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -91,6 +91,14 @@
 (define (pass3/$CONST iform labels) iform)
 (define (pass3/$IT iform labels) iform)
 
+(define (pass3/$DYNENV iform labels)
+  (let ([key (pass3/rec (car ($dynenv-kvs iform)) labels)]
+        [val (pass3/rec (cadr ($dynenv-kvs iform)) labels)]
+        [expr (pass3/rec ($dynenv-body iform) labels)])
+    ($dynenv-kvs-set! iform `(,key ,val))
+    ($dynenv-body-set! iform expr)
+    iform))
+
 ;; If optimization:
 ;;
 ;;  If the 'test' clause of $IF node contains another $IF that has $IT in
@@ -241,7 +249,12 @@
   iform)
 
 (define (pass3/$CLAMBDA iform labels)
-  iform)
+  (let loop ([cs ($clambda-closures iform)] [r '()])
+    (if (null? cs)
+      (begin
+        ($clambda-closures-set! iform (reverse r))
+        iform)
+      (loop (cdr cs) (cons (pass3/rec (car cs) labels) r)))))
 
 (define (pass3/$LABEL iform labels)
   (unless (label-seen? labels iform)
@@ -304,6 +317,16 @@
            ;; call ($call ($lambda ...) args ...)
            (has-tag? proc $LAMBDA)
            (pass3/inline-call iform proc args labels)]
+          [;; Similar to above, but ($call ($seq ... ($lambda ..)) args ...)
+           ;; We can convert it ot ($seq ... ($call ($lamba ...) args...))
+           (and (has-tag? proc $SEQ)
+                (has-tag? (last ($seq-body proc)) $LAMBDA))
+           (pass3/$SEQ ($seq `(,@(drop-right ($seq-body proc) 1)
+                               ,($call ($*-src iform)
+                                       (last ($seq-body proc)) ;$lambda
+                                       ($call-args iform)
+                                       ($call-flag iform))))
+                       labels)]
           [;; If we get ($call ($gref proc) args ...) and proc is inlinable,
            ;; we can inline the call.
            (and-let* ([ (has-tag? proc $GREF) ]
