@@ -1,7 +1,7 @@
 ;;;
 ;;; compile-4.scm - The compiler: Pass 4
 ;;;
-;;;   Copyright (c) 2004-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -166,7 +166,7 @@
                ;; we don't need to lift it, nor need to set free-lvars.
                ;; We just mark it by setting lifted-var to #t so that
                ;; pass4/lift phase can treat it specially.
-               (unless (eq? ($lambda-flag iform) 'dissolved)
+               (unless ($lambda-dissolved? iform)
                  (label-dic-info-push! labels iform) ;save the lambda node
                  (when t?                            ;mark this is toplevel
                    ($lambda-lifted-var-set! iform #t)))
@@ -177,6 +177,7 @@
                                fs
                                (loop (cdr inner-fs)
                                      (pass4/add-lvar (car inner-fs) bs fs))))]))]
+  [($CLAMBDA) (pass4/scan* ($clambda-closures iform) bs fs t? labels)]
   [($LABEL)  (cond [(label-seen? labels iform) fs]
                    [else (label-push! labels iform)
                          (pass4/scan ($label-body iform) bs fs t? labels)])]
@@ -189,6 +190,8 @@
   [($CONS $APPEND $MEMV $EQ? $EQV?) (pass4/scan2 iform bs fs t? labels)]
   [($VECTOR $LIST $LIST*) (pass4/scan* ($*-args iform) bs fs t? labels)]
   [($LIST->VECTOR) (pass4/scan ($*-arg0 iform) bs fs t? labels)]
+  [($DYNENV) (let1 fs (pass4/scan* ($dynenv-kvs iform) bs fs #f labels)
+               (pass4/scan ($dynenv-body iform) bs fs t? labels))]
   [else fs])
 
 (define (pass4/scan2 iform bs fs t? labels)
@@ -313,6 +316,7 @@
              (or (and-let* ([lifted ($lambda-lifted-var iform)])
                    ($lref lifted))
                  iform)]
+  [($CLAMBDA) (pass4/subst*! ($clambda-closures iform) labels) iform]
   [($LABEL)  (unless (label-seen? labels iform)
                (label-push! labels iform)
                (pass4/subst! ($label-body iform) labels))
@@ -325,4 +329,7 @@
              (pass4/subst! ($*-arg1 iform) labels)]
   [($VECTOR $LIST $LIST*) (pass4/subst*! ($*-args iform) labels) iform]
   [($LIST->VECTOR) (pass4/subst! ($*-arg0 iform) labels)]
+  [($DYNENV) (pass4/subst*! ($dynenv-kvs iform) labels)
+             (pass4/subst! ($dynenv-body iform) labels)
+             iform]
   [else iform])
