@@ -1,7 +1,7 @@
 ;;;
 ;;; compile-i.scm - Inliners of builtin procedures
 ;;;
-;;;   Copyright (c) 2004-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -248,6 +248,12 @@
 (define-builtin-inliner-uvref f32 F32)
 (define-builtin-inliner-uvref f64 F64)
 
+(define-builtin-inliner uvector-ref
+  (^[src args]
+    (match args
+      [(vec ind) ($asm src `(,UVEC-REF ,SCM_UVECTOR_GENERIC) `(,vec ,ind))]
+      [else (undefined)])))
+
 (define-builtin-inliner zero?
   (^[src args]
     (match args
@@ -294,21 +300,24 @@
              [r (make-lvar 'tmp)])
          (if (constant-lambda? a)
            ;; when after thunk is dummy, we don't bother to call it.
+           ;; NB: POP-HANDLERS does not affect value registers, so we don't
+           ;; need to save the result of thunk.
            ($let src 'let `(,at ,bt ,tt) `(,a ,b ,t)
                  ($seq
-                  `(,($call ($*-src b) ($lref bt) '())
+                  `(,($call (iform-src b) ($lref bt) '())
                     ,($asm src `(,PUSH-HANDLERS 0) `(,($lref bt) ,($lref at)))
-                    ,($call ($*-src t) ($lref tt) '()))))
+                    ,($call (iform-src t) ($lref tt) '())
+                    ,($asm src `(,POP-HANDLERS) '()))))
            ;; normal path
            ($let src 'let `(,at ,bt ,tt) `(,a ,b ,t)
                  ($seq
-                  `(,($call ($*-src b) ($lref bt) '())
+                  `(,($call (iform-src b) ($lref bt) '())
                     ,($asm src `(,PUSH-HANDLERS 0) `(,($lref bt) ,($lref at)))
                     ,($receive #f 0 1 (list r)
-                               ($call ($*-src t) ($lref tt) '())
+                               ($call (iform-src t) ($lref tt) '())
                                ($seq
                                 `(,($asm src `(,POP-HANDLERS) '())
-                                  ,($call ($*-src a) ($lref at) '())
+                                  ,($call (iform-src a) ($lref at) '())
                                   ,($asm #f `(,TAIL-APPLY 2)
                                          (list ($gref values.) ($lref r))))))
                     )))))]
