@@ -1,7 +1,7 @@
 ;;;
 ;;; compile.scm - The compiler
 ;;;
-;;;   Copyright (c) 2004-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -39,14 +39,14 @@
 (select-module gauche.internal)
 
 (inline-stub
- (declcode
-  (.include <gauche/code.h>
-            <gauche/vminsn.h>
-            <gauche/priv/builtin-syms.h>
-            <gauche/priv/glocP.h>
-            <gauche/priv/identifierP.h>
-            <gauche/priv/macroP.h>
-            <gauche/priv/moduleP.h>)))
+ (.include "gauche/priv/configP.h"
+           "gauche/code.h"
+           "gauche/vminsn.h"
+           "gauche/priv/builtin-syms.h"
+           "gauche/priv/glocP.h"
+           "gauche/priv/identifierP.h"
+           "gauche/priv/macroP.h"
+           "gauche/priv/moduleP.h"))
 
 ;;; THE COMPILER
 ;;;
@@ -167,6 +167,7 @@
   $EQ?
   $EQV?
   $IT
+  $DYNENV
   )
 
 ;; Define constants for VM instructions.
@@ -238,13 +239,12 @@
 
 ;; implemented in C for better performance.
 (inline-stub
- ;; offsets must be in sync with lvar definition above
- "#define LVAR_OFFSET_TAG        0"
- "#define LVAR_OFFSET_NAME       1"
- "#define LVAR_OFFSET_INITVAL    2"
- "#define LVAR_OFFSET_REF_COUNT  3"
- "#define LVAR_OFFSET_SET_COUNT  4"
- "#define LVAR_SIZE              5"
+ (.define LVAR_OFFSET_TAG       0)
+ (.define LVAR_OFFSET_NAME      (lvar-name-offset))
+ (.define LVAR_OFFSET_INITVAL   (lvar-initval-offset))
+ (.define LVAR_OFFSET_REF_COUNT (lvar-ref-count-offset))
+ (.define LVAR_OFFSET_SET_COUNT (lvar-set-count-offset))
+ (.define LVAR_SIZE             (lvar-size))
 
  ;; Specialized routine for (map (lambda (name) (make-lvar name)) objs)
  (define-cproc %map-make-lvar (names)
@@ -554,7 +554,9 @@
    optarg           ; 0 or 1, # of optional arg
    lvars            ; list of lvars
    body             ; IForm for the body
-   flag             ; Marks some special state of this node.
+   meta             ; List of meta information of the nature of this lambda
+                    ; node (*1)
+                    ; Members can be:
                     ;   'dissolved: indicates that this lambda has been
                     ;               inline expanded.
                     ;   'used: indicates that this lambda has been already dealt
@@ -562,6 +564,9 @@
                     ;          specifically used for communication between
                     ;          pass2/$CALL and pass2/$LET.
                     ;   <packed-iform>  : inlinable lambda
+                    ;   'constant: This lambda form is constant, meaning
+                    ;          it yields the same result if all the arguments
+                    ;          are constant.
    ;; The following slots are used temporarily during pass2-5, and
    ;; need not be saved when packed.
    (calls '())      ; list of call sites
@@ -571,12 +576,79 @@
                     ; is to be bound.  See pass 4.
    ))
 
+;; (*1) Up to 0.9.14, this slot contains a symbol or a packed-iform, not a
+;; list.  This slot is embedded in the precompiled files, so we need to
+;; deal with such a case.  Because of this, $lambda-meta slot must
+;; be accessed with the following utillities.
+
+;; TRANSIENT: Cover the case for code precompiled with older versions.
+;; Ensure that we always return a list.
+(define-inline ($lambda-metas iform)
+  (let1 f ($lambda-meta iform)
+    (if (or (null? f) (pair? f))
+      f
+      (list f))))
+
+(define-inline ($lambda-dissolved? iform)
+  (let1 f ($lambda-meta iform)
+    (or (eq? f 'dissolved) (memq 'dissolved f))))
+(define-inline ($lambda-used? iform)
+  (let1 f ($lambda-meta iform)
+    (or (eq? f 'used) (memq 'used f))))
+(define-inline ($lambda-constant? iform)
+  (let1 f ($lambda-meta iform)
+    (or (eq? f 'used) (memq 'used f))))
+(define-inline ($lambda-inlinable? iform)
+  (let1 f ($lambda-meta iform)
+    (or (vector? f) (and (pair? f) (any vector? f)))))
+
+(define-inline ($lambda-inliner iform)
+  (let1 f ($lambda-meta iform)
+    (if (vector? f) f (and (pair? f) (find vector? f)))))
+(define ($lambda-inliner-set! iform packed-inliner)
+  (let* ([fs ($lambda-metas iform)]
+         [fs. (if (any vector? fs)
+                (cons packed-inliner (remove vector? fs))
+                (cons packed-inliner fs))])
+    ($lambda-meta-set! iform fs.)))
+
+;; used and dissolved are mutually exclusive
+(define ($lambda-dissolved-set! iform)
+  (let* ([fs ($lambda-metas iform)]
+         [fs. (cond [(memq 'dissolved fs) fs]
+                    [(memq 'used fs) (cons 'dissolved (delete 'used fs))]
+                    [else (cons 'dissolved fs)])])
+    ($lambda-meta-set! iform fs.)))
+(define ($lambda-used-set! iform)
+  (let* ([fs ($lambda-metas iform)]
+         [fs. (cond [(memq 'used fs) fs]
+                    [(memq 'dissolved fs) (cons 'used (delete 'dissolved fs))]
+                    [else (cons 'used fs)])])
+    ($lambda-meta-set! iform fs.)))
+
+(define ($lambda-type iform)
+  ;; We avoid (find type? ($lambda-metas iform)), for it may allocate
+  ;; unnecessarily.  With the old compiler, $lambda-meta never directly
+  ;; contains type object.
+  (let1 f ($lambda-meta iform)
+    (and (pair? f) (find type? f))))
+(define ($lambda-type-set! iform type)
+  ;; we assume this is done at most once
+  ($lambda-meta-set! iform (cons type ($lambda-metas iform))))
+
 ;; $clambda <src> <name> <lambda-node> ...
 ;;   Case-lambda.
+;;   Closures slot contains two or more IForms.  They're initially $LAMBDA
+;;   nodes, but may be replaced with $LREF nodes by lambda-lifting pass.
+;;   The argcounts slot is computed when this node is constructed, and
+;;   unpacked.
 (define-simple-struct $clambda $CLAMBDA $clambda
-  (src
-   name
-   closures))
+  (src              ; original source for debugging
+   name             ; inferred name of this closure
+   closures         ; list of IForms
+   ;; The following slots are transient, not saved when packed.
+   (argcounts '())  ; ((#reqargs . #optargs) ...)
+   ))
 
 ;; $label <src> <label> <body>
 ;;    This kind of IForm node is introduced in Pass2 to record a shared
@@ -631,6 +703,18 @@
 (define $it (let ((c `#(,$IT))) (^[] c)))
 (define-inline ($it? x) (has-tag? x $IT))
 
+;; $dynenv <src> (<key> <value>) <expr>
+;;   Extend the dynamic environment and evaluate <expr>.
+;;   If this is at the tail context, the dynenv of the current
+;;   continuation is extended.  Otherwise, a new continuation frame is
+;;   pushed, dynenv is extended and <expr> is evalutaed in a tail
+;;   context.
+(define-simple-struct $dynenv $DYNENV $dynenv
+  (src       ; original source for debugging
+   kvs       ; (key value) exprs
+   body      ; body expression
+   ))
+
 ;; The followings are builtin version of standard procedures.
 ;;
 (define-simple-struct $cons $CONS #f (src arg0 arg1))
@@ -660,7 +744,9 @@
 (define-simple-struct $list*  $LIST*  $list*  (src args))
 (define-simple-struct $list->vector $LIST->VECTOR $list->vector (src arg0))
 
-;; common accessors
+;; Common accessors
+;; NB: These slots are not universal.  Make sure you have proper IForm type
+;; before using them.
 (define-inline ($*-src  iform)  (vector-ref iform 1))
 (define-inline ($*-args iform)  (vector-ref iform 2))
 (define-inline ($*-arg0 iform)  (vector-ref iform 2))
@@ -669,6 +755,20 @@
 (define-inline ($*-arg0-set! iform val)  (vector-set! iform 2 val))
 (define-inline ($*-arg1-set! iform val)  (vector-set! iform 3 val))
 
+;; Extract src info from iform.  Most of the time, source extraction
+;; is done after handlers are dispatched by iform types, so you can
+;; use $*-src instead (b/c you already know the type of the iform).
+;; You want this only in the custom inlininer that generates iforms.
+(define (iform-src iform)
+  (case/unquote
+   (iform-tag iform)
+   [($LREF) (lvar-name ($lref-lvar iform))]
+   [($LSET) (lvar-name ($lset-lvar iform))]
+   [($GREF) ($gref-id iform)]
+   [($GSET) ($gset-id iform)]
+   [($CONST) ($const-value iform)]
+   [else ($*-src iform)]))
+
 ;; look up symbolic name of iform tag (for debugging)
 (define (iform-tag-name tag)
   (let loop ((p .intermediate-tags.))
@@ -746,10 +846,14 @@
        [($LAMBDA) (format #t "($lambda[~a.~a~a~a] ~a" ($lambda-name iform)
                           (length ($lambda-calls iform))
                           (if (null? ($lambda-free-lvars iform)) "" "c")
-                          (if (vector? ($lambda-flag iform)) " inlinable" "")
+                          (if ($lambda-inlinable? iform) " inlinable" "")
                           (map lvar->string ($lambda-lvars iform)))
         (nl (+ ind 2))
         (rec (+ ind 2) ($lambda-body iform)) (display ")")]
+       [($CLAMBDA) (format #t "($clambda ~a" ($clambda-name iform))
+                   (for-each (^l (nl (+ ind 2)) (rec (+ ind 2) l))
+                             ($clambda-closures iform))
+                   (display ")")]
        [($LABEL) (if-let1 p (assq iform labels)
                    (format #t "label#~a" (cdr p))
                    (let1 num (length labels)
@@ -795,6 +899,14 @@
         (display "($LIST->VECTOR ")
         (rec (+ ind 14) (vector-ref iform 2))
         (display ")")]
+       [($DYNENV)
+        (display "($DYNENV (")
+        (rec (+ ind 10) (car ($dynenv-kvs iform)))
+        (nl (+ ind 10))
+        (rec (+ ind 10) (cadr ($dynenv-kvs iform)))
+        (nl (+ ind 2))
+        (rec (+ ind 2) ($dynenv-body iform))
+        (display ")")]
        [else (error "pp-iform: unknown tag:" (iform-tag iform))]))
     (newline)))
 
@@ -867,7 +979,10 @@
                       ($lambda-optarg iform)
                       (map get-ref ($lambda-lvars iform))
                       (get-ref ($lambda-body iform))
-                      ($lambda-flag iform))]
+                      ($lambda-metas iform))]
+     [($CLAMBDA) (put! iform '$CLAMBDA ($*-src iform)
+                       ($clambda-name iform)
+                       (map get-ref ($clambda-closures iform)))]
      [($LABEL)  (put! iform '$LABEL ($*-src iform) #f
                       (get-ref ($label-body iform)))]
      [($SEQ)    (put! iform '$SEQ (map get-ref ($seq-body iform)))]
@@ -889,6 +1004,9 @@
      [($LIST*)  (put!-nary iform '$LIST*)]
      [($LIST->VECTOR) (put! iform '$LIST->VECTOR ($*-src iform)
                             (get-ref ($*-arg0 iform)))]
+     [($DYNENV) (put! iform '$DYNENV ($*-src iform)
+                      (map get-ref ($dynenv-kvs iform))
+                      (get-ref ($dynenv-body iform)))]
      [('lvar)   (put! iform 'lvar (lvar-name iform))]
      [else (errorf "[internal-error] unknown IForm in pack-iform: ~S" iform)]
      ))
@@ -914,10 +1032,6 @@
             [else (rlet1 body (unpack-body ref)
                     (hash-table-put! dict ref body))]))
     (define (unpack-body i)
-      ;; NB: We dispatch by symbol, not the numeric tag value.
-      ;; TRANSIENT: Due to a bug, 0.9.7 precompiled IForm contains numeric
-      ;; tag 7 in place of $LET.  To keep binary compatibility, we have to
-      ;; support it until the next ABI bump.
       (case (V i)
        [($DEFINE) ($define (V i 1) (V i 2) (V i 3) (unpack-rec (V i 4)))]
        [($LREF)   ($lref (unpack-rec (V i 1)))]
@@ -925,20 +1039,23 @@
        [($GREF)   ($gref (V i 1))]
        [($GSET)   ($gset (V i 1) (unpack-rec (V i 2)))]
        [($CONST)  ($const (V i 1))]
-       [($IF)  ($if (V i 1) (unpack-rec (V i 2))
+       [($IF)     ($if (V i 1) (unpack-rec (V i 2))
                     (unpack-rec (V i 3)) (unpack-rec (V i 4)))]
-       [($LET 7) (rlet1 unpacked
-                     ($let (V i 1) (V i 2)
-                           (map unpack-rec (V i 3)) (map unpack-rec (V i 4))
-                           (unpack-rec (V i 5)))
-                   (ifor-each2 (^(lv in) (lvar-initval-set! lv in))
-                               ($let-lvars unpacked) ($let-inits unpacked)))]
+       [($LET)    (rlet1 unpacked
+                      ($let (V i 1) (V i 2)
+                            (map unpack-rec (V i 3)) (map unpack-rec (V i 4))
+                            (unpack-rec (V i 5)))
+                    (ifor-each2 (^(lv in) (lvar-initval-set! lv in))
+                                ($let-lvars unpacked) ($let-inits unpacked)))]
        [($RECEIVE) ($receive (V i 1) (V i 2) (V i 3)
                              (map unpack-rec (V i 4)) (unpack-rec (V i 5))
                              (unpack-rec (V i 6)))]
        [($LAMBDA) ($lambda (V i 1) (V i 2) (V i 3) (V i 4)
                            (map unpack-rec (V i 5))
                            (unpack-rec (V i 6)) (V i 7))]
+       [($CLAMBDA) (let1 lambda-nodes (map unpack-rec (V i 3))
+                     ($clambda (V i 1) (V i 2) lambda-nodes
+                               (compute-clambda-argcounts lambda-nodes)))]
        [($LABEL)  ($label (V i 1) (V i 2) (unpack-rec (V i 3)))]
        [($SEQ)    ($seq (map unpack-rec (V i 1)))]
        [($CALL)   ($call (V i 1) (unpack-rec (V i 2))
@@ -954,6 +1071,7 @@
        [($LIST)   ($list (V i 1) (map unpack-rec (V i 2)))]
        [($LIST*)  ($list* (V i 1) (map unpack-rec (V i 2)))]
        [($LIST->VECTOR) ($list->vector (V i 1) (unpack-rec (V i 2)))]
+       [($DYNENV) ($dynenv (V i 1) (map unpack-rec (V i 2)) (V i 3))]
        [(lvar)    (make-lvar (V i 1))]
        [else
         (errorf "[internal error] unpack-iform: ivec broken at ~a (tag=~s): ~S"
@@ -985,6 +1103,7 @@
        [($RECEIVE)
         (sum-items (+ cnt 1) ($receive-expr iform) ($receive-body iform))]
        [($LAMBDA) (sum-items (+ cnt 1) ($lambda-body iform))]
+       [($CLAMBDA) (sum-items (+ cnt 1) (* ($clambda-closures iform)))]
        [($LABEL)  (sum-items cnt ($label-body iform))]
        [($SEQ)    (sum-items cnt (* ($seq-body iform)))]
        [($CALL) (sum-items (+ cnt 1) ($call-proc iform) (* ($call-args iform)))]
@@ -993,6 +1112,8 @@
         (sum-items (+ cnt 1) ($*-arg0 iform) ($*-arg1 iform))]
        [($VECTOR $LIST $LIST*) (sum-items (+ cnt 1) (* ($*-args iform)))]
        [($LIST->VECTOR) (sum-items (+ cnt 1) ($*-arg0 iform))]
+       [($DYNENV) (sum-items (+ cnt 1) (* ($dynenv-kvs iform))
+                             ($dynenv-body iform))]
        [($IT) cnt]
        [else
         (error "[internal error] iform-count-size-upto: unknown iform tag:"
@@ -1049,7 +1170,10 @@
                          ($lambda-reqargs iform) ($lambda-optarg iform)
                          newlvs
                          (iform-copy ($lambda-body iform) newalist)
-                         ($lambda-flag iform)))]
+                         ($lambda-meta iform)))]
+   [($CLAMBDA) ($clambda ($*-src iform) ($clambda-name iform)
+                         (imap (cut iform-copy <> lv-alist)
+                               ($clambda-closures iform)))]
    [($LABEL)
     (cond [(assq iform lv-alist) => (^p (cdr p))]
           [else
@@ -1100,6 +1224,9 @@
                      (iform-copy ($*-arg0 iform) lv-alist)
                      (iform-copy ($*-arg1 iform) lv-alist))]
    [($IT) ($it)]
+   [($DYNENV) ($dynenv ($*-src iform)
+                       (imap (cut iform-copy <> lv-alist) ($dynenv-kvs iform))
+                       (iform-copy ($dynenv-body iform)))]
    [else iform]))
 
 (define (iform-copy-zip-lvs orig-lvars lv-alist)
@@ -1144,7 +1271,9 @@
                          ($lambda-reqargs iform) ($lambda-optarg iform)
                          ($lambda-lvars iform)
                          (rec ($lambda-body iform))
-                         ($lambda-flag iform))]
+                         ($lambda-meta iform))]
+     [($CLAMBDA) ($clambda ($*-src iform) ($clambda-name iform)
+                           (imap rec ($clambda-closures iform)))]
      [($LABEL)  (error "[compiler internal] $LABEL node shouldn't appear \
                         in the packed IForm")]
      [($SEQ)    ($seq (imap rec ($seq-body iform)))]
@@ -1182,6 +1311,9 @@
                        (rec ($*-arg0 iform))
                        (rec ($*-arg1 iform)))]
      [($IT) ($it)]
+     [($DYNENV) ($dynenv ($*-src iform)
+                         (imap rec ($dynenv-kvs iform))
+                         (rec ($dynenv-body iform)))]
      [else iform]))
 
   (pack-iform (rec (unpack-iform src))))
@@ -1216,6 +1348,7 @@
      [($RECEIVE)(and (rec ($receive-expr iform) labels)
                      (rec ($receive-body iform) labels))]
      [($LAMBDA) #t]
+     [($CLAMBDA) #t]
      [($LABEL)  (or (label-seen? labels iform)
                     (begin (label-push! labels iform)
                            (rec ($label-body iform) labels)))]
@@ -1231,6 +1364,8 @@
      [($LIST->VECTOR) (rec ($*-arg0 iform) labels)]
      [($IT) #t] ; this branch is only executed when $if-test of the parent is
                                         ; transparent, thus this node is also transparent.
+     [($DYNENV) (and (everyc rec ($dynenv-kvs iform) labels)
+                     (rec ($dynenv-body iform) labels))]
      [else #f]))
   (rec iform (make-label-dic #f)))
 
@@ -1259,6 +1394,7 @@
              (reset-lvars/rec ($receive-body iform) labels)]
   [($LAMBDA) (for-each lvar-reset ($lambda-lvars iform))
    (reset-lvars/rec ($lambda-body iform) labels)]
+  [($CLAMBDA) (reset-lvars/rec* ($clambda-closures iform) labels)]
   [($LABEL)  (unless (label-seen? labels iform)
                (label-push! labels iform)
                (reset-lvars/rec ($label-body iform) labels))]
@@ -1272,6 +1408,8 @@
              (reset-lvars/rec ($*-arg1 iform) labels)]
   [($VECTOR $LIST $LIST*) (reset-lvars/rec* ($*-args iform) labels)]
   [($LIST->VECTOR) (reset-lvars/rec ($*-arg0 iform) labels)]
+  [($DYNENV) (reset-lvars/rec* ($dynenv-kvs iform) labels)
+             (reset-lvars/rec ($dynenv-body iform) labels)]
   [else #f])
 
 ;; Replaces $LREF of matching lvar with given expression.
@@ -1356,7 +1494,10 @@
                     iform
                     ($lambda ($*-src iform) ($lambda-name iform)
                              ($lambda-reqargs iform) ($lambda-optarg iform)
-                             ($lambda-lvars iform) b ($lambda-flag iform))))]
+                             ($lambda-lvars iform) b ($lambda-meta iform))))]
+     [($CLAMBDA) ($clambda ($*-src iform) ($clambda-name iform)
+                           (imap (cut subst <> mapping dict)
+                                 ($clambda-closures iform)))]
      [($SEQ)    ($seq (imap (cut subst <> mapping dict) ($seq-body iform)))]
      [($CALL)   ($call ($*-src iform)
                        (subst ($call-proc iform) mapping dict)
@@ -1368,6 +1509,9 @@
      [($VECTOR $LIST $LIST*) (subst/* iform mapping dict)]
      [($LIST->VECTOR) ($list->vector ($*-src iform)
                                      (subst ($*-arg0 iform) mapping dict))]
+     [($DYNENV) ($dynenv ($*-src iform)
+                         (imap (cut subst <> mapping dict) ($dynenv-kvs iform))
+                         (subst ($dynenv-body iform) mapping dict))]
      [else iform]))
   (define (subst/2 iform mapping dict)
     (vector (vector-ref iform 0) ($*-src iform)
@@ -1431,6 +1575,8 @@
                                    vars
                                    (apply list* vars))])
                    `(lambda ,formals ,(rec ($lambda-body iform))))]
+     [($CLAMBDA) (let1 cs (map rec ($clambda-closures iform))
+                   `(case-lambda ,@(map cdr cs)))]
      [($SEQ)    `(begin ,@(map rec ($seq-body iform)))]
      [($CALL)   `(,(rec ($call-proc iform)) ,@(map rec ($call-args iform)))]
      [($ASM)    `(asm ,($asm-insn iform)
@@ -1446,12 +1592,37 @@
      [($LIST->VECTOR) `(list->vector ,(rec ($*-arg0 iform)))]
      [($IT) ; ($IT) node must have been handled by $IF
       (error "[internal] Stray $IT node.")]
+     [($DYNENV) `(with-continuation-mark
+                  ,(rec (car ($dynenv-kvs iform)))
+                  ,(rec (cadr ($dynenv-kvs iform)))
+                  ,(rec ($dynenv-body iform)))]
      [else (error "Cannot convert IForm:" (iform-tag-name (iform-tag iform)))]))
   (rec iform))
 
 ;; Some inline stuff
 (define-inline (pass2-4 iform module) (pass4 (pass3 (pass2 iform) #f) module))
 
+;; Compile target.  Used in pass5 to represent
+
+(define-simple-struct ctarget #f %make-compile-target
+  (ccb                                ;compiled code builder
+   env-header-size
+   cont-frame-size))
+
+(define (make-compile-target :optional
+                             (env-header-size #f)
+                             (cont-frame-size #f))
+  (%make-compile-target
+   (make-compiled-code-builder 0 0 '%toplevel #f #f)
+   (or env-header-size ENV_HEADER_SIZE)
+   (or cont-frame-size CONT_FRAME_SIZE)))
+
+(define (make-child-compile-target new-ccb parent-target)
+  (%make-compile-target
+   new-ccb
+   (ctarget-env-header-size parent-target)
+   (ctarget-cont-frame-size parent-target)))
+
 ;;============================================================
 ;; Entry points
 ;;
@@ -1461,23 +1632,45 @@
 ;;     #f, #<unbound> - compile on bottom env of the current module
 ;;     module - compile on bottom env of the given module
 ;;     cenv   - compile on the given cenv
-(define (compile program env)
+;;
+;; target-params is an key-value list, with the following keys
+;; recognized.
+;;    :env-header-size  - size of environment frame header
+;;    :cont-frame-size  - size of continuation frame
+(define (compile program env :key (target-params '()))
   (let1 cenv (cond [(module? env) (make-bottom-cenv env)]
                    [(vector? env) env] ; assumes env is cenv
                    [else (make-bottom-cenv)]) ; use default module
-    (guard (e [else
-               ;; TODO: check if e is an expected error (such as syntax error)
-               ;; or an unexpected error (compiler bug).
-               ($ raise $ make-compound-condition e
-                  $ make <compile-error-mixin> :expr program)])
-      (pass5 (pass2-4 (pass1 program cenv) (cenv-module cenv))
-             (make-compiled-code-builder 0 0 '%toplevel #f #f)
-             '() 'tail))))
+    (receive (env-header-size cont-frame-size)
+        (parse-target-params target-params)
+      (guard (e [else
+                 ;; Attach the source with <compile-error-mixin>, if it is
+                 ;; not alrady attached.
+                 ;; TODO: check if e is an expected error (such as syntax error)
+                 ;; or an unexpected error (compiler bug).
+                 (if (and (is-a? e <compound-condition>)
+                          (find (^c (and (is-a? c <compile-error-mixin>)
+                                         (eq? (~ c'expr) program)))
+                                (~ e '%conditions)))
+                   (raise e)
+                   ($ raise $ make-compound-condition e
+                      $ make <compile-error-mixin> :expr program))])
+        (pass5 (pass2-4 (pass1 program cenv) (cenv-module cenv))
+               (make-compile-target env-header-size cont-frame-size)
+               '() 'tail)))))
 
 ;; stub for future extension
 (define (compile-partial program module) #f)
 (define (compile-finish cc) #f)
 
+(define (parse-target-params target-params)
+  (let-keywords target-params ((env-header-size #f)
+                               (cont-frame-size #f)
+                               . other-keys)
+    (unless (null? other-keys)
+      (error "Unrecognized compiler target parameter key:" other-keys))
+    (values env-header-size cont-frame-size)))
+
 ;; For testing
 (define (compile-p1 program :optional (env (vm-current-module)))
   (pp-iform (pass1 program (make-bottom-cenv env))))
@@ -1495,7 +1688,7 @@
 (define (compile-p5 program :optional (env (vm-current-module)))
   (let1 cenv (make-bottom-cenv env)
     (vm-dump-code (pass5 (pass2-4 (pass1 program cenv) (cenv-module cenv))
-                         (make-compiled-code-builder 0 0 '%toplevel #f #f)
+                         (make-compile-target)
                          '() 'tail))))
 
 (include "compile-1")
@@ -1604,17 +1797,25 @@
 (define (%make-er-transformer xformer def-env
                               :key (has-inject? #f)
                                    (info-alist '())
-                                   (identifier-macro? #f))
+                                   (identifier-macro? #f));deprecated
   (define def-module (cenv-module def-env))
   (define def-frames (cenv-frames def-env))
   (define (expand form use-env)
     (define use-module (cenv-module use-env))
     (define use-frames (cenv-frames use-env))
     (let1 dict '()
-      (define (%rename sym)
-        (receive [id dict_] (er-rename sym dict def-module def-frames)
-          (set! dict dict_)
-          id))
+      (define %rename
+        ;; NB: We attach macro input in a procedure tag value (not the SRFI-229
+        ;; tag, but using the native alist tag.)  Quasirename retrieves this
+        ;; info and attach it to the output.
+        ;; For now, we don't have lambda/tags-alist, so we have to copy
+        ;; the procedure.
+        (%procedure-copy
+         (lambda (sym)
+           (receive [id dict_] (er-rename sym dict def-module def-frames)
+             (set! dict dict_)
+             id))
+         `((macro-input . ,(unwrap-syntax form)))))
       (define (%compare a b) (er-compare a b use-module use-frames))
       (define (%inject sym)
         (receive [id dict_] (er-rename sym dict use-module use-frames)
@@ -1634,13 +1835,29 @@
 (define (%make-er-transformer/toplevel xformer def-module def-name
                                        :key (has-inject? #f)
                                             (info-alist '())
-                                            (identifier-macro? #f))
+                                            (identifier-macro? #f));deprecated
   (%make-er-transformer xformer
                         (%make-cenv def-module '() def-name)
                         :has-inject? has-inject?
                         :info-alist info-alist
                         :identifier-macro? identifier-macro?))
 
+;; Turn a macro to an id macro.
+(define (%make-id-transformer macro name)
+  (assume-type macro <macro>)
+  (%make-macro-transformer name
+                           (~ macro'transformer)
+                           (~ macro'info-alist)
+                           (cons 'identifier-macro (~ macro'flags))))
+
+;; Turn a macro to a parameterizable macro.
+(define (%make-parameterizable-transformer macro name)
+  (assume-type macro <macro>)
+  (%make-macro-transformer name
+                           (~ macro'transformer)
+                           (~ macro'info-alist)
+                           (cons 'parameterizable (~ macro'flags))))
+
 ;; Returns an S-expr all macros in which are expanded.
 ;; The resulting form may not be equivalent to the input form, though,
 ;; since we strip off identifier information so toplevel hygiene isn't
@@ -1747,7 +1964,7 @@
 ;;   I still don't feel "right" about having this hack here, though.
 ;;   Just keep it for now, for the record.
 (inline-stub
- (if "defined(RECORD_DEPENDED_MODULES)"
+ (.if (defined RECORD_DEPENDED_MODULES)
    (begin
      "static ScmModule *stdmods[3];"
 
@@ -1807,6 +2024,29 @@
      ))
  )
 
+;; GLOBAL-REF-TYPE
+;;
+;;   Similar to global-call-type, but this is called when ID is used
+;;   other than the head of list.  Should be called after we know ID is
+;;   global reference.
+;;
+;;   If ID is globally bound to a macro, returns #<macro>.
+;;   Otherwise, returns ID itself.
+;;
+;;   We separate this from global-call-type, since we don't need to
+;;   handle inlinable function case.
+;;
+(inline-stub
+ (define-cproc global-ref-type (id) ::<top>
+   (let* ([gloc::ScmGloc* (Scm_IdentifierGlobalBinding (SCM_IDENTIFIER id))])
+     (if gloc
+       (let* ([gval (SCM_GLOC_GET gloc)])
+         (if (SCM_MACROP gval)
+           (return gval)
+           (return id)))
+       (return id))))
+ )
+
 ;; Returns #t iff z is a keyword, or an identifier made from a keyword.
 ;; TODO: We should also check iff k isn't bound to something else!
 (define (keyword-like? k)
@@ -1925,10 +2165,20 @@
 ;;
 
 (inline-stub
- ;; standard
- (define-constant ENV_HEADER_SIZE  (c "SCM_MAKE_INT(ENV_SIZE(0))"))
- (define-constant CONT_FRAME_SIZE (c "SCM_MAKE_INT(CONT_FRAME_SIZE)"))
+ (define-cproc %get-env-frame-size () ::<fixnum>
+   (return (ENV_SIZE 0)))
 
+ (define-cproc %get-cont-frame-size () ::<fixnum>
+   (return CONT_FRAME_SIZE))
+
+ )
+
+;; These values can be overridden when we cross-compile (including
+;; inter-version compiling).
+(define ENV_HEADER_SIZE (%get-env-frame-size))
+(define CONT_FRAME_SIZE (%get-cont-frame-size))
+
+(inline-stub
  ;; Eval situation flag (for eval-when constrcut)
  (define-cproc vm-eval-situation (:optional val) ::<int>
    (let* ([vm::ScmVM* (Scm_VM)])
@@ -1987,6 +2237,10 @@
 ;; Initialization
 ;;
 
+;; This is called after all built-in procedures and compilers are
+;; brought up.  Remaining initializations are autoloads and cond-features.
+;; We can put miscellaneous bookkeeping here.  Be careful not to trigger
+;; autoloads, for it requires the compiler being functional.
 (define (init-compiler)
   #f
   )
@@ -2013,6 +2267,7 @@
 ;; translated to C.  They can't be evaluated at runtime, so
 ;; we have dummy macros to warn.
 (%precomp-only inline-stub
+               in-module
                define-cproc
                define-cfn
                define-enum
