@@ -1,7 +1,7 @@
 /*
  * dispatch.c - method dispatch accelerator
  *
- *   Copyright (c) 2017-2021  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2017-2024  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,6 +33,7 @@
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
+#include "gauche/priv/configP.h"
 #include "gauche/priv/atomicP.h"
 #include "gauche/priv/dispatchP.h"
 
@@ -89,7 +90,7 @@
  *   - We don't need to allocate to construct the list of applicable
  *     methods.
  *
- *  Note that if the entry has at least one non-leaf methods, we have to
+ *  Note that if the entry has at least one non-leaf method, we have to
  *  fall back to the normal path, since there may be other applicable
  *  methods that might be called via next-method.  To cut the overhead
  *  of checking, we keep the list of leaf methods and the one of non-leaf
@@ -117,7 +118,7 @@
 struct ScmMethodDispatcherRec {
     int axis;                    /* Which argument we look at?
                                     This is immutable. */
-    ScmAtomicVar methodHash;	 /* mhash.  In case mhash is extended,
+    ScmAtomicVar methodHash;     /* mhash.  In case mhash is extended,
                                     we atomically swap reference. */
 };
 
@@ -131,7 +132,7 @@ typedef struct mhash_entry_rec {
 typedef struct mhash_rec {
     int size;                   /* # of bins.  power of 2. */
     int num_entries;            /* # of active entries.  */
-    ScmAtomicVar bins[1];	/* Table.  Each entry may have one of:
+    ScmAtomicVar bins[1];       /* Table.  Each entry may have one of:
                                      0 - free
                                      1 - deleted
                                      mhash_entry
