@@ -1,7 +1,7 @@
 /*
  * vmP.h - ScmVM private API
  *
- *   Copyright (c) 2017-2021  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2017-2024  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -38,6 +38,93 @@
 
 SCM_DECL_BEGIN
 
+/*
+ * Tagged continuation frame
+ *
+ *   A continuation frame can be tagged with call-with-continuation-prompt,
+ *   and it marks the bottom of a delimited continuation chain.
+ *
+ *   A prompt tag is an ScmObj to distinguish tagged continuation frame.
+ *   ScmContFrame->pc points to &(ScmPromptTag->insn), which contains dummy
+ *   RET instruction (so that it won't confuse codes that inspect VM state.)
+ *
+ *   A tagged continuation frame also points ScmPromptData from
+ *   ScmContFrame->cpc.  ScmPromptData is not an ScmObj.  It is a struct
+ *   to hold abort handler and some dynamic states.  The first word of
+ *   ScmPromptData contains a dummy RET instruction, so that it won't confuse
+ *   codes that inspect VM state.
+ */
+
+struct ScmPromptTagRec {
+    SCM_HEADER;
+    ScmObj name;                /* arbitrary object for debugging */
+    ScmWord insn;               /* a dummy field contains RET insn.
+                                   We make cont->pc point to here
+                                   for the prompt-tag marked continuation,
+                                   so that VM won't crash if it tries
+                                   to execute it accidentally. */
+};
+
+#define SCM_PROMPT_TAG_PC(ptag)   (&SCM_PROMPT_TAG(ptag)->insn)
+#define SCM_PC_TO_PROMPT_TAG(pc)  \
+    ((ScmPromptTag*)((char*)(pc) - offsetof(ScmPromptTag, insn)))
+
+/* ScmPromptData is allocated on the VM stack.  The size must be
+   multiple of ScmWord. */
+typedef struct ScmPromptDataRec {
+    ScmWord dummy;              /* RET insn */
+    ScmObj abortHandler;        /* abort handler */
+    ScmObj dynamicHandlers;     /* dynamic-wind handler chain */
+} ScmPromptData;
+
+#define PROMPT_DATA_SIZE       sizeof(ScmPromptData)/sizeof(ScmWord)
+
+/*
+ * Continuation mark set
+ *
+ *   Continuation mark set is an opaque object to extract a set of continuation
+ *   marks.  In Gauche, it is just a wrapper of the dynamic environment
+ *   and continuation chain; continuation-mark-set->list will do the actual
+ *   extraction work.
+ */
+
+struct ScmContinuationMarkSetRec {
+    SCM_HEADER;
+    ScmContFrame *cont;         /* current cont frame */
+    ScmObj denv;                /* vm->denv */
+    ScmObj bottomDenv;          /* limit of denv to look for */
+};
+
+/* To obtain denv keys */
+typedef enum {
+    SCM_DENV_KEY_EXCEPTION_HANDLER,
+    SCM_DENV_KEY_DYNAMIC_HANDLER,
+    SCM_DENV_KEY_PARAMETERIZATION,
+    SCM_DENV_KEY_EXPRESSION_NAME,
+} ScmDenvKeyName;
+
+SCM_EXTERN ScmObj Scm__GetDenvKey(ScmDenvKeyName name);
+
+/*
+ * Dynamic handler entry
+ *
+ *   Dynamic-wind creates this entry, packaging before/after handler,
+ *   the dynamic env on which it should be run, and extra arguments
+ *   for the handlers.
+ */
+typedef struct ScmDynamicHandlerRec {
+    SCM_HEADER;
+    ScmObj before;
+    ScmObj after;
+    ScmObj denv;
+    ScmObj args;
+} ScmDynamicHandler;
+
+SCM_CLASS_DECL(Scm_DynamicHandlerClass);
+#define SCM_CLASS_DYNAMIC_HANDLER  (&Scm_DynamicHandlerClass)
+#define SCM_DYNAMIC_HANDLER(obj)   ((ScmDynamicHandler*)obj)
+#define SCM_DYNAMIC_HANDLER_P(obj) SCM_ISA(obj,SCM_CLASS_DYNAMIC_HANDLER)
+
 /*
  * Escape point
  *
@@ -62,25 +149,14 @@ SCM_DECL_BEGIN
  *      (lambda () ...)      ;; <- (2)this is executed while EP1 is current
  *
  *  If the error handler returns, we pass its result to the continuation of
- *  with-error-handler, which is kept in EP1->cont.  The problem arises
- *  if a stack overflow occurs within the error handler, continuation frames
- *  in the stack are relocated to the heap, but EP1->cont isn't updated
- *  since it is out of vm->escapePoint chain.
- *
- *  'Floating' pointer is used to catch such case.  When an EP is popped
- *  before an error handler is ivoked, EP0's floating pointer is set to
- *  point EP1.  When a new EP is pushed, it inherits the previous EP's
- *  floating pointer.  With this scheme, active floating EPs are always
- *  reachable from vm->esapePoint->floating chain.  (NB: the chain length
- *  can be more than 1, if with-error-handler is used within an error handler
- *  and an error is signalled in its body.
+ *  with-error-handler, which is kept in EP1->cont.
  */
 typedef struct ScmEscapePointRec {
     struct ScmEscapePointRec *prev;
-    struct ScmEscapePointRec *floating;
     ScmObj ehandler;            /* handler closure */
     ScmContFrame *cont;         /* saved continuation */
-    ScmObj handlers;            /* saved dynamic handler chain */
+    ScmObj denv;                /* saved denv */
+    ScmObj dynamicHandlers;     /* saved dynamic handler chain */
     ScmCStack *cstack;          /* vm->cstack when escape point is created.
                                    this will be used to rewind cstack.
                                    this is NULL for partial continuations,
@@ -102,27 +178,18 @@ typedef struct ScmEscapePointRec {
                                    with-error-handler uses the latter model,
                                    but SRFI-34's guard needs the former model.
                                 */
-    int reraised;               /* EXPERIMENTAL: if exception is reraised,
-                                   this flag is set to TRUE and the exception
-                                   handler can return to the caller. */
-} ScmEscapePoint;
 
-/* Link management */
-#define SCM_VM_FLOATING_EP(vm) \
-    ((vm)->escapePoint? (vm)->escapePoint->floating : vm->escapePointFloating)
-#define SCM_VM_FLOATING_EP_SET(vm, ep)          \
-    do {                                        \
-        if ((vm)->escapePoint) {                \
-            (vm)->escapePoint->floating = (ep); \
-        } else {                                \
-            (vm)->escapePointFloating = (ep);   \
-        }                                       \
-    } while (0)
+    /* The following fields are used for new implementation of partial cont. */
+    ScmObj promptTag;
+    ScmObj abortHandler;
+    struct ScmEscapePointRec *bottom;
+} ScmEscapePoint;
 
 /* Escape types */
 #define SCM_VM_ESCAPE_NONE   0
 #define SCM_VM_ESCAPE_ERROR  1
 #define SCM_VM_ESCAPE_CONT   2
+#define SCM_VM_ESCAPE_ABORT  4
 #define SCM_VM_ESCAPE_EXIT   3
 
 /*
@@ -141,7 +208,7 @@ struct ScmCallTraceRec {
 };
 
 ScmCallTrace *Scm__MakeCallTraceQueue(u_long size);
-
+ScmCallTrace *Scm__CopyCallTraceQueue(ScmCallTrace *master);
 
 /* For BF and BT instructions, we check for #<undef>.
    This macro assumes to the local variable VM holding ScmVM*.
@@ -150,6 +217,48 @@ ScmCallTrace *Scm__MakeCallTraceQueue(u_long size);
     (SCM_FALSEP(obj) || (SCM_UNDEFINEDP(obj)&&Scm_VMUndefinedBool(vm)))
 SCM_EXTERN int Scm_VMUndefinedBool(ScmVM*); /* in boolean.c */
 
+/* For machine-level introspection */
+SCM_EXTERN ScmObj Scm__VMInsnAddress(int, _Bool);
+
+/*
+ * Thread Locals
+ *   We keep the definition private, so that we can extend it later.
+ */
+
+struct ScmThreadLocalRec {
+    SCM_HEADER;
+    ScmObj name;                /* for debugging. #f or symbol. */
+    ScmSize index;              /* slot index */
+    u_long generation;          /* to detect slot being reused */
+    ScmObj initialValue;
+    u_long flags;
+};
+
+/* Each VM has vector of thread-local values.  vm->threadLocals points to this.
+   The vector is extended on demand.
+   We might swap this to more sophisticated data structure than
+   a simple flat vector in future.
+ */
+typedef struct ScmVMThreadLocalSlotRec {
+    ScmObj value;
+    u_long generation;
+} ScmVMThreadLocalSlot;
+
+typedef struct ScmVMThreadLocalVectorRec {
+    ScmSize size;
+    ScmVMThreadLocalSlot *vector;
+} ScmVMThreadLocalVector;
+
+struct ScmVMThreadLocalTableRec {
+    ScmVMThreadLocalVector vs[2];
+};
+
+enum {
+      SCM_THREAD_LOCAL_VECTOR_INHERITABLE = 0,
+      SCM_THREAD_LOCAL_VECTOR_NONINHERITABLE = 1
+};
+
+SCM_EXTERN ScmVMThreadLocalTable *Scm__MakeVMThreadLocalTable(ScmVM *base);
 
 SCM_DECL_END
 
