@@ -1,7 +1,7 @@
 /*
  * writerP.h - Writer private API
  *
- *   Copyright (c) 2013-2021  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2013-2024  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -43,6 +43,9 @@ struct ScmWriteControlsRec {
     int printBase;              /* 2-36 */
     int printRadix;             /* boolean, #t to print radix for all numbers */
     int printPretty;            /* boolean, #t to use pretty printer */
+    int printIndent;            /* >=0 extra indent to be added after each
+                                   newline when pretty printing. */
+    int stringLength;           /* -1 for no limit.  Length of literal string */
 };
 
 SCM_CLASS_DECL(Scm_WriteControlsClass);
@@ -50,6 +53,22 @@ SCM_CLASS_DECL(Scm_WriteControlsClass);
 #define SCM_WRITE_CONTROLS(obj)   ((ScmWriteControls*)(obj))
 #define SCM_WRITE_CONTROLS_P(obj) SCM_XTYPEP(obj, SCM_CLASS_WRITE_CONTROLS)
 
+/*
+ * NB: Flip the following condition to use ellipsis (U+2026) to indicate
+ * truncated output, instead of three periods.  This is turned off because,
+ * on Windows environment, ellipsis may not be displayable depending on
+ * terminal settings.  To our astonishment, they are not fullly
+ * unicode-capable by default.
+ * We may probe the terminal at runtime to switch, but I don't want to
+ * clutter the code.  So for now, no ellipsis.
+ */
+#if 0
+#define SCM_WRITTEN_ELLIPSIS "\xe2\x80\xa6"
+#else
+#define SCM_WRITTEN_ELLIPSIS "..."
+#endif
+
+
 /* WriteContext and WriteState
 
    WriteContext affects write operation below the current subtree.
