@@ -1,7 +1,7 @@
 /*
  * hash.c - hash table implementation
  *
- *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -34,6 +34,7 @@
 #define LIBGAUCHE_BODY
 #include <math.h>
 #include "gauche.h"
+#include "gauche/priv/configP.h"
 #include "gauche/priv/atomicP.h"
 
 /*============================================================
@@ -325,6 +326,7 @@ static u_long internal_uvector_hash(ScmUVector *u, u_long salt, int portable)
         case SCM_UVECTOR_C128:
         case SCM_UVECTOR_RESERVED2:
         case SCM_UVECTOR_INVALID:
+        case SCM_UVECTOR_GENERIC:
             Scm_Panic("invalid uvector class.");
         }
         return 0;           /* dummy */
@@ -445,13 +447,13 @@ static u_long equal_hash_common(ScmObj obj, u_long salt, int portable)
    and object-hash definitions in libomega.scm. */
 static ScmPrimitiveParameter *current_recursive_hash;
 
-ScmObj Scm_CurrentRecursiveHash(ScmObj newval)
+/* This must be called after symbols and modules are initialized.
+   See Scm__InitModule(). */
+void Scm__InitCurrentRecursiveHash()
 {
-    if (newval == SCM_UNBOUND) {
-        return Scm_PrimitiveParameterRef(Scm_VM(), current_recursive_hash);
-    } else {
-        return Scm_PrimitiveParameterSet(Scm_VM(), current_recursive_hash, newval);
-    }
+    SCM_DEFINE(Scm_GaucheInternalModule(),
+               "%current-recursive-hash",
+               Scm_MakePrimitiveParameterSubr(current_recursive_hash));
 }
 
 /* 'Portable' general hash function.
@@ -732,43 +734,6 @@ static int string_cmp(const ScmHashCore *ht SCM_UNUSED, intptr_t k1, intptr_t k2
                        SCM_STRING_BODY_SIZE(b1)) == 0));
 }
 
-/*
- * Accessor function for multiword raw hashtable.
- * Key points to an array of N words.
- */
-#if 0                           /* not used yet */
-static u_long multiword_hash(const ScmHashCore *table, intptr_t key)
-{
-    ScmWord keysize = (ScmWord)table->data;
-    ScmWord *keyarray = (ScmWord*)key;
-    u_long h = 0, h1;
-    for (int i=0; i<keysize; i++) {
-        ADDRESS_HASH(h1, keyarray[i]);
-        h = COMBINE(h, h1);
-    }
-    return h;
-}
-#endif
-
-#if 0
-static Entry *multiword_access(ScmHashCore *table, intptr_t k, ScmDictOp op)
-{
-    u_long hashval, index;
-    ScmWord keysize = (ScmWord)table->data;
-
-    hashval = multiword_hash(table, k);
-    index = HASH2INDEX(table->numBuckets, table->numBucketsLog2, hashval);
-    Entry **buckets = (Entry**)table->buckets;
-
-    for (Entry *e = buckets[index], *p = NULL; e; p = e, e = e->next) {
-        if (memcmp((void*)k, (void*)e->key, keysize*sizeof(ScmWord)) == 0)
-            FOUND(table, op, e, p, index);
-    }
-    NOTFOUND(table, op, k, hashval, index);
-}
-#endif
-
-
 /*
  * Accessor function for general case
  *    (hashfn and cmpfn are given by user)
@@ -1236,15 +1201,6 @@ ScmHashEntry *Scm__HashIterNextCompat(ScmHashIter *iter)
     return (ScmHashEntry*)e;
 }
 
-#if 0
-ScmObj Scm_MakeHashTableMultiWord(int keysize, int initsize)
-{
-    return make_hash_table(SCM_CLASS_HASH_TABLE, SCM_HASH_MULTIWORD,
-                           multiword_access, multiword_hash,
-                           NULL, initsize, (void*)SCM_WORD(keysize));
-}
-#endif
-
 /* Legacy constructor.  DEPRECATED.  Will go away soon. */
 ScmObj Scm_MakeHashTable(ScmHashProc *hashfn,
                          ScmHashCompareProc *cmpfn SCM_UNUSED,
@@ -1259,14 +1215,7 @@ ScmObj Scm_MakeHashTable(ScmHashProc *hashfn,
     } else if (hashfn == (ScmHashProc*)SCM_HASH_STRING) {
         return Scm_MakeHashTableSimple(SCM_HASH_STRING, initSize);
     }
-#if 0
-    else {
-        return Scm_MakeHashTableFull(SCM_CLASS_HASH_TABLE, SCM_HASH_GENERAL,
-                                     hashfn, cmpfn, initSize, NULL);
-    }
-#else
     return SCM_UNDEFINED;
-#endif
 }
 #endif /*GAUCHE_API_VERSION < 1000*/
 
