@@ -1,7 +1,7 @@
 ;;;
 ;;; libalpha.scm - Procedures needed by other lib*.scm
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -37,10 +37,19 @@
 (select-module gauche.internal)
 
 (inline-stub
- (declcode (.include "gauche/vminsn.h"
-                     "gauche/priv/glocP.h"
-                     "gauche/priv/macroP.h"
-                     "gauche/priv/procP.h")))
+ (.include "gauche/priv/configP.h"
+           "gauche/vminsn.h"
+           "gauche/vm.h"
+           "gauche/priv/glocP.h"
+           "gauche/priv/macroP.h"
+           "gauche/priv/procP.h"
+           "gauche/priv/vmP.h"))
+
+;; This is referred from the compiled code of toplevel definitions, so
+;; we need it before anything else.
+(select-module gauche.internal)
+(define-cproc %expression-name-mark-key ()
+  (return (Scm__GetDenvKey SCM_DENV_KEY_EXPRESSION_NAME)))
 
 ;;;
 ;;;  errors
@@ -55,7 +64,7 @@
       (loop (cddr args) (list* (cadr args) (car args) keys))
       (values (reverse! keys) args))))
 
-(define (%make-error-message msg args) ; srfi-23 style message
+(define (%make-error-message msg args) ; SRFI-23 style message
   (let1 p (open-output-string)
     (display msg p)
     (dolist [obj args] (display " " p) (write/ss obj p))
@@ -102,7 +111,7 @@
 (define-cproc make-vector (k::<fixnum> :optional fill) Scm_MakeVector)
 
 ;;;
-;;;  srfi-17 support
+;;;  SRFI-17 support
 ;;;
 
 ;; some libraries needs to set the setter procedures during initialization,
@@ -162,115 +171,166 @@
  ;;
  ;;  (case-lambda [(x y) a] [(x . y) b]
  ;;  =>
- ;;  #( (lambda (x . y) c)         min-reqargs == 1
+ ;;  #( (lambda (x . y) b)         min-reqargs == 1
  ;;     (lambda (x y) a)
- ;;     (lambda (x . y) c))
+ ;;     (lambda (x . y) b))
  ;;
  ;; In the last example, 2 argument case matches the first procedure,
  ;; for it has precedence.
 
- "typedef struct case_lambda_packet_rec {
-    ScmVector *dispatch_vector;
-    int max_optargs;
-    int min_reqargs;
- } case_lambda_packet;"
+ (define-ctype case_lambda_packet
+   :: (.struct case_lambda_packet_rec
+               (dispatch_vector::ScmVector*
+                max_optargs::int
+                min_reqargs::int)))
 
  (define-cfn case-lambda-dispatch (args::ScmObj* nargs::int data::void*):static
    (let* ([d::case_lambda_packet* (cast case_lambda_packet* data)]
-          [rarg (aref args (- nargs 1))])
-     (SCM_ASSERT (> nargs (-> d min_reqargs)))
-     (SCM_ASSERT (<= nargs (+ (-> d min_reqargs) (-> d max_optargs) 1)))
-     ;; NB: Since this is variable-arg subr, rarg is always a list.
-     ;; If nargs <= d->max_optargs + d->min_reqargs, rargs should always
-     ;; be '().  See vmcall.c for how the arguments are folded.
-     (let* ([p (SCM_VECTOR_ELEMENT (-> d dispatch_vector)
-                                   (- nargs (-> d min_reqargs) 1))])
-       (when (SCM_FALSEP p)
-         (Scm_Error "wrong number of arguments to case lambda: %S"
-                    (Scm_ArrayToListWithTail args (- nargs 1)
-                                             (aref args (- nargs 1)))))
-       (if (SCM_NULLP rarg)
-         (case (- nargs 1)
-           [(0) (return (Scm_VMApply0 p))]
-           [(1) (return (Scm_VMApply1 p (aref args 0)))]
-           [(2) (return (Scm_VMApply2 p (aref args 0) (aref args 1)))]
-           [(3) (return (Scm_VMApply3 p (aref args 0) (aref args 1)
-                                      (aref args 2)))]
-           [(4) (return (Scm_VMApply4 p (aref args 0) (aref args 1)
-                                      (aref args 2) (aref args 3)))]
-           [else (return (Scm_VMApply p (Scm_ArrayToList args (- nargs 1))))])
-         (return
-          (Scm_VMApply p (Scm_ArrayToListWithTail args (- nargs 1) rarg))))
-       )))
+          [rarg (aref args (- nargs 1))]
+          [_ (SCM_ASSERT (> nargs (-> d min_reqargs)))]
+          [_ (SCM_ASSERT (<= nargs (+ (-> d min_reqargs) (-> d max_optargs) 1)))]
+          ;; NB: Since this is variable-arg subr, rarg is always a list.
+          ;; If nargs <= d->max_optargs + d->min_reqargs, rargs should always
+          ;; be '().  See vmcall.c for how the arguments are folded.
+          [p (SCM_VECTOR_ELEMENT (-> d dispatch_vector)
+                                 (- nargs (-> d min_reqargs) 1))])
+     (when (SCM_FALSEP p)
+       (Scm_Error "wrong number of arguments to case lambda: %S"
+                  (Scm_ArrayToListWithTail args (- nargs 1)
+                                           (aref args (- nargs 1)))))
+     (if (SCM_NULLP rarg)
+       (case (- nargs 1)
+         [(0) (return (Scm_VMApply0 p))]
+         [(1) (return (Scm_VMApply1 p (aref args 0)))]
+         [(2) (return (Scm_VMApply2 p (aref args 0) (aref args 1)))]
+         [(3) (return (Scm_VMApply3 p (aref args 0) (aref args 1)
+                                    (aref args 2)))]
+         [(4) (return (Scm_VMApply4 p (aref args 0) (aref args 1)
+                                    (aref args 2) (aref args 3)))]
+         [else (return (Scm_VMApply p (Scm_ArrayToList args (- nargs 1))))])
+       (return
+        (Scm_VMApply p (Scm_ArrayToListWithTail args (- nargs 1) rarg))))
+     ))
 
- ;; NB: We keep dispatch-vector in the procedure info, so that disasm
- ;; and other inspection routines can find it.  The structure of
- ;; the procedure info here is tentative; it may be changed later.
- ;; Routines that depends on this info must be aware of that.
- (define-cproc make-case-lambda-dispatcher (dispatch-vector::<vector>
-                                            min-reqargs::<int>
-                                            name)
-   (let* ([max-optargs::int (SCM_VECTOR_SIZE dispatch-vector)]
+ ;; A case-lambda procedure has special inliner procedure in its 'inliner' slot,
+ ;; which is made by pass1/make-case-labmda-inliner.   There's a catch,
+ ;; though.  Case-lambda procedures are made and called during booting,
+ ;; but pass1/make-case-labmda-inliner won't be available until the late
+ ;; stage of booting when the compiler is initialized.  So, if we don't
+ ;; have pass1/make-case-labmda-inliner yet, we assign
+ ;; delayed-case-lambda-inliner to delay actual calling.
+ (define-cfn delayed-case-lambda-inliner (args::ScmObj* nargs::int data::void*)
+   :static
+   (let* ([clambda-proc (SCM_OBJ data)]
+          [make-inliner
+           (Scm_GlobalVariableRef (Scm_GaucheInternalModule)
+                                  (SCM_SYMBOL 'pass1/make-case-lambda-inliner)
+                                  0)])
+     (when (SCM_UNBOUNDP make-inliner)
+       (return SCM_UNDEFINED))          ;not fully booted
+     (SCM_ASSERT (== nargs 2))
+     (let* ([src (aref args 0)]
+            [arg-iforms (aref args 1)]
+            [inliner (Scm_ApplyRec1 make-inliner clambda-proc)])
+       (SCM_ASSERT (SCM_PROCEDUREP inliner))
+       (set! (SCM_PROCEDURE_INLINER clambda-proc) inliner)
+       (return (Scm_VMApply2 inliner src arg-iforms)))))
+
+ ;; The compiler expands case-lambda to a call of make-case-lambda.
+ ;; It may be called during initialization, so we need it here.
+ ;; TRANSIENT: The third argument used to be a list of formals of the closures.
+ ;; We no longer need it, for we extract info from closures.  It is left for
+ ;; the backward compatibility until we switch ABI.
+ (define-cproc make-case-lambda (minarg::<int>
+                                 maxarg::<int>
+                                 _
+                                 closures
+                                 :optional (name #f))
+   (let* ([max-optargs::int (+ (- maxarg minarg) 2)]
+          [v-closures (Scm_MakeVector max-optargs SCM_FALSE)]
+          [num-closures::int 0]
+          [v-formals]
           [packet::case_lambda_packet* (SCM_NEW case_lambda_packet)])
-     (set! (-> packet min_reqargs) min-reqargs
+     (for-each
+      (lambda (c)
+        (let* ([req::int (SCM_PROCEDURE_REQUIRED c)]
+               [opt::int (SCM_PROCEDURE_OPTIONAL c)]
+               [n::int minarg])
+          (for (() (<= n (+ maxarg 1)) (post++ n))
+            (when (and (or (== n req)
+                           (and (> n req) (> opt 0)))
+                       (SCM_FALSEP (SCM_VECTOR_ELEMENT v-closures (- n minarg))))
+              (set! (SCM_VECTOR_ELEMENT v-closures (- n minarg)) c)
+              (pre++ num-closures)))))
+      closures)
+     ;; v-formals is #(<formals> ...)).  it is kept in the procedure-info
+     ;; slot.  It is only for information, the program should not count on
+     ;; its content.
+     ;; NB: We need a standard way to extract formals from a closure. Here
+     ;; I take cdr of each closure's procedure-info, but it's not guaranteed
+     ;; that they always have the formals.
+     (set! v-formals (Scm_MakeVector num-closures SCM_FALSE))
+     (let* ([i::int 0])
+       (dotimes [n max-optargs]
+         (let* ([c (SCM_VECTOR_ELEMENT v-closures n)])
+           (when (and (SCM_PROCEDUREP c)
+                      (SCM_PAIRP (SCM_PROCEDURE_INFO c)))
+             (set! (SCM_VECTOR_ELEMENT v-formals i)
+                   (SCM_CDR (SCM_PROCEDURE_INFO c)))
+             (post++ i)))))
+     (set! (-> packet min_reqargs) minarg
            (-> packet max_optargs) max-optargs
-           (-> packet dispatch_vector) dispatch-vector)
-     (return (Scm_MakeSubr case_lambda_dispatch packet
-                           min_reqargs max_optargs
-                           (SCM_LIST3 (?: (SCM_FALSEP name)
-                                         'case-lambda-dispatcher
-                                         name)
-                                      (SCM_MAKE_INT min-reqargs)
-                                      (SCM_OBJ dispatch-vector))))))
+           (-> packet dispatch_vector) (SCM_VECTOR v-closures))
+     (let* ([r (Scm_MakeSubr case_lambda_dispatch packet
+                             minarg max_optargs
+                             (SCM_LIST2 (?: (SCM_FALSEP name)
+                                            'case-lambda-dispatcher
+                                            name)
+                                        v-formals))]
+            [make-inliner
+             (Scm_GlobalVariableRef (Scm_GaucheInternalModule)
+                                    (SCM_SYMBOL 'pass1/make-case-lambda-inliner)
+                                    0)])
+       (if (SCM_UNBOUNDP make-inliner)
+         (set! (SCM_PROCEDURE_INLINER r)
+               (Scm_MakeSubr delayed-case-lambda-inliner
+                             (cast void* r) 2 0
+                             'delayed-case-lambda-inliner))
+         (set! (SCM_PROCEDURE_INLINER r)
+               (Scm_ApplyRec1 make-inliner r)))
+       (return r))))
+
+ ;; Returns a plist of case-lambda info if OBJ is a case lambda closure,
+ ;; otherwise #f.
+ (define-cproc %case-lambda-info (obj)
+   (if (and (SCM_SUBRP obj)
+            (== (SCM_SUBR_FUNC obj) case_lambda_dispatch))
+     (let* ([packet::case_lambda_packet* (cast case_lambda_packet*
+                                               (SCM_SUBR_DATA obj))])
+       (return (list ':min-reqargs
+                     (SCM_MAKE_INT (-> packet min_reqargs))
+                     ':max-optargs
+                     (SCM_MAKE_INT (-> packet max_optargs))
+                     ':dispatch-vector
+                     (SCM_OBJ (-> packet dispatch_vector)))))
+     (return SCM_FALSE)))
  )
 
-;; The compiler expands case-lambda to a call of make-case-lambda.
-;; It may be called during initialization, so we need it here.
-(define-in-module gauche.internal (make-case-lambda minarg maxarg
-                                                    formals closures
-                                                    :optional (name #f))
-  (define (fill-dispatch-vector! v formals closure)
-    (define (%set n)
-      (let1 i (- n minarg)
-        (unless (vector-ref v i) (vector-set! v i closure))))
-    (let loop ([formals formals] [n 0])
-      (cond [(> n (+ maxarg 1))]
-            [(null? formals) (%set n)]
-            [(pair? formals) (loop (cdr formals) (+ n 1))]
-            [else (%set n) (loop formals (+ n 1))])))
-
-  (let1 v (make-vector (+ (- maxarg minarg) 2) #f)
-    ;; Avoid using for-each, since the definition of for-each itself
-    ;; nees to call make-case-lambda.
-    (let loop ([fs formals] [cs closures])
-      (unless (null? fs)
-        (fill-dispatch-vector! v (car fs) (car cs))
-        (loop (cdr fs) (cdr cs))))
-    ((with-module gauche.internal make-case-lambda-dispatcher) v minarg name)))
-
 ;; Returns ((<required args> <optional arg> <procedure>) ...)
-;; This also depends on the structure of dispatch vector info.
-;; Programs that needs to deal with case-lambda should use this procedure
-;; instead of directly interpret dispatch vector info.
-(define-in-module gauche (case-lambda-info proc)
-  (and (subr? proc)
-       (let1 info (procedure-info proc)
-         (and (pair? info)
-              (pair? (cdr info))
-              (integer? (cadr info))
-              (pair? (cddr info))
-              (vector? (caddr info))
-              (let* ([min-args (cadr info)]
-                     [dispatch-vec (caddr info)]
-                     [len (vector-length dispatch-vec)])
-                (let loop ([r '()] [i 0])
-                  (if (= i (- len 1))
-                    (if-let1 optproc (vector-ref dispatch-vec i)
-                      (reverse r `((,(+ min-args i -1) #t ,optproc)))
-                      (reverse r))
-                    (if-let1 proc (vector-ref dispatch-vec i)
-                      (loop (cons `(,(+ min-args i) #f ,proc) r) (+ i 1))
-                      (loop r (+ i 1))))))))))
+;; for easier introspection.
+(define-in-module gauche (case-lambda-decompose proc)
+  (and-let1 info (%case-lambda-info proc)
+    (let ([min-args (get-keyword :min-reqargs info)]
+          [len (get-keyword :max-optargs info)]
+          [dispatch-vec (get-keyword :dispatch-vector info)])
+      (let loop ([r '()] [i 0])
+        (if (= i (- len 1))
+          (if-let1 optproc (vector-ref dispatch-vec i)
+            (reverse r `((,(+ min-args i -1) #t ,optproc)))
+            (reverse r))
+          (if-let1 proc (vector-ref dispatch-vec i)
+            (loop (cons `(,(+ min-args i) #f ,proc) r) (+ i 1))
+            (loop r (+ i 1))))))))
 
 ;;;
 ;;; Transfer binding to other module
