@@ -1,7 +1,7 @@
 ;;;
 ;;; libchar.scm - builtin character procedures
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -34,10 +34,11 @@
 (select-module gauche.internal)
 
 (inline-stub
- (declcode (.include <gauche/vminsn.h>
-                     <gauche/char_attr.h>
-                     <gauche/priv/charP.h>
-                     <ctype.h>))
+ (.include "gauche/priv/configP.h"
+           "gauche/vminsn.h"
+           "gauche/char_attr.h"
+           "gauche/priv/charP.h"
+           <ctype.h>)
  )
 
 ;;;
