@@ -1,7 +1,7 @@
 ;;;
 ;;; libcmp.scm - compare and sort
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -32,16 +32,18 @@
 ;;;
 
 (select-module gauche.internal)
+(inline-stub
+ (.include "gauche/priv/configP.h"))
 
 ;;;
-;;; Comparator (a la srfi-114/128)
+;;; Comparator (a la SRFI-114/128)
 ;;;
 
-;; Our built-in comparator is compatible to *both* srfi-114 and srfi-128;
+;; Our built-in comparator is compatible to *both* SRFI-114 and SRFI-128;
 ;; we automatically "fill-in" the difference (comparison-proc vs ordering-pred)
 ;; as needed.  The only incompatibility is the constructor---both srfi
 ;; uses the same name but the mearning of arguments differ.  We provide
-;; srfi-114 constructor under different name (make-comparator/compare).
+;; SRFI-114 constructor under different name (make-comparator/compare).
 ;;
 ;; Because of this on-demand filling, the internal accessors are a
 ;; bit complicated; it is justified by the simple api with low overhead
@@ -87,7 +89,7 @@
   (cond [(or (eq? hash #f) (applicable? hash <bottom>)) hash]
         [else (error "make-comparator needs a procedure or #f as hash, but got:" hash)]))
 
-;; API - srfi-114 constructor
+;; API - SRFI-114 constructor
 (define-in-module gauche (make-comparator/compare type-test equality-test
                                                   comparison-proc hash
                                                   :optional (name #f))
@@ -106,7 +108,7 @@
                       (eq? equality-test #t)
                       #f)))
 
-;; API - srfi-128 constructor
+;; API - SRFI-128 constructor
 (define-in-module gauche (make-comparator type-test equality-test
                                           ordering-pred hash
                                           :optional (name #f))
@@ -228,6 +230,21 @@
 ;;;   so we fiddle with C.
 ;;;
 
+;; Common routine
+;; TODO: Ideally, we want to use trampoline to call typeFn so that
+;; we won't recurse to VM loop.
+(inline-stub
+ (define-cfn cmpr-typecheck (c::ScmComparator* a b rest) ::void :static
+   (when (SCM_FALSEP (Scm_ApplyRec1 (-> c typeFn) a))
+     (Scm_Error "Object is not suitable for a comparator %S: %S" c a))
+   (when (SCM_FALSEP (Scm_ApplyRec1 (-> c typeFn) b))
+     (Scm_Error "Object is not suitable for a comparator %S: %S" c b))
+   (for-each
+    (lambda (z)
+      (unless (Scm_ApplyRec1 (-> c typeFn) z)
+        (Scm_Error "Object is not suitable for a comparator %S: %S" c z)))
+    rest)))
+
 ;;; =?
 (inline-stub
  (define-cfn cmpr-eq (c::ScmComparator* a b more) :static
@@ -240,6 +257,7 @@
      (return (Scm_VMApply2 (-> c eqFn) a b))))
 
  (define-cproc =? (c::<comparator> a b :rest more)
+   (cmpr-typecheck c a b more)
    (return (cmpr-eq c a b more))))
 
 ;;; <?
@@ -265,6 +283,7 @@
        (return (SCM_MAKE_BOOL (< (SCM_INT_VALUE r) 0))))))
 
  (define-cproc <? (c::<comparator> a b :rest more)
+   (cmpr-typecheck c a b more)
    (cond [(logand (-> c flags) SCM_COMPARATOR_SRFI_128)
           (when (SCM_FALSEP (-> c orderFn))
             (Scm_Error "%S doesn't have ordering predicate" (SCM_OBJ c)))
@@ -305,6 +324,7 @@
        (return (SCM_MAKE_BOOL (<= (SCM_INT_VALUE r) 0))))))
 
  (define-cproc <=? (c::<comparator> a b :rest more)
+   (cmpr-typecheck c a b more)
    (cond [(logand (-> c flags) SCM_COMPARATOR_SRFI_128)
           (when (SCM_FALSEP (-> c orderFn))
             (Scm_Error "%S doesn't have ordering predicate" (SCM_OBJ c)))
@@ -347,6 +367,7 @@
        (return (SCM_MAKE_BOOL (> (SCM_INT_VALUE r) 0))))))
 
  (define-cproc >? (c::<comparator> a b :rest more)
+   (cmpr-typecheck c a b more)
    (cond [(logand (-> c flags) SCM_COMPARATOR_SRFI_128)
           (when (SCM_FALSEP (-> c orderFn))
             (Scm_Error "%S doesn't have ordering predicate" (SCM_OBJ c)))
@@ -381,6 +402,7 @@
        (return (SCM_MAKE_BOOL (>= (SCM_INT_VALUE r) 0))))))
 
  (define-cproc >=? (c::<comparator> a b :rest more)
+   (cmpr-typecheck c a b more)
    (cond [(logand (-> c flags) SCM_COMPARATOR_SRFI_128)
           (when (SCM_FALSEP (-> c orderFn))
             (Scm_Error "%S doesn't have ordering predicate" (SCM_OBJ c)))
