@@ -1,7 +1,7 @@
 ;;;
 ;;; libdict.scm - dictionary-like data structures (hashtables etc.)
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -38,6 +38,8 @@
 ;;;
 
 (inline-stub
+ (.include "gauche/priv/configP.h")
+
  (define-cise-stmt dict-check-entry
    [(_ dict key expr)
     `(when ,expr (Scm_Error "%S doesn't have an entry for key %S" ,dict ,key))])
@@ -128,9 +130,6 @@
        )])
  )
 
-(select-module gauche.internal)
-(define-cproc %current-recursive-hash (:optional obj) Scm_CurrentRecursiveHash)
-
 (select-module gauche)
 (define-cproc hash-salt () ::<fixnum> Scm_HashSaltRef)
 
@@ -223,7 +222,7 @@
               comparator))
      ;; If comparator's equality predicate is eq?/eqv?, we ignore
      ;; the its hash function and force using eq-/eqv-hash,
-     ;; as permitted in srfi-125.  It allows object that doesn't have
+     ;; as permitted in SRFI-125.  It allows object that doesn't have
      ;; custom hash method can still be used with hashtables based on
      ;; (make-eq[v]-comparator).
      ;; https://github.com/shirok/Gauche/issues/708
@@ -253,12 +252,11 @@
 (select-module gauche.internal)
 (define-cproc %hash-table-comparator-int (hash::<hash-table>)
   (if (== (Scm_HashTableType hash) SCM_HASH_GENERAL)
-    (begin
-      (let* ([r (SCM_OBJ (-> (SCM_HASH_TABLE_CORE hash) data))])
-        (unless (SCM_COMPARATORP r)
-          (Scm_Error "Got some weird hashtable - possibly internal bug: %S"
-                     hash))
-        (return r)))
+    (let* ([r (SCM_OBJ (-> (SCM_HASH_TABLE_CORE hash) data))])
+      (unless (SCM_COMPARATORP r)
+        (Scm_Error "Got some weird hashtable - possibly internal bug: %S"
+                   hash))
+      (return r))
     (return '#f)))
 (select-module gauche)
 (define (hash-table-comparator hash)
@@ -332,8 +330,8 @@
 
 (select-module gauche)
 (define-cproc hash-table-copy (ht::<hash-table> :optional _)
-  ;; we don't have immutable hash table, and ignore the optional 
-  ;; mutable? argument (srfi-125)
+  ;; we don't have immutable hash table, and ignore the optional
+  ;; mutable? argument (SRFI-125)
   (return (Scm_HashTableCopy ht)))
 (define-cproc hash-table-keys (ht::<hash-table>)   Scm_HashTableKeys)
 (define-cproc hash-table-values (ht::<hash-table>) Scm_HashTableValues)
@@ -360,7 +358,7 @@
 (define unique (cons #f #f))
 
 ;; vararg constructor 'hash-table' has conflicting API between
-;; legacy Gauche and R7RS scheme.hash-table (srfi-125).
+;; legacy Gauche and R7RS scheme.hash-table (SRFI-125).
 ;; R7RS's API is more convenient to use, so we eventually
 ;; switch to it.  Meantime, we provide original hash-table
 ;; with a different name, hash-table-from-pairs, and R7RS
@@ -554,7 +552,7 @@
 
 ;; We delegate most hash calculation to the built-in default-hash.
 ;; These type-specific hash functions are mostly
-;; for the compatibility of srfi-128.
+;; for the compatibility of SRFI-128.
 
 (define (boolean-hash obj)
   (assume-type obj <boolean>)
@@ -580,7 +578,7 @@
   (assume-type obj <number>)
   (default-hash obj))
 
-;; This is a placeholder to conform srfi-128.
+;; This is a placeholder to conform SRFI-128.
 (define (hash-bound) (greatest-fixnum))
 
 ;; Compare two hash-tables as sets.
@@ -636,15 +634,15 @@
  ;; If tree-map-cmp is called, core->data should contain a comparator.
  (define-cfn tree-map-cmp (core::ScmTreeCore* x::intptr_t y::intptr_t)
    ::int :static
-   (let* ([cmpr (SCM_OBJ (-> core data))])
-     (SCM_ASSERT (and cmpr (SCM_COMPARATORP cmpr)))
-     (let* ([r (Scm_ApplyRec2 (Scm_ComparatorComparisonProcedure
-                               (SCM_COMPARATOR cmpr))
-                              (SCM_OBJ x) (SCM_OBJ y))])
-       (unless (SCM_INTP r)
-         (Scm_Error "compare procedure of tree-map's comparator %S returned \
-                     non-integral value: %S" cmpr r))
-       (return (SCM_INT_VALUE r)))))
+   (let* ([cmpr (SCM_OBJ (-> core data))]
+          [_ (SCM_ASSERT (and cmpr (SCM_COMPARATORP cmpr)))]
+          [r (Scm_ApplyRec2 (Scm_ComparatorComparisonProcedure
+                             (SCM_COMPARATOR cmpr))
+                            (SCM_OBJ x) (SCM_OBJ y))])
+     (unless (SCM_INTP r)
+       (Scm_Error "compare procedure of tree-map's comparator %S returned \
+                   non-integral value: %S" cmpr r))
+     (return (SCM_INT_VALUE r))))
  )
 
 (define-cproc %make-tree-map (comparator)
