@@ -1,7 +1,7 @@
 ;;;
 ;;; libeval.scm - eval, load and related stuff
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -32,10 +32,13 @@
 ;;;
 
 (select-module gauche.internal)
+(use util.match)
 
 (inline-stub
- (declcode (.include <gauche/vminsn.h>
-                     <gauche/priv/readerP.h>)))
+ (.include "gauche/priv/configP.h"
+           "gauche/vminsn.h"
+           "gauche/priv/readerP.h"
+           "gauche/priv/vmP.h"))
 
 (declare (keep-private-macro autoload add-load-path))
 
@@ -78,7 +81,10 @@
     (let* ([path (car r)]
            [remaining-paths (cadr r)]
            [hooked? (pair? (cddr r))]
-           [opener (if hooked? (caddr r) open-input-file)]
+           [opener (if hooked?
+                     (caddr r)
+                     (cut open-input-file <>
+                          :encoding (gauche-character-encoding)))]
            [port (guard (e [else e]) (opener path))])
       (when main-script
         ;; record full path of the script
@@ -91,11 +97,13 @@
                 (if hooked? " (hooked) " "")))
       (if (not (input-port? port))
         (and error-if-not-found (raise port))
-        (load-from-port (if ignore-coding
-                          port
-                          (open-coding-aware-port port))
-                        :environment environment
-                        :paths remaining-paths)))))
+        (begin
+          (load-from-port (if ignore-coding
+                            port
+                            (open-coding-aware-port port))
+                          :environment environment
+                          :paths remaining-paths)
+          path)))))
 
 
 (select-module gauche.internal)
@@ -173,17 +181,17 @@
 
 ;; A few helper procedures
 (define-cproc %record-load-stat (path) ::<void>
-  (.if "defined(HAVE_GETTIMEOFDAY)"
-       (let* ([vm::ScmVM* (Scm_VM)])
-         (when (SCM_VM_RUNTIME_FLAG_IS_SET vm SCM_COLLECT_LOAD_STATS)
-           (let* ([t0::(struct timeval)])
-             (gettimeofday (& t0) NULL)
-             (let* ([t (Scm_MakeIntegerU (+ (* (ref t0 tv_sec) 1000000)
-                                            (ref t0 tv_usec)))])
-               (set! (ref (-> vm stat) loadStat)
-                     (Scm_Cons
-                      (?: (SCM_FALSEP path) t (Scm_Cons path t))
-                      (ref (-> vm stat) loadStat)))))))))
+  (.when "defined(HAVE_GETTIMEOFDAY)"
+    (let* ([vm::ScmVM* (Scm_VM)])
+      (when (SCM_VM_RUNTIME_FLAG_IS_SET vm SCM_COLLECT_LOAD_STATS)
+        (let* ([t0::(struct timeval)]
+               [_ (gettimeofday (& t0) NULL)]
+               [t (Scm_MakeIntegerU (+ (* (ref t0 tv_sec) 1000000)
+                                       (ref t0 tv_usec)))])
+          (set! (ref (-> vm stat) loadStat)
+                (Scm_Cons
+                 (?: (SCM_FALSEP path) t (Scm_Cons path t))
+                 (ref (-> vm stat) loadStat))))))))
 
 (define-cproc %new-read-context-for-load ()
   (let* ([ctx::ScmReadContext* (Scm_MakeReadContext NULL)])
@@ -206,7 +214,7 @@
     (let* ([base (sys-basename dsopath)]
            [stem (or (string-scan-right base #\. 'before) base)])
       (string-append "_Scm_Init_"
-                     ;; string-map is in srfi-13, so we roll our own.
+                     ;; string-map is in SRFI-13, so we roll our own.
                      ;; This looks awful, but we don't call this frequently.
                      (list->string
                       (map (^c (cond [(char-alphabetic? c) (char-downcase c)]
@@ -469,6 +477,8 @@
 ;;;
 ;;; Invoke other version of gosh
 ;;;   This is called when gosh gets -vVERSION option.
+;;;   On success, this never returns.
+;;;   On failure, returns a globpath that is tried to find the other versions.
 ;;;
 (select-module gauche.internal)
 (define (%invoke-other-version version args)
@@ -483,7 +493,11 @@
                   (string-append (gauche-architecture-directory) "/../../..")
                   :canonicalize #t)]
          [globpath (string-append prefix "/gauche-*/" version "/"
-                                  (gauche-architecture) "/gosh")]
+                                  (gauche-architecture) "/"
+                                  (cond-expand
+                                   ;; glob requires a file extension on Windows
+                                   [gauche.os.windows "gosh.exe"]
+                                   [else              "gosh"]))]
          [goshes (glob globpath)]
          ;; Remove -v option from args
          [args (let loop ([args (cdr args)]
@@ -492,23 +506,34 @@
                        [(equal? (car args) "-v") (loop (cddr args) r)]
                        [(#/^-v/ (car args)) (loop (cdr args) r)]
                        [else (loop (cdr args) (cons (car args) r))]))])
-    (unless (pair? goshes)
-      (exit 1 "No installed Gauche with version ~a under ~a.\n"
-            version prefix))
-    (sys-exec (car goshes) (cons (car goshes) args))))
+    (if (pair? goshes)
+      (cond-expand
+       [gauche.os.windows
+        ;; On windows, sys-exec (execvp) doesn't work like unix.
+        ;; Notably, exit status of the exec'ed process is lost.
+        ;; So we use sys-fork-and-exec (CreateProcess) and wait for the
+        ;; child process to exit.
+        (let1 pid (sys-fork-and-exec (car goshes) (cons (car goshes) args))
+          (receive (_ status) (sys-waitpid pid)
+            (cond
+             [(sys-wait-exited? status)
+              (exit (sys-wait-exit-status status))] ; return exit code
+             [(sys-wait-signaled? status)
+              (sys-kill (sys-getpid) (sis-wait-termsig status))]
+             [else (exit 70)])))        ;EX_SOFTWARE
+        ]
+       [else
+        (sys-exec (car goshes) (cons (car goshes) args)) ;never return
+        ])
+      prefix)))
 
 ;;;
 ;;; System termination
 ;;;
 
-(select-module gauche)
+(select-module gauche.internal)
 (define-cproc %exit (:optional (code 0)) ::<void>
-  (let* ([status::int 0])
-    (cond
-     [(SCM_EQ code SCM_TRUE)]  ; status == 0
-     [(SCM_INTP code) (set! status (SCM_INT_VALUE code))]
-     [else (set! status 70)])  ; EX_SOFTWARE
-    (Scm_Exit status)))
+  (Scm_Exit (Scm_ObjToExitCode code)))
 
 ;; API
 ;; exit handler.
@@ -628,7 +653,10 @@
    trace port maxdepth skip offset))
 
 (select-module gauche.internal)
-(define-cproc %vm-get-insn-offsets () Scm__VMInsnOffsets)
+(define-cproc %vm-get-insn-address (code::<int>
+                                    :optional (offset?::<boolean> #f))
+  ::<integer>
+  (return (Scm__VMInsnAddress code offset?)))
 
 ;; This is also called from C's Scm_ShowStackTrace
 ;; TRACE is what vm-get-stack-trace-lite returns.
@@ -646,14 +674,24 @@
                [else #f])]
       (unless (> skip 0)
         (format port "~3d  ~,,,,65:s\n" depth (unwrap-syntax (car trace)))
+        ;; Each source info consists of (<file> <line> <form>).
+        ;; The first <form> is the same as (car trace), so we omit it.
         (let1 sis (%source-info (car trace))
-          (if-let1 si (find (^[si] (and (car si) (cadr si))) (reverse sis))
-            (begin
-              (when (and (pair? (cddr si))
-                         (not (eq? (car trace) (caddr si))))
-                (format port "        expanded from ~,,,,60s\n" (caddr si)))
-              (format port "        at ~s:~d\n" (car si) (cadr si)))
-            (format port "        [unknown location]\n")))))))
+          (match sis
+            [((file line  _) . rest)
+             (when (and file line)
+               (format port "        at ~s:~d\n" file line))
+             (let loop ([sis rest])
+               (match sis
+                 [() #f]
+                 [((file line form) . rest)
+                  (format port "        expanded from ~,,,,57:s\n"
+                          (unwrap-syntax form))
+                  (when (and file line)
+                    (format port "        at ~s:~d\n" file line))
+                  (loop rest)]
+                 [(_ . rest) (loop rest)]))]
+            [_ #f]))))))
 
 ;; API
 (select-module gauche.internal)
