@@ -1,7 +1,7 @@
 ;;;
 ;;; libio.scm - builtin port and I/O procedures
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -34,12 +34,13 @@
 (select-module gauche.internal)
 
 (inline-stub
- (declcode (.include <gauche/vminsn.h>
-                     <gauche/exception.h>
-                     <gauche/priv/portP.h>
-                     <gauche/priv/writerP.h>
-                     <stdlib.h>
-                     <fcntl.h>)))
+ (.include "gauche/priv/configP.h"
+           "gauche/vminsn.h"
+           "gauche/exception.h"
+           "gauche/priv/portP.h"
+           "gauche/priv/writerP.h"
+           <stdlib.h>
+           <fcntl.h>))
 
 ;;;
 ;;; Ports
@@ -57,38 +58,25 @@
 (select-module gauche)
 (define-cproc port-closed? (obj::<port>) ::<boolean> SCM_PORT_CLOSED_P)
 
+;;
+;; Default coding
+;;
+
+(inline-stub
+ (define-cvar default-file-encoding::ScmPrimitiveParameter* :static)
+ (initcode
+  (set! default-file-encoding
+        (Scm_BindPrimitiveParameter (Scm_GaucheModule)
+                                    "default-file-encoding"
+                                    (Scm_CharEncodingName) 0)))
+ )
+
 ;;
 ;; Preexisting ports
 ;;
 
-(select-module scheme)
-
-(define-cproc current-input-port (:optional newport)
-  (cond [(SCM_IPORTP newport)
-         (return (Scm_SetCurrentInputPort (SCM_PORT newport)))]
-        [(not (SCM_UNBOUNDP newport))
-         (Scm_TypeError "current-input-port" "input port" newport)
-         (return SCM_UNDEFINED)]
-        [else (return (SCM_OBJ SCM_CURIN))]))
-
-(define-cproc current-output-port (:optional newport)
-  (cond [(SCM_OPORTP newport)
-         (return (Scm_SetCurrentOutputPort (SCM_PORT newport)))]
-        [(not (SCM_UNBOUNDP newport))
-         (Scm_TypeError "current-output-port" "output port" newport)
-         (return SCM_UNDEFINED)]
-        [else (return (SCM_OBJ SCM_CUROUT))]))
-
 (select-module gauche)
 
-(define-cproc current-error-port (:optional newport)
-  (cond
-   [(SCM_OPORTP newport) (return (Scm_SetCurrentErrorPort (SCM_PORT newport)))]
-   [(not (SCM_UNBOUNDP newport))
-    (Scm_TypeError "current-error-port" "output port" newport)
-    (return SCM_UNDEFINED)]
-   [else (return (SCM_OBJ SCM_CURERR))]))
-
 (define-cproc standard-input-port (:optional (p::<input-port>? #f))
   (return (?: p (Scm_SetStdin p) (Scm_Stdin))))
 (define-cproc standard-output-port (:optional (p::<output-port>? #f))
@@ -148,10 +136,19 @@
            port (Scm_BufferingMode mode (-> port direction) -1)))
   (return (Scm_GetPortBufferingModeAsKeyword port)))
 
-(define-cproc port-case-fold-set! (port::<port> flag::<boolean>) ::<void>
-  (if flag
-    (logior= (SCM_PORT_FLAGS port) SCM_PORT_CASE_FOLD)
-    (logand= (SCM_PORT_FLAGS port) (lognot SCM_PORT_CASE_FOLD))))
+(define-cproc port-link! (iport::<input-port> oport::<output-port>) ::<void>
+  Scm_LinkPorts)
+
+(define-cproc port-unlink! (port::<port>) ::<void> Scm_UnlinkPorts)
+
+(select-module gauche.internal)
+(define-cproc port-case-fold (port::<port>) ::<boolean>
+  (setter (port::<port> flag::<boolean>) ::<void>
+          (if flag
+            (logior= (SCM_PORT_FLAGS port) SCM_PORT_CASE_FOLD)
+            (logand= (SCM_PORT_FLAGS port) (lognot SCM_PORT_CASE_FOLD))))
+  (return (logand (SCM_PORT_FLAGS port) SCM_PORT_CASE_FOLD)))
+
 
 ;;
 ;; Open and close
@@ -168,9 +165,9 @@
  ;; NB: On MinGW, if we try to create a file and a directory with the
  ;; same name exists, open(2) throws EACCES.  Weird, eh?  We don't want
  ;; to catch EACCES on other platforms, hence this dirty trick.
- (if "defined(GAUCHE_WINDOWS)"
-   "#define DIRECTORY_GETS_IN_WAY(x) ((x)==EACCES)"
-   "#define DIRECTORY_GETS_IN_WAY(x) FALSE")
+ (.if (defined GAUCHE_WINDOWS)
+   (.define DIRECTORY_GETS_IN_WAY (x) (== x EACCES))
+   (.define DIRECTORY_GETS_IN_WAY (x) FALSE))
 
  ;; Some cise macros for common idioms
  (define-cise-expr %open/allow-noexist?
@@ -204,10 +201,10 @@
                 (SCM_EQ element-type ':binary))
       (Scm_Error "bad element-type argument: either :character or :binary \
                   expected, but got %S" element-type))
-    (.if "defined(O_BINARY) && defined(O_TEXT)"
-         (if (SCM_EQ element-type ':character)
-           (logior= flags O_TEXT)
-           (logior= flags O_BINARY)))
+    (.if (and (defined O_BINARY) (defined O_TEXT))
+      (if (SCM_EQ element-type ':character)
+        (logior= flags O_TEXT)
+        (logior= flags O_BINARY)))
     (let* ([bufmode::int (Scm_BufferingMode buffering SCM_PORT_INPUT
                                             SCM_PORT_BUFFER_FULL)]
            [o (Scm_OpenFilePort (Scm_GetStringConst path)
@@ -230,10 +227,10 @@
                 (SCM_EQ element-type ':binary))
       (Scm_Error "bad element-type argument: either :character or :binary \
                   expected, but got %S" element-type))
-    (.if "defined(O_BINARY) && defined(O_TEXT)"
-         (if (SCM_EQ element-type ':character)
-           (logior= flags O_TEXT)
-           (logior= flags O_BINARY)))
+    (.if (and (defined O_BINARY) (defined O_TEXT))
+      (if (SCM_EQ element-type ':character)
+        (logior= flags O_TEXT)
+        (logior= flags O_BINARY)))
     ;; check if-exists flag
     (cond
      [(SCM_EQ if-exists ':append) (logior= flags O_APPEND)]
@@ -373,7 +370,7 @@
     (return (Scm_MakeBufferedPort SCM_CLASS_PORT SCM_FALSE SCM_PORT_OUTPUT
                                   TRUE (& bufrec)))))
 
-;; String ports (srfi-6)
+;; String ports (SRFI-6)
 ;;   By default, string ports are named as "(input string port)" and
 ;;   "(output string port)", which aren't very informative.  The caller
 ;;   can specify alternative name with :name keyword argument. NB: Currently,
@@ -412,7 +409,7 @@
 ;; Miscellaneous
 ;;
 
-;; srfi-191
+;; SRFI-191
 (define-cproc port-has-port-position? (port::<port>) ::<boolean>
   (return (Scm_PortPositionable port FALSE)))
 (define-cproc port-has-set-port-position!? (port::<port>) ::<boolean>
@@ -526,6 +523,12 @@
     (set! (%port-write-state port) #f)
     (%port-unlock! port)))
 
+;; For the time being, we keep this in gauche.internal, for we don't know
+;; how complete column count feature can be.
+(define-cproc port-column (port::<port>) ::<fixnum>
+  (return (Scm_PortColumn port)))
+
+
 ;;;
 ;;; Input
 ;;;
@@ -544,7 +547,7 @@
 (define-cproc peek-char (:optional (port::<input-port> (current-input-port)))
   (inliner PEEK-CHAR)
   (let* ([ch::ScmChar (Scm_Peekc port)])
-    (return (?: (== ch SCM_CHAR_INVALID) SCM_EOF (SCM_MAKE_CHAR ch)))))
+    (return (?: (== ch EOF) SCM_EOF (SCM_MAKE_CHAR ch)))))
 
 (define-cproc eof-object? (obj) ::<boolean> :fast-flonum
   (inliner EOFP) SCM_EOFP)
@@ -599,8 +602,7 @@
 (define (write-string string :optional (port (current-output-port))
                                        (start 0)
                                        (end -1))
-  (display ((with-module gauche.internal %maybe-substring) string start end)
-           port))
+  (display (opt-substring string start end) port))
 
 ;; Consume trailing whiespaces up to (including) first EOL.
 ;; This is mainly intended for interactive REPL,
@@ -625,6 +627,9 @@
                  (read-byte port))]
               [(eqv? b 10) (read-byte port)])))))        ;lf
 
+(define-cproc char-word-constituent? (ch::<char>) ::<boolean>
+  (return (Scm_CharWordConstituent ch 0)))
+
 ;; DEPRECATED - read-uvector should be used
 (define-cproc read-block (bytes::<fixnum>
                           :optional (port::<input-port> (current-input-port)))
@@ -682,7 +687,7 @@
 (define-cproc %port-ungotten-bytes (port::<input-port>)
   Scm_UngottenBytes)
 
-;; Read time constructor (srfi-10)
+;; Read time constructor (SRFI-10)
 (select-module gauche)
 
 (define-cproc define-reader-ctor (symbol proc :optional (finisher #f))
@@ -720,7 +725,7 @@
     (Scm_Error "read reference hasn't been resolved"))
   (return (-> ref value)))
 
-;; srfi-38
+;; SRFI-38
 (define-in-module gauche read-with-shared-structure read)
 (define-in-module gauche read/ss read)
 
@@ -784,6 +789,12 @@
 (define-cproc newline (:optional (port::<output-port> (current-output-port)))
   ::<void> (SCM_PUTC #\newline port))
 
+(define-cproc fresh-line (:optional (port::<output-port> (current-output-port)))
+  ::<boolean>
+  (if (== (Scm_PortColumn port) 0)
+    (return FALSE)
+    (begin (SCM_PUTC #\newline port) (return TRUE))))
+
 (define-cproc write-char
   (ch::<char> :optional (port::<output-port> (current-output-port)))
   ::<void> (inliner WRITE-CHAR) (SCM_PUTC ch port))
@@ -838,6 +849,7 @@
         (cond
          [(symbol? obj)] ; uninterned symbols
          [(string? obj)]
+         [(uvector? obj)]
          [(pair? obj)
           (%write-walk-rec (car obj) port tab)
           (%write-walk-rec (cdr obj) port tab)]
@@ -857,7 +869,7 @@
 
 (select-module gauche.internal)
 
-;; srfi-38
+;; SRFI-38
 (define-in-module gauche (write-with-shared-structure obj :optional (port (current-output-port)))
   (write* obj port))
 (define-in-module gauche write/ss write-with-shared-structure)
@@ -866,7 +878,7 @@
 
 ;;
 ;; Write controls
-;;  For performance reasons, we don't make them a srfi-39 parameters.
+;;  For performance reasons, we don't make them a SRFI-39 parameters.
 ;;
 
 (select-module gauche)
@@ -909,13 +921,25 @@
     (radix  :type <boolean> :c-name "printRadix"
             :setter "obj->printRadix = !SCM_FALSEP(value);")
     (pretty :type <boolean> :c-name "printPretty"
-            :setter "obj->printPretty = !SCM_FALSEP(value);"))
+            :setter "obj->printPretty = !SCM_FALSEP(value);")
+    (indent :type <int>     :c-name "printIndent"
+            :setter "if (SCM_INTP(value) && SCM_INT_VALUE(value) >= 0) \
+                       obj->printIndent = SCM_INT_VALUE(value); \
+                     else obj->printIndent = 0;")
+    (string-length :type <int> :c-name "stringLength"
+                   :getter "if (obj->stringLength < 0) return SCM_FALSE; \
+                            else return SCM_MAKE_INT(obj->stringLength);"
+                   :setter "if (SCM_INTP(value) && SCM_INT_VALUE(value) >= 0) \
+                              obj->stringLength = SCM_INT_VALUE(value); \
+                            else obj->stringLength = -1;")
+    )
    (allocator (c "write_controls_allocate")))
  ;; NB: Printer is defined in libobj.scm via write-object method
  )
 
 ;; TRANSIENT: The print-* keyword arguments for the backward compatibility
-(define (make-write-controls :key length level width base radix pretty
+(define (make-write-controls :key length level width base radix pretty indent
+                                  string-length
                                   print-length print-level print-width
                                   print-base print-radix print-pretty)
   (define (arg k k-alt) (if (undefined? k-alt) k k-alt))
@@ -925,7 +949,9 @@
     :width  (arg width  print-width)
     :base   (arg base   print-base)
     :radix  (arg radix  print-radix)
-    :pretty (arg pretty print-pretty)))
+    :pretty (arg pretty print-pretty)
+    :string-length string-length
+    :indent indent))
 
 ;; Returns fresh write-controls where the specified slot value is replaced
 ;; from the original WC.
@@ -933,7 +959,8 @@
 ;; we don't bother to create a copy.  This assumes we treat WC immutable.
 ;; (Maybe we should write this in C to avoid overhead.)
 ;; TRANSIENT: The print-* keyword arguments for the backward compatibility
-(define (write-controls-copy wc :key length level width base radix pretty
+(define (write-controls-copy wc :key length level width base radix pretty indent
+                                     string-length
                                      print-length print-level print-width
                                      print-base print-radix print-pretty)
   (let-syntax [(select
@@ -949,13 +976,19 @@
           [width  (select width  print-width)]
           [base   (select base   print-base)]
           [radix  (select radix  print-radix)]
-          [pretty (select pretty print-pretty)])
+          [pretty (select pretty print-pretty)]
+          [indent (if (undefined? indent) (slot-ref wc 'indent) indent)]
+          [string-length (if (undefined? string-length)
+                           (slot-ref wc 'string-length)
+                           string-length)])
       (if (and (eqv? length (slot-ref wc 'length))
                (eqv? level  (slot-ref wc 'level))
                (eqv? width  (slot-ref wc 'width))
                (eqv? base   (slot-ref wc 'base))
                (eqv? radix  (slot-ref wc 'radix))
-               (eqv? pretty (slot-ref wc 'pretty)))
+               (eqv? pretty (slot-ref wc 'pretty))
+               (eqv? indent (slot-ref wc 'indent))
+               (eqv? string-length (slot-ref wc 'string-length)))
         wc
         (make <write-controls>
           :length length
@@ -963,7 +996,9 @@
           :width  width
           :base   base
           :radix  radix
-          :pretty pretty)))))
+          :pretty pretty
+          :indent indent
+          :string-length string-length)))))
 
 ;;;
 ;;; With-something
@@ -975,8 +1010,9 @@
 ;; %open-{input|output}-file/conv are autoloaded.
 
 (define-in-module scheme (open-input-file filename . args)
-  (let1 e (get-keyword :encoding args #f)
-    (cond [(eq? e #f) (apply %open-input-file filename args)]
+  (let1 e (get-keyword :encoding args (default-file-encoding))
+    (cond [(eq? e (gauche-character-encoding))
+           (apply %open-input-file filename (delete-keyword :encoding args))]
           [(eq? e #t)                   ;using coding-aware port
            (and-let* ([p (apply %open-input-file filename
                                 (delete-keyword :encoding args))])
@@ -984,9 +1020,11 @@
           [else (apply %open-input-file/conv filename args)])))
 
 (define-in-module scheme (open-output-file filename . args)
-  (if (get-keyword :encoding args #f)
-    (apply %open-output-file/conv filename args)
-    (apply %open-output-file filename args)))
+  (let1 e (get-keyword :encoding args #f)
+    (if (and (not e)
+             (eq? (default-file-encoding) (gauche-character-encoding)))
+      (apply %open-output-file filename args)
+      (apply %open-output-file/conv filename args))))
 
 ;; R6RS call-with-port
 ;; Make sure to close PORT when proc returns or throws an error
@@ -1049,7 +1087,7 @@
 
 (define-in-module gauche (read-from-string string . args)
   (with-input-from-string
-      (if (null? args) string (apply %maybe-substring string args))
+      (if (null? args) string (apply opt-substring string args))
     read))
 
 ;; with-port
@@ -1090,17 +1128,17 @@
   (^[sym port ctx]
     (warn "Reading R6RS source file.  Note that Gauche is not R6RS compliant.")
     ;; TODO: we could do some adjustments, such as switching the semantics of
-    ;; '#,' from srfi-10 to r6rs 'unsyntax'.
+    ;; '#,' from SRFI-10 to r6rs 'unsyntax'.
     (values)))
 
 (define-reader-directive 'fold-case
   (^[sym port ctx]
-    (port-case-fold-set! port #t)
+    (set! ((with-module gauche.internal port-case-fold) port) #t)
     (values)))
 
 (define-reader-directive 'no-fold-case
   (^[sym port ctx]
-    (port-case-fold-set! port #f)
+    (set! ((with-module gauche.internal port-case-fold) port) #f)
     (values)))
 
 (define-reader-directive 'gauche-legacy
