@@ -1,7 +1,7 @@
 ;;;
 ;;; libmacro.scm - built-in macros
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -38,17 +38,21 @@
                              ^ ^_ ^a ^b ^c ^d ^e ^f ^g ^h ^i ^j ^k ^l ^m ^n
                              ^o ^p ^q ^r ^s ^t ^u ^v ^w ^x ^y ^z $ cut cute rec
                              guard check-arg
-                             push! pop! inc! dec! update!
+                             push! push-unique! pop! inc! dec! update! rotate!
                              let1 if-let1 and-let1 let/cc begin0 rlet1
                              let-values let*-values define-values set!-values
                              values-ref values->list
                              assume assume-type
                              dotimes dolist do-plist doplist
-                             ecase cond-list define-condition-type condition
+                             ecase typecase etypecase cond-list
+                             define-condition-type condition
+                             with-continuation-marks
                              unwind-protect
                              let-keywords let-keywords* let-optionals*
+                             define-class define-generic define-method
                              lcons lcons* llist*
                              rxmatch-let rxmatch-if rxmatch-cond rxmatch-case
+                             independently
                              define-compiler-macro))
 
 ;; This file defines built-in macros.
@@ -66,6 +70,7 @@
      (define use. (r 'use))
      (define begin. (r 'begin))
      (define else. (r 'else))
+     (define allow-srfi-feature-id? #f)
 
      ;; Check feature requirement.  Returns #f if requirement is not
      ;; satisfied.  Returns a list of features to be use'd if requirement
@@ -75,7 +80,10 @@
        (cond
         [(identifier? req)
          (and-let1 p (assq (identifier->symbol req) features)
-           ((with-module gauche.internal %check-feature-id) (car p))
+           (unless allow-srfi-feature-id?
+             ((with-module gauche.internal %check-feature-id)
+              (car p)
+              (debug-source-info f)))
            (if (null? (cdr p)) seed (cons (cadr p) seed)))]
         [(not (pair? req)) (error "Invalid cond-expand feature-id:" req)]
         [else
@@ -117,7 +125,7 @@
 
      (define (fulfill-package rest seed)
        (match rest
-         [(package) 
+         [(package)
           (or (equal? package 'gauche)
               (find-gauche-package-description package))]
          [(package version-spec)
@@ -144,23 +152,50 @@
                         ,@(cdar cls)))]
         [else (rec (cdr cls))]))
 
-     (rec (cdr f)))))
+     ;; :allow-srfi-feature-id is a hidden extension to skip checking
+     ;; of srfi-N feature identifier. They're deprecated and warned,
+     ;; but certain legacy srfis such as SRFI-7 that expands to cond-expand
+     ;; do not provide alternative (library (srfi N)) form.  We only support
+     ;; such srfis for the legacy code, and it's simpler to use this
+     ;; compatibility mode rather than changing those srfis to use
+     ;; library clause.
+     (match f
+       [(_ ':allow-srfi-feature-id . clauses)
+        (set! allow-srfi-feature-id? #t)
+        (rec clauses)]
+       [(_ . clauses)
+        (rec clauses)]))))
 
 ;; transitional
 (with-module gauche.internal
-  (define (%check-feature-id sym)
-    (when (and (#/^srfi-\d+$/ (symbol->string sym))
-               (not (vm-compiler-flag-is-set? SCM_COMPILE_SRFI_FEATURE_ID)))
-      (warn "Feature identifier ~a is deprecated.  Use (library ~a)\n"
-            sym sym))))
+  (define (%check-feature-id sym source-info)
+    (let1 s (symbol->string sym)
+      (when (and (#/^srfi-\d+$/ s)
+                 (not (vm-compiler-flag-is-set? SCM_COMPILE_SRFI_FEATURE_ID)))
+        (warn "Feature identifier '~a' is deprecated~a.  Use (library ~a)\n"
+              sym
+              (if source-info
+                (format ", at ~s:~a" (car source-info) (cadr source-info))
+                "")
+              (regexp-replace #/^srfi-(\d+)$/ s "srfi.\\1"))))))
 
 ;;; quasirename
 
+(with-module gauche.internal
+  (define *quasirename-mode*
+    (and-let1 m (sys-getenv "GAUCHE_QUASIRENAME_MODE")
+      (cond [(equal? m "legacy") 'legacy]
+            [(equal? m "compatible") 'compatible]
+            [(equal? m "strict") 'strict]
+            [(equal? m "warn") 'warn]
+            [else (warn "Invalid GAUCHE_QUASIRENAME_MODE value; must be one \
+                         of compatible, legacy, warn or strict, but got: ~s\n"
+                        m)]))))
+
 ;; NB: The walk code has the same structure as quasiquote expander; the
 ;; output is different, however, and it's not so straightforward to refactor
 ;; them.  We'll eventually make a generic walker, but for now we have
 ;; two separately.
-
 (define-syntax quasirename
   (er-macro-transformer
    (^[f r c]
@@ -181,6 +216,7 @@
        (and (identifier? x)
             (c (r x) quasiquote.)))
      (define cons. (r'cons))
+     (define extended-cons. (r'extended-cons))
      (define list. (r'list))
      (define append. (r'append))
      (define vector. (r'vector))
@@ -248,7 +284,13 @@
              [yys (quasi* ys level)])
          (if (and (eq? x xx) (eq? ys yys))
            objs
-           `(,cons. ,xx ,yys))))
+           (if-let1 si (pair-attribute-get objs 'source-info #f)
+             (let1 orig (assoc-ref ((with-module gauche.internal %procedure-tags-alist) r)
+                                   'macro-input)
+               `(,extended-cons. ,xx ,yys '((source-info ,@si)
+                                            ,@(cond-list
+                                               [orig `(original . ,orig)]))))
+             `(,cons. ,xx ,yys)))))
 
      (define (quasi-vector obj level)
        (if (vector-has-splicing? obj)
@@ -269,12 +311,6 @@
                      (unquote-splicing? (car (vector-ref obj i))))]
                [else (loop (+ i 1))])))
 
-     ;; TRANSIENT: For the backward compatibility with legacy quasirename form.
-     ;; *quasirename-mode* is set according to GAUCHE_QUASIRENAME_MODE env var.
-     ;; see below.  We use global-variable-ref, since while compiling
-     ;; this with 0.9.7 *quasirename-mode* isn't defined.
-     (define qmode
-       (global-variable-ref 'gauche.internal '*quasirename-mode* #f))
      (define (legacy-message f)
        (if-let1 srcinfo (debug-source-info f)
          (format "Legacy quasirename form (~a:~a): ~s"
@@ -283,13 +319,13 @@
 
      (match f
        [(_ rr ((? quasiquote? qq) ff))
-        (if (eq? qmode 'legacy)
+        (if (eq? (with-module gauche.internal *quasirename-mode*) 'legacy)
           `(,let. ((,rename. ,rr))
               ,(quasi `(,qq ,ff) 0))
           `(,let. ((,rename. ,rr))
               ,(quasi ff 0)))]
        [(_ rr ff)                       ; old format
-        (case qmode
+        (case (with-module gauche.internal *quasirename-mode*)
           [(warn) (let1 srcinfo (debug-source-info f)
                     (warn "~a\n" (legacy-message f)))]
           [(strict) (let1 srcinfo (debug-source-info f)
@@ -298,17 +334,6 @@
            ,(quasi ff 0))]
        [_ (error "malformed quasirename:" f)]))))
 
-(with-module gauche.internal
-  (define *quasirename-mode*
-    (and-let1 m (sys-getenv "GAUCHE_QUASIRENAME_MODE")
-      (cond [(equal? m "legacy") 'legacy]
-            [(equal? m "compatible") 'compatible]
-            [(equal? m "strict") 'strict]
-            [(equal? m "warn") 'warn]
-            [else (warn "Invalid GAUCHE_QUASIRENAME_MODE value; must be one \
-                         of compatible, legacy, warn or strict, but got: ~s\n"
-                        m)]))))
-
 ;;; syntax-error msg arg ...
 ;;; syntax-errorf fmtstr arg ...
 ;;;   Signal an error at compile time.
@@ -414,7 +439,7 @@
                   `(,(r'lambda) ,arg ,($-fold (cdr f) '() arg)))]
                [else ($-fold (cdr f) '() #f)]))))))
 
-;;; cut, cute (srfi-26)
+;;; cut, cute (SRFI-26)
 
 (define-syntax cut
   (er-macro-transformer
@@ -468,7 +493,7 @@
        `(,(r'letrec) ,binds
          (,(r'lambda) ,tmps ,elts))))))
 
-;;; rec (srfi-31)
+;;; rec (SRFI-31)
 
 (define-syntax rec
   (er-macro-transformer
@@ -482,7 +507,7 @@
           `(letrec ((,name ,expr)) ,name))]
        [_ (error "malformed rec:" f)]))))
 
-;;; guard (srfi-34)
+;;; guard (SRFI-34)
 
 (define-syntax guard
   (er-macro-transformer
@@ -653,11 +678,15 @@
   (er-macro-transformer
    (^[f r c]
      (match f
-       [(_ (var ...) expr)
-        (let1 tmps (map (^_ (gensym)) var)
+       [(_ formals expr)
+        (let1 tmps (map* (^_ (gensym)) (^r (if (null? r) r (gensym))) formals)
           (quasirename r
             `(receive ,tmps ,expr
-               ,@(map (^[v t] (quasirename r `(set! ,v ,t))) var tmps)
+               ,@(map* (^[v t] (quasirename r `(set! ,v ,t)))
+                       (^[v t] (if (null? v)
+                                 '()
+                                 (quasirename r `((set! ,v ,t)))))
+                       formals tmps)
                (undefined))))]
        [_ (error "Malformed set!-values:" f)]))))
 
@@ -721,6 +750,28 @@
           `(set! ,loc (cons ,val ,loc)))]
        [_ (error "malformed push!:" f)]))))
 
+(define-syntax push-unique!
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ p val) (quasirename r `(push-unique! ,p ,val eqv?))]
+       [(_ (proc arg ...) val equal)
+        (let1 vars (map (^_ (gensym)) arg)
+          (quasirename r
+            `(let ([val ,val]
+                   [getter ,proc]
+                   ,@(map list vars arg))
+               (let1 vs (getter ,@vars)
+                 (unless (member val vs ,equal)
+                   ((setter getter) ,@vars (cons val vs)))))))]
+       [(_ loc val equal)
+        (quasirename r
+          `(let ([val ,val]
+                 [vs ,loc])
+             (unless (member val vs ,equal)
+               (set! ,loc (cons val vs)))))]
+       [_ (error "malformed push-unique!:" f)]))))
+
 (define-syntax pop!
   (er-macro-transformer
    (^[f r c]
@@ -787,17 +838,64 @@
           `(set! ,loc (,updater ,@val ,loc)))]
        [_ (error "malformed update!:" f)]))))
 
-;;; assume (srfi-145) and co.
+(define-syntax rotate!
+  (er-macro-transformer
+   (^[f r c]
+     ;; Returns
+     ;;  - list of bindings
+     ;;  - expression to get value
+     ;;  - a procedure returns an expression to set value
+     (define (gen-bind loc)
+       (match loc
+         [(proc arg ...)
+          (let ([gettervar (gensym)]
+                [argvars (map (^_ (gensym)) arg)])
+            (values `((,gettervar ,proc) ,@(map list argvars arg))
+                    `(,gettervar ,@argvars)
+                    (^[val] (quasirename r
+                              `((setter ,gettervar) ,@argvars ,val)))))]
+         [loc (values '()
+                      loc
+                      (^[val] (quasirename r
+                                `(set! ,loc ,val))))]))
+     (if (null? (cdr f))
+       (quasirename r `(begin)) ; special case
+       (let loop ([binds '()]
+                  [getters '()]
+                  [setter-gens '()]
+                  [locs (cdr f)])
+         (if (null? locs)
+           (let ([getters-rot (let* ([tail (list #f)]
+                                     [r (reverse getters tail)])
+                                (set! (car tail) (car r))
+                                (cdr r))]
+                 [tmps (map (^_ (gensym)) getters)])
+             (quasirename r
+               `(let ,(concatenate (reverse binds))
+                  (let ,(map list tmps getters-rot)
+                    ,@(map (^[gen tmp] (gen tmp)) (reverse setter-gens) tmps)))))
+           (receive (b g s) (gen-bind (car locs))
+             (loop (cons b binds)
+                   (cons g getters)
+                   (cons s setter-gens)
+                   (cdr locs))))))
+     )))
+
+;;; assume (SRFI-145) and co.
 
 ;; We might add run-time optimization switch to expand assume to nothing.
 (define-syntax assume
   (er-macro-transformer
    (^[f r c]
      (match f
-       [(_ expr . objs)
+       [(_ expr)
         (quasirename r
           `(or ,expr
-               (error (format "Invalid assumption: ~s" ',expr) ,@objs)))]))))
+               (error (format "Invalid assumption: ~s" ',expr))))]
+       [(_ expr msg . objs)
+        (quasirename r
+          `(or ,expr
+               (error ,msg ,@objs)))]))))
 
 ;; This will eventually folded into the compiler.  The argumet must be
 ;; a literal <type>.
@@ -810,7 +908,13 @@
           `(let1 v ,expr
              (if (of-type? v ,type)
                v
-               (type-error ',expr ,type v))))]))))
+               (type-error ',expr ,type v))))]
+       [(_ expr type msg . objs)
+        (quasirename r
+          `(let1 v ,expr
+             (if (of-type? v ,type)
+               v
+               (error ,msg ,@objs))))]))))
 
 ;;; repeat construct
 
@@ -898,6 +1002,51 @@
                                     expecting one of ~s" v ',choices)))))))]
        [_ (error "Malformed ecase:" f)]))))
 
+;;; typecase, a la CL
+
+;; NB: Eventually typecase should be recognized by the compiler so that
+;;  - The compiler can inference and propagate type information of expr
+;;  - The compiler can do better job for dispatching on built-in types
+;;    rather than using generic type-of?.
+(define-syntax typecase
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ expr clause ...)
+        (let1 v (gensym)
+          (define (clauses cs)
+            (match cs
+              [() (undefined)]
+              [((type-expr expr ...) . rest)
+               (if (c (r'else) type-expr)
+                 (if (null? rest)
+                   (quasirename r `(begin ,@expr))
+                   (error "'else' clause isn't at the end:" f))
+                 (quasirename r
+                   `(if (of-type? ,v ,type-expr)
+                      (begin ,@expr)
+                      ,(clauses rest))))]
+              [_ (error "Malformed clause in " f)]))
+          (quasirename r
+            `(let ([,v ,expr]) ,(clauses clause))))]
+       [_ (error "Malformed typecase:" f)]))))
+
+(define-syntax etypecase
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ expr clause ...)
+        (if (find (^[cl] (and (pair? cl) (c (r'else) (car cl)))) clause)
+          (quasirename r
+            `(typecase ,expr ,@clause))
+          (quasirename r
+            `(let ((v ,expr))
+               (typecase v ,@clause
+                         (else
+                          (errorf "etypecase fell through: expecting one of \
+                                 types in ~s, but got ~s"
+                                  (map car '(,@clause)) v))))))]
+       [_ (error "Malformed etypecase:" f)]))))
 
 ;;; cond-list - a syntax to construct a list
 
@@ -942,9 +1091,9 @@
                (if tmp (cons (begin ,@expr) r) r)))]
          )))))
 
-;;; srfi-35 condition macros
+;;; SRFI-35 condition macros
 
-;; we extend srfi-35 to allow #f as predicate and accessors, as well as
+;; we extend SRFI-35 to allow #f as predicate and accessors, as well as
 ;; omitting accessors.
 
 (define-syntax define-condition-type
@@ -1002,6 +1151,19 @@
                    type-field-binding)))]
        [_ (error "malformed condition:" f)]))))
 
+;;; continuation marks
+
+(define-syntax with-continuation-marks
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ () . body) (quasirename r `(let () ,@body))]
+       [(_ ((key val) . rest) . body)
+        (quasirename r
+          `(with-continuation-mark ,key ,val
+             (with-continuation-marks ,rest ,@body)))]
+       [_ (error "malformed with-continuation-marks:" f)]))))
+
 
 ;;; unwind-protect
 
@@ -1100,7 +1262,7 @@
   (er-macro-transformer
    (^[f r c]
      (match f
-       [(_ arg specs . body)
+       [(_ arg (? pair? specs) . body)
         (define (rec arg vars&inits rest)
           (cond
            [(null? (cdr vars&inits))
@@ -1135,6 +1297,12 @@
                               (^_ '()) ; ignore last cdr of dotted list
                               specs)
                       (cdr (last-pair specs))))))]
+       [(_ arg (? identifier? var) . body)
+        ;; A special case when the optional binding list is ().  This may
+        ;; occur as a result of macro expansion, and it's convenient to
+        ;; handle it here instead of each macro expander special-case it.
+        (quasirename r
+          `(let ((,var ,arg)) ,@body))]
        [_ (error "Malformed let-optionals*:" f)]))))
 
 (select-module gauche)
@@ -1248,6 +1416,42 @@
                (if (eq? %let 'let) 'let-keywords 'let-keywords*)
                form)]))
 
+;;; object system
+
+(select-module gauche)
+
+(define-syntax define-class
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ name supers slots . options)
+        ($ (with-module gauche.object %expand-define-class)
+           name supers slots options)]
+       [_ (error "syntax-error: malformed define-class:" f)]))))
+
+(define-syntax define-generic
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ name . opts)
+        ((with-module gauche.object %expand-define-generic) name opts)]
+       [_ (error "syntax-error: malformed define-generic:" f)]))))
+
+(define-syntax define-method
+  (er-macro-transformer
+   (^[f r c]
+     (define (gather-quals name rest quals)
+       (match rest
+         [((? keyword? q) . rest)
+          (gather-quals name rest (cons q quals))]
+         [(specs . body)
+          ($ (with-module gauche.object %expand-define-method)
+             name (reverse quals) specs body f)]
+         [_ (error "syntax-error: malformed define-method (empty body):" f)]))
+     (match f
+       [(_ name . rest) (gather-quals name rest '())]
+       [_ (error "syntax-error: malformed define-method:" f)]))))
+
 ;;; lseq
 (select-module gauche)
 (define-syntax lcons
@@ -1272,8 +1476,48 @@
 
 (define-syntax llist* lcons*)
 
+
+;;; format
+
+(select-module gauche.format)
+(use util.match)
+
+(define-syntax make-format-transformer
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ shared?)
+        (quasirename r
+          `(er-macro-transformer
+            (^[f r c]
+              (define (context-literal pos) `(,',shared? ,pos ,(box #f)))
+              (match f
+                [(_ (? string?) . _)
+                 (quasirename r
+                   `(format-internal ',(context-literal 0) (list ,@(cdr f))))]
+                [(_ _ (? string?) . _)
+                 (quasirename r
+                   `(format-internal ',(context-literal 1) (list ,@(cdr f))))]
+                [(_ _ _ (? string?) . _)
+                 (quasirename r
+                   `(format-internal ',(context-literal 2) (list ,@(cdr f))))]
+                [_ f]))))]))))
+
+(define-hybrid-syntax format
+  (^ args (format-internal '(#f #f #f) args))
+  (make-format-transformer #f))
+
+(define-hybrid-syntax format/ss
+  (^ args (format-internal '(#t #f #f) args))
+  (make-format-transformer #t))
+
+(select-module gauche)
+(define-inline format (with-module gauche.format format))
+(define-inline format/ss (with-module gauche.format format/ss))
+
 ;;; rxmatch-* macros
 
+(select-module gauche)
 ;; rxmatch-let expr (var ...) form ...
 (define-syntax rxmatch-let
   (er-macro-transformer
@@ -1388,11 +1632,13 @@
        [_ (error "malformed rxmatch-case:" f)]))))
 
 
-;;;
-;;; OBSOLETED - Tentative compiler macro
-;;;
+;;; SRFI-236
 
-(select-module gauche)
-;; TRANSIENT: Remove by 1.0
-(define-macro (define-compiler-macro name xformer-spec)
-  (error "define-compiler-macro is obsoleted.  Use define-hybrid-syntax."))
+(define-syntax independently
+  (er-macro-transformer
+   (^[f r c]
+     (quasirename r
+       `(let ,(map (^[expr] (quasirename r
+                              `(,(gensym) (begin ,expr #f))))
+                   (cdr f))
+          (undefined))))))
