@@ -1,7 +1,7 @@
 ;;;
 ;;; object.scm - object system
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,8 +35,9 @@
 
 ;; preparing inline stub code
 (inline-stub
- (declcode (.include <gauche/priv/classP.h>
-                     <gauche/vminsn.h>))
+ (.include "gauche/priv/configP.h"
+           "gauche/priv/classP.h"
+           "gauche/vminsn.h")
  (declare-stub-type <slot-accessor> "ScmSlotAccessor*")
  (declare-stub-type <generic> "ScmGeneric*")
  (declare-stub-type <method> "ScmMethod*")
@@ -84,9 +85,7 @@
 ;; Generic function
 ;;
 
-;(define-macro (define-generic name . opts)
-;  (%expand-define-generic name opts))
-
+;; Called from define-generic macro expander in libmacro.scm
 (define (%expand-define-generic name opts)
   (receive (true-name getter-name) (%check-setter-name name)
     (let ([class (get-keyword :class opts <generic>)]
@@ -122,10 +121,8 @@
 ;; Method
 ;;
 
-;(define-macro (define-method name [quals ...] specs . body)
-;  (%expand-define-macro name quals specs body))
-
-(define (%expand-define-method name quals specs body)
+;; Called from define-method macro expander in libmacro.scm
+(define (%expand-define-method name quals specs body form)
   ;; check qualifiers.  we only support :locked for now
   ;; TRANSIENT: Must use hygienic compare.
   (if-let1 bad (find (^q (not (memq q '(:locked)))) quals)
@@ -148,26 +145,35 @@
            [real-args    (if rest
                            `(,@reqargs ,rest next-method)
                            `(,@reqargs next-method))]
-           [real-body (if opts
-                        (quasirename %id
-                          `(lambda ,real-args
-                             (apply (lambda ,opts ,@body) ,rest)))
-                        (quasirename %id
-                          `(lambda ,real-args ,@body)))])
+           [real-body ((with-module gauche.internal with-source-info)
+                       (if opts
+                         (quasirename %id
+                           `(lambda ,real-args
+                              (apply (lambda ,opts ,@body) ,rest)))
+                         (quasirename %id
+                           `(lambda ,real-args ,@body)))
+                       form)]
+           ;; We bind method body to an artificially weird module-local name,
+           ;; so that test-module can find and scan the method's body.
+           [method-unique-name (symbol-append name specs)]
+           )
       (receive (true-name getter-name) (%check-setter-name name)
         (let1 gf (gensym "gf")
           (quasirename %id
-            `(rlet1 ,gf (%ensure-generic-function ',true-name (current-module))
-               (add-method! ,gf (make <method>
-                                  ':generic ,gf
-                                  ':specializers (list ,@specializers)
-                                  ':lambda-list ',lambda-list
-                                  ':method-locked (boolean (memq ':locked ',quals))
-                                  ':body ,real-body))
-               ,@(cond-list [getter-name
-                             (quasirename %id
-                               `(unless (has-setter? ,getter-name)
-                                  (set! (setter ,getter-name) ,gf)))]))))))))
+            `(define ,method-unique-name
+               (rlet1 ,method-unique-name ,real-body
+                 (let1 ,gf (%ensure-generic-function ',true-name (current-module))
+                   (add-method! ,gf (make <method>
+                                      ':generic ,gf
+                                      ':specializers (list ,@specializers)
+                                      ':lambda-list ',lambda-list
+                                      ':method-locked (boolean (memq ':locked ',quals))
+                                      ':body ,method-unique-name))
+                   ,@(cond-list [getter-name
+                                 (quasirename %id
+                                   `(unless (has-setter? ,getter-name)
+                                      (set! (setter ,getter-name) ,gf)))]))))
+            ))))))
 
 (inline-stub
  ;; internal for %ensure-generic-function
@@ -229,15 +235,7 @@
           [(eqv? key (unwrap-syntax-1 (car as))) (cadr as)]
           [else (loop (cddr as))])))
 
-;; TRANSIENT: We should employ er-macro-transformer to expand
-;; define-class etc., but this file need to be compiled by 0.9.5
-;; and we've improved the transformer since then.  So we postponed
-;; rewriting these after 0.9.6 release.  Meanwhile, we manually
-;; replacing identifiers.
-
-;(define-macro (define-class name supers slots . options)
-;  (%expand-define-class name supers slots options))
-
+;; Called from define-class macro expander in libmacro.scm
 (define (%expand-define-class name supers slots options)
   (let* ([metaclass (or (%get-keyword :metaclass options #f)
                         (quasirename %id
@@ -498,11 +496,23 @@
 (define (class-slot-bound? class slot-name)
   (apply (%class-slot-gns class slot-name 'bound?) '(#f)))
 
-;; default class printer.  Avoid using class-name so that in case
-;; when obj's class has been redefined, this wouldn't trigger updating obj.
+;; Default class printer.  This is only called if the class is an instance
+;; of a metaclass other than <class>.  (If it's <class>, built-in print
+;; method is called; see class.c)
+;; Avoid using directly access the slots of obj, so that it won't trigger
+;; updating obj in case when obj's class has been redefined.
+;; We also need a cproc to check if OBJ is redefined; for, accessing
+;; 'redefined' slot needs to lock OBJ, which blocks when OBJ is being
+;; redefined.
 (define-method write-object ((obj <class>) out)
-  (format out "#<class ~a>"
-          (slot-ref-using-class (current-class-of obj) obj 'name)))
+  (let1 name (slot-ref-using-class (current-class-of obj) obj 'name)
+    (if (%class-redefined? obj)
+      (format out "#<class ~a (redefined)>" name)
+      (format out "#<class ~a>" name))))
+
+(define-cproc %class-redefined? (obj) ::<boolean>
+  (return (and (SCM_ISA obj SCM_CLASS_CLASS)
+               (not (SCM_FALSEP (-> (cast ScmClass* obj) redefined))))))
 
 ;; convenient routine to push/pop a value to the slot.
 ;; this can be optimized later.
@@ -781,7 +791,7 @@
 (define-method (setter ref) :locked ((obj <tree-map>) key value)
   (tree-map-put! obj key value))
 
-(define-method ref  ((obj <box>) field) ; srfi-123
+(define-method ref  ((obj <box>) field) ; SRFI-123
   (unless (eq? field '*)
     (error "Box content can be accessed with a field name '*, but got:"
            field))
@@ -893,7 +903,9 @@
             :base   ,(~ obj'base)
             :radix  ,(~ obj'radix)
             :pretty ,(~ obj'pretty)
-            :width  ,(~ obj'width))))
+            :width  ,(~ obj'width)
+            :indent ,(~ obj'indent)
+            :string-length ,(~ obj'string-length))))
 
 ;;----------------------------------------------------------------
 ;; Describe
@@ -932,8 +944,7 @@
 (let ((xfer (with-module gauche.internal %transfer-bindings)))
   (xfer (current-module)
         (find-module 'gauche)
-        '(;define-generic define-method define-class
-          compute-slots
+        '(compute-slots
           compute-get-n-set compute-slot-accessor
           class-slot-ref class-slot-set! class-slot-bound?
           slot-push! slot-pop! slot-unbound slot-missing
@@ -958,12 +969,4 @@
           x->string x->integer x->number ref |setter of ref|
           ~ ref*
           describe describe-common describe-slots
-
-          ;; These shouldn't be necessary to be injected into gauche
-          ;; module; unfortunately, the current define-method and
-          ;; define-class are unhygienic, and we need them visible
-          ;; from the expanded code.  Should be removed once we rewrite
-          ;; related macros hygienic.
-          %ensure-generic-function
-          %check-class-binding
           )))
