@@ -1,7 +1,7 @@
 ;;;
 ;;; libomega.scm - the stuff to be run after other parts are initialized
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -57,16 +57,16 @@
 
 ;; Printing auxiliary error information.  This also is here instead
 ;; of libexc.scm because of the initialization order.
-(define-method report-mixin-condition ((c <mixin-condition>) port) #f)
+(define-method report-additional-condition (c port) #f)
 
-(define-method report-mixin-condition ((c <load-condition-mixin>) port)
+(define-method report-additional-condition ((c <load-condition-mixin>) port)
   (and-let* ([p (~ c'port)]
              [ (port? p) ]
              [name (port-name p)]
              [line (port-current-line p)])
     (format port "    While loading ~s at line ~d\n" name line)))
 
-(define-method report-mixin-condition ((c <compile-error-mixin>) port)
+(define-method report-additional-condition ((c <compile-error-mixin>) port)
   (let* ([expr (~ c'expr)]
          [src-info (find (^[si] (and (car si) (cadr si)))
                          ((with-module gauche.internal %source-info) expr))])
@@ -75,6 +75,21 @@
               (car src-info) (cadr src-info) expr)
       (format port "    While compiling: ~,,,,90:s\n" expr))))
 
+(define-method report-additional-condition ((c <unbound-variable-error>) port)
+  ;; Show potentially missed modules to be imported.
+  ;; NB: This is inefficient, for export lists are built for all modules.
+  ;; It'd be better to have a predicate that directly queries if a binding
+  ;; is exported from a module.
+  ;; NB: We may also search non-exported bindings, to detect a binding
+  ;; that are missed in the export list.
+  (let* ([name (~ c'identifier)]
+         [xs (filter (^m (memq name (module-exports m))) (all-modules))]
+         [num-xs (length xs)])
+    (unless (null? xs)
+      (format port
+              "    NOTE: `~s' is exported from the following module~p:\n"
+              name num-xs)
+      (dolist [m xs] (format port "     - ~s\n" (module-name m))))))
 
 ;;;
 ;;; Comparator finish-up
@@ -82,7 +97,7 @@
 
 ;; Built-in comparators.  These are here instead of libcmp.scm, for
 ;; hash functions need to be defined before this.
-;; NB: These are in srfi-114 but not in srfi-128.  We provide them
+;; NB: These are in SRFI-114 but not in SRFI-128.  We provide them
 ;; so that when they are used as the argument of make-hash-table,
 ;; we can recognize so and use more efficient built-in hash types.
 (define eq-comparator
@@ -98,12 +113,12 @@
   (make-comparator/compare #t (with-module gauche.internal default-comparator-equal?)
                            compare default-hash 'default-comparator))
 
-;; NB: srfi-128's make-eq-comparator and make-eqv-comparator will NOT
+;; NB: SRFI-128's make-eq-comparator and make-eqv-comparator will NOT
 ;; return eq-comparator and eqv-comparator.  They're defined in computil.scm.
-(define-inline (make-equal-comparator) equal-comparator); srfi-128
-(define-inline (make-default-comparator) default-comparator) ;srfi-128
+(define-inline (make-equal-comparator) equal-comparator); SRFI-128
+(define-inline (make-default-comparator) default-comparator) ;SRFI-128
 
-(define (comparator-register-default! cmpr) ;srfi-128
+(define (comparator-register-default! cmpr) ;SRFI-128
   (unless (is-a? cmpr <comparator>)
     (error "Comparator required, but got:" cmpr))
   (with-module gauche.internal
@@ -126,7 +141,7 @@
            (eqv? (~ x'hash) (~ y'hash)))
        (equal? (slot-ref x 'name) (slot-ref y 'name))))
 
-;; Extend default-comparator by srfi-128 way
+;; Extend default-comparator by SRFI-128 way
 ;;
 ;;  Our default-comparator already handles any type of objects,
 ;;  and delegates customized behavior for object-equal?, object-compare
@@ -214,16 +229,12 @@
   (object-hash obj (if salt (^o (hash o salt)) hash)))
 
 ;; This is the fallback in case we have legacy one-argument object-hash.
-;; We don't need full-brown parameterize, for %current-recursive-hash
-;; isn't supposed to be changed other than this method.
 (define-method object-hash (obj hash)
   (let1 h (%current-recursive-hash)
     (if (eq? h hash)
       (object-hash obj) ; shortcut
-      (dynamic-wind
-        (^[] (%current-recursive-hash hash))
-        (^[] (object-hash obj))
-        (^[] (%current-recursive-hash h))))))
+      (parameterize ([%current-recursive-hash hash])
+        (object-hash obj)))))
 
 ;; Make hashtable hashable.  For the time being, we ignore hashtable's
 ;; comparators.
@@ -246,12 +257,24 @@
 ;;; Read-edit mode flag
 ;;;
 
-(with-module gauche.internal)
-;; This variable affects how gauche.interactive handles input; if true,
-;; it sets up the input editor if possible.  This is also modified by
-;; main.c with -fread-edit/-fno-read-edit flag.
-;; Note that once gauche.interactive is loaded, this variable has no effect.
-(define *read-edit* (not (sys-getenv "GAUCHE_NO_READ_EDIT")))
+(with-module gauche.internal
+  ;; This variable affects how gauche.interactive handles input; if true,
+  ;; it sets up the input editor if possible.  This is also modified by
+  ;; main.c with -fread-edit/-fno-read-edit flag.
+  ;; Note that once gauche.interactive is loaded, this variable has no effect.
+  (define *read-edit* (not (sys-getenv "GAUCHE_NO_READ_EDIT")))
+  )
+
+;;;
+;;; Stdio setup
+;;;
+
+(with-module gauche.internal
+  (when (or (and (sys-isatty (standard-input-port))
+                 (sys-isatty (standard-output-port)))
+            (and (%sys-mintty? (standard-input-port))
+                 (%sys-mintty? (standard-output-port))))
+    (port-link! (standard-input-port) (standard-output-port))))
 
 ;;;
 ;;; Miscellaneous
