@@ -1,7 +1,7 @@
 ;;;
 ;;; libsys.scm - builtin system inteface
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -36,24 +36,21 @@
 
 (select-module gauche)
 (inline-stub
- (declcode
-  (.include <gauche/priv/mmapP.h>)
+  (.include "gauche/priv/configP.h"
+            "gauche/priv/mmapP.h")
   (.include <stdlib.h> <locale.h> <math.h> <sys/types.h> <sys/stat.h> <fcntl.h>)
-  (.cond ["TIME_WITH_SYS_TIME" (.include <sys/time.h> <time.h>)]
-         ["HAVE_SYS_TIME_H"    (.include <sys/time.h>)]
-         [else                 (.include <time.h>)])
-  (.if "!defined(GAUCHE_WINDOWS)"
-       (.include <grp.h> <pwd.h> <sys/wait.h> <utime.h>
-                 <sys/times.h> <sys/utsname.h>))
-  (.if "HAVE_CRYPT_H"        (.include <crypt.h>))
-  (.if "HAVE_SYS_RESOURCE_H" (.include <sys/resource.h>))
-  (.if "HAVE_SYS_LOADAVG_H"  (.include <sys/loadavg.h>))
-  (.if "HAVE_UNISTD_H"       (.include <unistd.h>))
-  (.if "HAVE_SYS_MMAN_H"     (.include <sys/mman.h>))
+  (.unless (defined "GAUCHE_WINDOWS")
+    (.include <grp.h> <pwd.h> <sys/wait.h> <utime.h>
+              <sys/times.h> <sys/utsname.h>))
+  (.when "HAVE_CRYPT_H"        (.include <crypt.h>))
+  (.when "HAVE_SYS_RESOURCE_H" (.include <sys/resource.h>))
+  (.when "HAVE_SYS_LOADAVG_H"  (.include <sys/loadavg.h>))
+  (.when "HAVE_UNISTD_H"       (.include <unistd.h>))
+  (.when "HAVE_SYS_MMAN_H"     (.include <sys/mman.h>))
 
-  (.if "defined(GAUCHE_WINDOWS)"
-       (.undef _SC_CLK_TCK)) ;; avoid undefined reference to sysconf
-  ))
+  (.when (defined "GAUCHE_WINDOWS")
+    (.undef _SC_CLK_TCK)) ;; avoid undefined reference to sysconf
+  )
 
 ;;---------------------------------------------------------------------
 ;; dirent.h - read directory
@@ -147,12 +144,12 @@
 (define-macro (define-errno symbol)
   `(inline-stub
     (initcode
-     (.if ,#"defined(~symbol)"
-          (begin
-            (Scm_Define (Scm_GaucheModule)
-                        (SCM_SYMBOL ',symbol) (SCM_MAKE_INT ,symbol))
-            (Scm_HashTableSet errno_n2y (SCM_MAKE_INT ,symbol) ',symbol 0)
-            (Scm_HashTableSet errno_y2n ',symbol (SCM_MAKE_INT ,symbol) 0))))))
+     (.when (defined ,symbol)
+       (begin
+         (Scm_Define (Scm_GaucheModule)
+                     (SCM_SYMBOL ',symbol) (SCM_MAKE_INT ,symbol))
+         (Scm_HashTableSet errno_n2y (SCM_MAKE_INT ,symbol) ',symbol 0)
+         (Scm_HashTableSet errno_y2n ',symbol (SCM_MAKE_INT ,symbol) 0))))))
 
 (define-cproc sys-errno->symbol (num::<fixnum>)
   (return (Scm_HashTableRef errno_n2y (SCM_MAKE_INT num) SCM_FALSE)))
@@ -320,7 +317,7 @@
   [(_ conv sym) `(Scm_Cons ',sym (,conv (-> lc ,sym)))])
 )
 
-(define-cproc sys-setlocale (category::<fixnum> locale::<const-cstring>)
+(define-cproc sys-setlocale (category::<fixnum> locale::<const-cstring>?)
   ::<const-cstring>? setlocale)
 
 (define-cproc sys-localeconv ()
@@ -424,12 +421,13 @@
                           newname::<const-cstring>)
   ::<void>
   (let* ([r::int])
-    (.if "defined(GAUCHE_WINDOWS)"
-         ;; Windows doesn't allow renaming to the existing file, so we unlink
-         ;; it first.  This breaks the atomicity of rename operation.
-         ;; We don't check and raise an error here, since the error will be
-         ;; caught by rename() call.
-         (begin (chmod newname #o666) (unlink newname)))
+    (.when (defined "GAUCHE_WINDOWS")
+      ;; Windows doesn't allow renaming to the existing file, so we unlink
+      ;; it first.  This breaks the atomicity of rename operation.
+      ;; We don't check and raise an error here, since the error will be
+      ;; caught by rename() call.
+      (chmod newname #o666)
+      (unlink newname))
     (SCM_SYSCALL r (rename oldname newname))
     (when (< r 0) (Scm_SysError "renaming %s to %s failed" oldname newname))))
 
@@ -451,15 +449,16 @@
 
 ;; ctermid
 (define-cproc sys-ctermid ()
-  (.if "defined(GAUCHE_WINDOWS)"
-       (return '"CON")
-       (let* ([buf::(.array char [(+ L_ctermid 1)])])
-         (return (SCM_MAKE_STR_COPYING (ctermid buf))))))
+  (.if (defined "GAUCHE_WINDOWS")
+    (return '"CON")
+    (let* ([buf::(.array char [(+ L_ctermid 1)])])
+      (return (SCM_MAKE_STR_COPYING (ctermid buf))))))
 
 ;;---------------------------------------------------------------------
 ;; stdlib.h
 
-(define-cproc sys-exit (code::<fixnum>) ::<void> _exit)
+(define-cproc sys-exit (code) ::<void>
+  (_exit (Scm_ObjToExitCode code)))
 
 (define-cproc sys-getenv (name::<const-cstring>) ::<const-cstring>? Scm_GetEnv)
 
@@ -475,9 +474,9 @@
     (SCM_SYSCALL SCM_RESULT (system command))))
 
 (define-cproc sys-random () ::<long>
-  (.cond ["defined(HAVE_RANDOM) && defined(HAVE_SRANDOM)"
+  (.cond [(and (defined "HAVE_RANDOM") (defined "HAVE_SRANDOM"))
           (return (random))]
-         ["defined(LRAND48) && defined(SRAND48)"
+         [(and (defined "LRAND48") (defined "SRAND48"))
           (return (lrand48))]
          [else
           ;; fallback - we don't want to use rand(), for it is not
@@ -486,9 +485,9 @@
 
 (define-cproc sys-srandom (seed) ::<void>
   (unless (SCM_EXACTP seed) (Scm_Error "exact integer required: %S" seed))
-  (.cond ["defined(HAVE_RANDOM) && defined(HAVE_SRANDOM)"
+  (.cond [(and (defined "HAVE_RANDOM") (defined "HAVE_SRANDOM"))
           (srandom (Scm_GetUInteger seed))]
-         ["defined(LRAND48) && defined(SRAND48)"
+         [(and (defined "LRAND48") (defined "SRAND48"))
           (srand48 (Scm_GetUInteger seed))]
          [else
           ;; fallback - we don't want to use rand(), for it is not
@@ -540,23 +539,38 @@
 ;; various systems.
 (define-cproc sys-strerror (errno_::<int>) ::<const-cstring> strerror)
 
+;; Added in POSIX.1-2008.  We could do something more meaningful if
+;; the platform doesn't have it, but this is the minimum support.
+;; TODO: strsignal() isn't MT-safe; not only with another thread
+;; calling strsignal(), but also with setlocale().  Eventually we'll
+;; add manual global lock.
+;; POSIX also doesn't say anything when signum isn't a supported signal
+;; number.  We can't do much about it, though, so it's better just to expose
+;; the underlying function.
+(inline-stub
+  (.if HAVE_STRSIGNAL
+    (define-cproc sys-strsignal (signum::<int>) ::<const-cstring>?
+      (return (strsignal signum)))
+    (define-cproc sys-strsignal (_::<int>) ::<const-cstring>?
+      (return NULL))))
+
 ;;---------------------------------------------------------------------
 ;; sys/loadavg.h
 
 (inline-stub
 (.if (defined HAVE_GETLOADAVG)
   (define-cproc sys-getloadavg (:optional (nsamples::<int> 3))
-    (let* ([samples::(.array double [3])])
-      (when (or (<= nsamples 0) (> nsamples 3))
-        (Scm_Error "sys-getloadavg: argument out of range: %d" nsamples))
-      (let* ([count::int (getloadavg samples nsamples)])
-        (if (< count 0)
-          (return '#f)
-          (let* ([h '()] [t '()])
-            (dotimes [i count]
-              (let* ([n (Scm_MakeFlonum (aref samples i))])
-                (SCM_APPEND1 h t n)))
-            (return h))))))
+    (let* ([samples::(.array double [3])]
+           [_ (when (or (<= nsamples 0) (> nsamples 3))
+                (Scm_Error "sys-getloadavg: argument out of range: %d" nsamples))]
+           [count::int (getloadavg samples nsamples)])
+      (if (< count 0)
+        (return '#f)
+        (let* ([h '()] [t '()])
+          (dotimes [i count]
+            (let* ([n (Scm_MakeFlonum (aref samples i))])
+              (SCM_APPEND1 h t n)))
+          (return h)))))
   (define-cproc sys-getloadavg (:optional _::<int>)
     (begin
       (Scm_Error "sys-getloadavg isn't supported on this platform")
@@ -571,8 +585,7 @@
    ()
    ((address :c-name "ptr"
              :c-spec "Scm_MakeIntegerU((uintptr_t)obj->ptr)" :setter #f)
-    ;; TRANSIENT: After 0.9.11, we can say :c-type <size_t>.
-    (size :c-spec "Scm_MakeIntegerU(obj->size)" :setter #f)
+    (size :type <size_t>  :setter #f)
     (protection :c-name "prot" :type <int> :setter #f)
     (flags :type <int> :setter #f))
    (printer (let* ((m::ScmMemoryRegion* (SCM_MEMORY_REGION obj)))
@@ -582,9 +595,9 @@
                           (?: (logand (-> m prot) PROT_WRITE) "w" "")
                           (?: (logand (-> m prot) PROT_EXEC) "x" ""))))))
 
-(define-cproc sys-mmap (maybe-port prot::<int> flags::<int> size::<integer>
-                                   :optional (off::<integer> 0))
-  (let* ([fd::int -1] [isize::size_t 0] [ioff::off_t 0])
+(define-cproc sys-mmap (maybe-port prot::<int> flags::<int> size::<size_t>
+                                   :optional (off::<off_t> 0))
+  (let* ([fd::int -1])
     (cond [(SCM_PORTP maybe-port)
            (set! fd (Scm_PortFileNo (SCM_PORT maybe-port)))
            (when (< fd 0)
@@ -592,13 +605,7 @@
                         maybe-port))]
           [(SCM_FALSEP maybe-port)]
           [else (SCM_TYPE_ERROR maybe-port "port or #f")])
-    (cond [(SCM_UNBOUNDP size)]
-          [(SCM_INTEGERP size) (set! isize (Scm_IntegerToSize size))]
-          [else (SCM_TYPE_ERROR size "integer suitable for size_t")])
-    (cond [(SCM_UNBOUNDP off)]
-          [(SCM_INTEGERP off) (set! ioff (Scm_IntegerToOffset off))]
-          [else (SCM_TYPE_ERROR off "integer suitable for off_t")])
-    (return (Scm_SysMmap NULL fd isize ioff prot flags))))
+    (return (Scm_SysMmap NULL fd size off prot flags))))
 
 (inline-stub
  (define-enum PROT_EXEC)
@@ -613,18 +620,17 @@
 ;; sys/resource.h
 
 (inline-stub
- (when "defined(HAVE_SYS_RESOURCE_H)"
-   "
-#if SIZEOF_RLIM_T == 4
-#  define MAKERLIMIT(val)  Scm_MakeIntegerU(val)
-#  define GETRLIMIT(obj)   Scm_GetIntegerU(obj)
-#elif SIZEOF_RLIM_T == 8
-#  define MAKERLIMIT(val)  Scm_MakeIntegerU64(val)
-#  define GETRLIMIT(obj)   Scm_GetIntegerU64(obj)
-#else
-#  error \"rlim_t must be 32bit or 64bit\"
-#endif
-"
+ (.when (defined "HAVE_SYS_RESOURCE_H")
+   (.if (== SIZEOF_RLIM_T 4)
+     (begin
+       (.define MAKERLIMIT (val) (Scm_MakeIntegerU val))
+       (.define GETRLIMIT (obj)  (Scm_GetIntegerU obj)))
+     (.if (== SIZEOF_RLIM_T 8)
+       (begin
+         (.define MAKERLIMIT (val) (Scm_MakeIntegerU64 val))
+         (.define GETRLIMIT (obj)  (Scm_GetIntegerU64 obj)))
+       (.error "rlim_t must be 32bit or 64bit")))
+
    (define-cproc sys-getrlimit (rsrc::<int>) ::(<integer> <integer>)
      (let* ([limit::(struct rlimit)] [ret::int])
        (SCM_SYSCALL ret (getrlimit rsrc (& limit)))
@@ -684,7 +690,7 @@
  ;; except if we're stat()-ing the root directory.  For the convenience
  ;; we remove the trailing separator if any.  It is quite complicated,
  ;; for we have to deal with optional drive letters.
- (when "defined(GAUCHE_WINDOWS)"
+ (.when (defined "GAUCHE_WINDOWS")
    (define-cfn check-trailing-separator (path::(const char*))
      ::(const char*) :static
      (let* ([size::int (strlen path)]
@@ -704,7 +710,7 @@
            (set! (aref pcopy (- size 1)) 0)
            (set! path (cast (const char *) pcopy))))
        (return path))))
- (when "!defined(GAUCHE_WINDOWS)"
+ (.unless (defined "GAUCHE_WINDOWS")
    (define-cfn check-trailing-separator (path::(const char*))
      ::(const char*) :static (return path)))
 
@@ -716,14 +722,13 @@
  ;; Since lstat() works identical to stat() if the path is a symlink, and
  ;; on Windows path can never be a symlink, so we can just make sys-lstat
  ;; work the same as sys-stat.
- (when "!defined(GAUCHE_WINDOWS)"
+ (.if (not (defined "GAUCHE_WINDOWS"))
    (define-cproc sys-lstat (path::<const-cstring>) ::<sys-stat>
-     (stat-common lstat)))
- (when "defined(GAUCHE_WINDOWS)"
+     (stat-common lstat))
    (define-cproc sys-lstat (path::<const-cstring>) ::<sys-stat>
      (stat-common stat)))
 
- (when "!defined(GAUCHE_WINDOWS)"
+ (.unless (defined "GAUCHE_WINDOWS")
    (define-cproc sys-mkfifo (path::<const-cstring> mode::<int>) ::<int>
      (SCM_SYSCALL SCM_RESULT (mkfifo path mode))
      (when (< SCM_RESULT 0) (Scm_SysError "mkfifo failed on %s" path))))
@@ -792,13 +797,13 @@
 ;; we have emulation of times() in auxsys.c for mingw.
 (define-cproc sys-times ()
   (let* ([info::(struct tms)] [r::clock_t] [tick::long])
-    (SCM_SYSCALL r (times (& info)))
+    (SCM_SYSCALL3 r (times (& info)) (== r (cast clock_t -1)))
     (when (== r (cast clock_t -1)) (Scm_SysError "times failed"))
-    (.if "defined(_SC_CLK_TCK)"
-         (set! tick (sysconf _SC_CLK_TCK))
-         (.if "defined(CLK_TCK)"
-              (set! tick CLK_TCK)   ; older name
-              (set! tick 100)))     ; fallback
+    (.if (defined "_SC_CLK_TCK")
+      (set! tick (sysconf _SC_CLK_TCK))
+      (.if (defined "CLK_TCK")
+        (set! tick CLK_TCK)   ; older name
+        (set! tick 100)))     ; fallback
     (return (list (Scm_MakeInteger (ref info tms_utime))
                   (Scm_MakeInteger (ref info tms_stime))
                   (Scm_MakeInteger (ref info tms_cutime))
@@ -809,16 +814,16 @@
 ;; sys/utsname.h
 
 (define-cproc sys-uname ()
-  (.if "!defined(GAUCHE_WINDOWS)"
-       (let* ([info::(struct utsname)])
-         (when (< (uname (& info)) 0) (Scm_SysError "uname failed"))
-         (return (list (SCM_MAKE_STR_COPYING (ref info sysname))
-                       (SCM_MAKE_STR_COPYING (ref info nodename))
-                       (SCM_MAKE_STR_COPYING (ref info release))
-                       (SCM_MAKE_STR_COPYING (ref info version))
-                       (SCM_MAKE_STR_COPYING (ref info machine)))))
-       ;; TODO: Fill with appropriate info.
-       (return (list SCM_FALSE SCM_FALSE SCM_FALSE SCM_FALSE SCM_FALSE))))
+  (.if (not (defined "GAUCHE_WINDOWS"))
+    (let* ([info::(struct utsname)])
+      (when (< (uname (& info)) 0) (Scm_SysError "uname failed"))
+      (return (list (SCM_MAKE_STR_COPYING (ref info sysname))
+                    (SCM_MAKE_STR_COPYING (ref info nodename))
+                    (SCM_MAKE_STR_COPYING (ref info release))
+                    (SCM_MAKE_STR_COPYING (ref info version))
+                    (SCM_MAKE_STR_COPYING (ref info machine)))))
+    ;; TODO: Fill with appropriate info.
+    (return (list SCM_FALSE SCM_FALSE SCM_FALSE SCM_FALSE SCM_FALSE))))
 
 ;;---------------------------------------------------------------------
 ;; sys/wait.h
@@ -873,18 +878,40 @@
       (begin (set! SCM_RESULT0 SCM_FALSE)
              (set! SCM_RESULT1 SCM_FALSE)))))
 
-(define-cproc current-time ()           ;SRFI-18, SRFI-19, SRFI-21
+(define-cproc current-time ()           ;SRFI-18, SRFI-19, SRFI-21, SRFI-226
   Scm_CurrentTime)
 
-(define-cproc time? (obj)               ;SRFI-18, SRFI-19, SRFI-21
+(define-cproc time? (obj)               ;SRFI-18, SRFI-19, SRFI-21, SRFI-226
   ::<boolean> SCM_TIMEP)
 
+;; Obj can be <time>, real num or #f, as used in timeout argument for many
+;; procedures.  Returns (<?> <time>)
+(define-cproc absolute-time (obj :optional (t0::<time>? #f)) ::<time>?
+  (let* ([ts::ScmTimeSpec]
+         [pts::ScmTimeSpec* (Scm_ToTimeSpec obj t0 (& ts))])
+    (if (== pts NULL)
+      (return NULL)
+      (return (SCM_TIME (Scm_MakeTime64 (?: t0 (-> t0 type) 'time-utc)
+                                        (-> pts tv_sec)
+                                        (-> pts tv_nsec)))))))
+
+(define (seconds+ t dt)
+  (assume-type t <time>)
+  (assume-type dt <real>)
+  (absolute-time dt t)) ;SRFI-226 comatibility
+
 (define-cproc time->seconds (t::<time>) ;SRFI-18
   Scm_TimeToSeconds)
 
 (define-cproc seconds->time (t::<double>) ;SRFI-18
   Scm_RealSecondsToTime)
 
+(define time-comparator
+  (make-comparator time?
+                   (^[a b] (zero? (compare a b)))
+                   (^[a b] (< (compare a b) 0))
+                   default-hash))
+
 (inline-stub
  (declare-cfn tm_print (obj port::ScmPort* ctx::ScmWriteContext*)
               ::void :static)
@@ -902,12 +929,12 @@
    (printer (c "tm_print")))
 
  (define-cfn tm_print (obj port::ScmPort* _::ScmWriteContext*) ::void
-   (let* ([buf::(.array char (30))]
-          [st::(struct tm*) (SCM_SYS_TM obj)])
+   (let* ([st::(struct tm*) (SCM_SYS_TM obj)]
+          [fmt::(const char*)])
      (.if (not (defined "GAUCHE_WINDOWS"))
-          (strftime buf 30 "%a %b %e %T %Y" st)
-          (strftime buf 30 "%a %b %d %H:%M:%S %Y" st))
-     (Scm_Printf port "#<sys-tm \"%s\">" buf)))
+       (set! fmt "%a %b %e %T %Y")
+       (set! fmt "%a %b %d %H:%M:%S %Y"))
+     (Scm_Printf port "#<sys-tm %S>" (Scm_StrfTime fmt st SCM_FALSE))))
  )
 
 (define-cproc sys-asctime (tm::<sys-tm>)
@@ -924,9 +951,7 @@
   (return (difftime (Scm_GetSysTime time1) (Scm_GetSysTime time0))))
 
 (define-cproc sys-strftime (format::<const-cstring> tm::<sys-tm>)
-  (let* ([tmpbuf::(.array char [256])])
-    (strftime tmpbuf (sizeof tmpbuf) format tm)
-    (return (SCM_MAKE_STR_COPYING tmpbuf))))
+  (return (Scm_StrfTime format tm SCM_FALSE)))
 
 ;; NB: Windows doesn't have gmtime_r/localtime_r, but its gmtime/localtime
 ;; is thread-safe.
@@ -935,19 +960,19 @@
 ;; auto-boxing feature but manually call Scm_Make_sys_tm.
 (define-cproc sys-gmtime (time)
   (.if (defined "GAUCHE_WINDOWS")
-       (let* ([tim::time_t (Scm_GetSysTime time)])
-         (return (Scm_Make_sys_tm (gmtime (& tim)))))
-       (let* ([tim::time_t (Scm_GetSysTime time)]
-              [buf::(struct tm)])
-         (return (Scm_Make_sys_tm (gmtime_r (& tim) (& buf)))))))
+    (let* ([tim::time_t (Scm_GetSysTime time)])
+      (return (Scm_Make_sys_tm (gmtime (& tim)))))
+    (let* ([tim::time_t (Scm_GetSysTime time)]
+           [buf::(struct tm)])
+      (return (Scm_Make_sys_tm (gmtime_r (& tim) (& buf)))))))
 
 (define-cproc sys-localtime (time)
   (.if (defined "GAUCHE_WINDOWS")
-       (let* ([tim::time_t (Scm_GetSysTime time)])
-         (return (Scm_Make_sys_tm (localtime (& tim)))))
-       (let* ([tim::time_t (Scm_GetSysTime time)]
-              [buf::(struct tm)])
-         (return (Scm_Make_sys_tm (localtime_r (& tim) (& buf)))))))
+    (let* ([tim::time_t (Scm_GetSysTime time)])
+      (return (Scm_Make_sys_tm (localtime (& tim)))))
+    (let* ([tim::time_t (Scm_GetSysTime time)]
+           [buf::(struct tm)])
+      (return (Scm_Make_sys_tm (localtime_r (& tim) (& buf)))))))
 
 (define-cproc sys-mktime (tm::<sys-tm>)
   (return (Scm_MakeSysTime (mktime tm))))
@@ -981,7 +1006,7 @@
     (when (< r 0) (Scm_SysError "chmod failed"))))
 
 (inline-stub
- (when "!defined(GAUCHE_WINDOWS)"
+ (.unless (defined "GAUCHE_WINDOWS")
    (define-cproc sys-fchmod (port-or-fd mode::<int>) ::<void>
      (let* ([r::int] [fd::int (Scm_GetPortFd port-or-fd TRUE)])
        (SCM_SYSCALL r (fchmod fd mode))
@@ -994,14 +1019,14 @@
   ::<int>
   (cast void owner) ; suppress unused var warning
   (cast void group) ; suppress unused var warning
-  (.if "!defined(GAUCHE_WINDOWS)"
-       (SCM_SYSCALL SCM_RESULT (chown path owner group))
-       (set! SCM_RESULT 0))
+  (.if (not (defined "GAUCHE_WINDOWS"))
+    (SCM_SYSCALL SCM_RESULT (chown path owner group))
+    (set! SCM_RESULT 0))
   (when (< SCM_RESULT 0) (Scm_SysError "chown failed on %s" path)))
 
 (inline-stub
  ;; lchown
- (when "defined HAVE_LCHOWN"
+ (.when (defined "HAVE_LCHOWN")
    (define-cproc sys-lchown (path::<const-cstring> owner::<int> group::<int>)
      ::<int>
      (SCM_SYSCALL SCM_RESULT (lchown path owner group))
@@ -1070,7 +1095,7 @@
 (define-cproc sys-getppid () ::<int> getppid)
 
 (inline-stub
- (when "!defined(GAUCHE_WINDOWS)"
+ (.unless (defined "GAUCHE_WINDOWS")
    (define-cproc sys-setgid (gid::<int>) ::<int>
      (SCM_SYSCALL SCM_RESULT (setgid gid))
      (when (< SCM_RESULT 0) (Scm_SysError "setgid failed on %d" gid)))
@@ -1088,7 +1113,7 @@
    ;;      if (r < 0) Scm_SysError(\"setpgrp failed\");
    ;;      SCM_RETURN(Scm_MakeInteger(r));"))
 
-   (when "defined HAVE_GETPGID"
+   (.when (defined "HAVE_GETPGID")
      (define-cproc sys-getpgid (pid::<int>) ::<int>
        (SCM_SYSCALL SCM_RESULT (cast int (getpgid pid)))
        (when (< SCM_RESULT 0) (Scm_SysError "getpgid failed")))
@@ -1136,7 +1161,7 @@
                       (set! pglist (SCM_NEW_ATOMIC_ARRAY gid_t size))]
                      [else (Scm_SysError "getgroups failed")])))))
 
-   (when "defined HAVE_SETGROUPS"
+   (.when (defined "HAVE_SETGROUPS")
      (define-cproc sys-setgroups (gids) ::<void>
        (let* ([ngid::int (Scm_Length gids)]
               [glist::gid_t* NULL]
@@ -1202,9 +1227,9 @@
 (define-cproc sys-mkdir (pathname::<const-cstring> mode::<int>) ::<void>
   (let* ([r::int])
     (cast void mode) ; suppress unused var warning
-    (.if "!defined(GAUCHE_WINDOWS)"
-         (SCM_SYSCALL r (mkdir pathname mode))
-         (SCM_SYSCALL r (mkdir pathname)))
+    (.if (not (defined "GAUCHE_WINDOWS"))
+      (SCM_SYSCALL r (mkdir pathname mode))
+      (SCM_SYSCALL r (mkdir pathname)))
     (when (< r 0) (Scm_SysError "mkdir failed on %s" pathname))))
 
 (define-cproc sys-rmdir (pathname::<const-cstring>) ::<void>
@@ -1224,18 +1249,18 @@
                          :optional (no-retry::<boolean> #f))
   ::<int>
   (cast void no-retry) ; suppress unused var warning
-  (.if "defined(GAUCHE_WINDOWS)"
-       (begin (Sleep (* seconds 1000)) (return 0))
-       (let* ([k::u_int (cast (u_int) seconds)]
-              [vm::ScmVM* (Scm_VM)])
-         (while (> k 0)
-           (set! k (sleep k))
-           (SCM_SIGCHECK vm)
-           (when no-retry (break)))
-         (return k))))
+  (.if (defined "GAUCHE_WINDOWS")
+    (begin (Sleep (* seconds 1000)) (return 0))
+    (let* ([k::u_int (cast (u_int) seconds)]
+           [vm::ScmVM* (Scm_VM)])
+      (while (> k 0)
+        (set! k (sleep k))
+        (SCM_SIGCHECK vm)
+        (when no-retry (break)))
+      (return k))))
 
 (inline-stub
- (when "defined(HAVE_NANOSLEEP) || defined(GAUCHE_WINDOWS)"
+ (.when (or (defined "HAVE_NANOSLEEP") (defined "GAUCHE_WINDOWS"))
    (define-cproc sys-nanosleep (nanoseconds
                                 :optional (no-retry::<boolean> #f))
      (let* ([spec::ScmTimeSpec] [rem::ScmTimeSpec]
@@ -1273,11 +1298,11 @@
 
 (define-cproc sys-unlink (pathname::<const-cstring>)
   (let* ([r::int])
-    (.if "defined(GAUCHE_WINDOWS)"
-         ;; Windows doesn't allow unlinking a read-only file.  We don't check
-         ;; an error here, since the error will be caught by next unlink call.
-         (when (not (access pathname F_OK))
-           (chmod pathname #o600)))
+    (.if (defined "GAUCHE_WINDOWS")
+      ;; Windows doesn't allow unlinking a read-only file.  We don't check
+      ;; an error here, since the error will be caught by next unlink call.
+      (when (not (access pathname F_OK))
+        (chmod pathname #o600)))
     (SCM_SYSCALL r (unlink pathname))
     (if (< r 0)
       (begin
@@ -1309,34 +1334,33 @@
  ;; NB. Linux needs _XOPEN_SOURCE defined before unistd.h to get crypt()
  ;; prototype.  However, it screws up something else.  Just for now I
  ;; cast the return value of crypt() to avoid it...such a kludge...
- (when "defined(HAVE_CRYPT)"
+ (.when (defined "HAVE_CRYPT")
    (define-cproc sys-crypt (key::<const-cstring> salt::<const-cstring>)
      ::<const-cstring> (return (cast (const char *) (crypt key salt))))
    )
  )
 
 (inline-stub
- "#ifndef HOSTNAMELEN
-#define HOSTNAMELEN 1024
-#endif")
+ (.when (not (defined HOSTNAMELEN))
+   (.define HOSTNAMELEN 1024)))
 
 (define-cproc sys-gethostname ()
-  (.if "defined HAVE_GETHOSTNAME"
-       (let* ([buf::(.array char [HOSTNAMELEN])] [r::int])
-         (SCM_SYSCALL r (gethostname buf HOSTNAMELEN))
-         (when (< r 0) (Scm_SysError "gethostname failed"))
-         (return (SCM_MAKE_STR_COPYING buf)))
-       ;; TODO: find better alternative
-       (return (SCM_MAKE_STR_IMMUTABLE "localhost"))))
+  (.if (defined "HAVE_GETHOSTNAME")
+    (let* ([buf::(.array char [HOSTNAMELEN])] [r::int])
+      (SCM_SYSCALL r (gethostname buf HOSTNAMELEN))
+      (when (< r 0) (Scm_SysError "gethostname failed"))
+      (return (SCM_MAKE_STR_COPYING buf)))
+    ;; TODO: find better alternative
+    (return (SCM_MAKE_STR_IMMUTABLE "localhost"))))
 
 (define-cproc sys-getdomainname ()
-  (.if "defined HAVE_GETDOMAINNAME"
-       (let* ([buf::(.array char [HOSTNAMELEN])] [r::int])
-         (SCM_SYSCALL r (getdomainname buf HOSTNAMELEN))
-         (when (< r 0) (Scm_SysError "getdomainame failed"))
-         (return (SCM_MAKE_STR_COPYING buf)))
-       ;; TODO: find better alternative
-       (return (SCM_MAKE_STR_IMMUTABLE "local"))))
+  (.if (defined "HAVE_GETDOMAINNAME")
+    (let* ([buf::(.array char [HOSTNAMELEN])] [r::int])
+      (SCM_SYSCALL r (getdomainname buf HOSTNAMELEN))
+      (when (< r 0) (Scm_SysError "getdomainame failed"))
+      (return (SCM_MAKE_STR_COPYING buf)))
+    ;; TODO: find better alternative
+    (return (SCM_MAKE_STR_IMMUTABLE "local"))))
 
 ;; not supported yet:
 ;;  fpathconf lseek pathconf read sysconf write
@@ -1345,7 +1369,7 @@
 ;; symbolic link
 
 (inline-stub
- (when "defined(HAVE_SYMLINK)"
+ (.when (defined "HAVE_SYMLINK")
    (define-cproc sys-symlink (existing::<const-cstring>
                               newpath::<const-cstring>)
      ::<void>
@@ -1355,7 +1379,7 @@
          (Scm_SysError "symlink from %s to %s failed" newpath existing))))
    )
 
- (when "defined(HAVE_READLINK)"
+ (.when (defined "HAVE_READLINK")
    (define-cproc sys-readlink (path::<const-cstring>)
      (let* ([buf::(.array char [1024])] ; TODO: needs to be configured
             [n::int])
@@ -1370,19 +1394,19 @@
 ;; select
 
 (inline-stub
- (when "defined(HAVE_SELECT)"
+ (.when (defined "HAVE_SELECT")
    ;; NB: On Windows, FD_SETSIZE merely indicates the maximum # of socket
    ;; descriptors fd_set can contain, and unrelated to the actual value
    ;; of the descriptor.  This check is thus only valid on unixen.
    (define-cise-stmt check-fd-range
      [(_ fd)
       (let1 fd_ (gensym)
-        `(.if "!defined(GAUCHE_WINDOWS)"
-              (let* ((,fd_ :: int ,fd))
-                (when (or (< ,fd_ 0) (>= ,fd_ FD_SETSIZE))
-                  (Scm_Error "File descriptor value is out of range: %d \
+        `(.unless (defined "GAUCHE_WINDOWS")
+           (let* ((,fd_ :: int ,fd))
+             (when (or (< ,fd_ 0) (>= ,fd_ FD_SETSIZE))
+               (Scm_Error "File descriptor value is out of range: %d \
                          (must be between 0 and %d, inclusive)"
-                             ,fd_ (- FD_SETSIZE 1))))))])
+                          ,fd_ (- FD_SETSIZE 1))))))])
 
    (define-cproc sys-fdset-ref (fdset::<sys-fdset> pf) ::<boolean>
      (setter sys-fdset-set!)
@@ -1434,12 +1458,213 @@
  (define-cproc sys-available-processors () ::<int>
    Scm_AvailableProcessors))
 
+;;---------------------------------------------------------------------
+;; globbing
+
+(with-module gauche)
+
+;; glob-fold provides the fundamental logic of glob.  It does not
+;; depend on filesystems---any tree structure that has "pathname"
+;; will do.
+;;
+;; <glob-pattern> : [<separator>] (<selector> <separator>)* [<separator>]
+;; <selector>     : '**' | <element>*
+;; <element>      : <ordinary> | '*' | '?' | <char-range>
+;; <char-range>   : '[' <char-set-spec> ']'
+;; <ordinary>     : characters except #[,*?\{\}\[\]\\] and <separator>
+;;                  | '\\' <character>
+;;
+;; <separator> splits the components in the path.
+
+(define (glob patterns . opts)
+  (apply glob-fold patterns cons '() opts))
+
+(define sys-glob glob) ;; backward compatibility
+
+(define (glob-fold patterns proc seed :key (separator #[/])
+                                           (folder glob-fs-folder)
+                                           (sorter sort)
+                                           (prefix #f))
+  (let1 r (fold (cut glob-fold-1 <> proc <> separator folder prefix) seed
+                (fold glob-expand-braces '()
+                      (if (list? patterns) patterns (list patterns))))
+    (if sorter (sorter r) r)))
+
+;; NB: we avoid util.match due to the hairy dependency problem.
+(define (glob-fold-1 pattern proc seed separator folder prefix)
+  (define (rec node matcher seed)
+    (cond [(null? matcher) seed]
+          [(eq? (car matcher) '**) (rec* node (cdr matcher) seed)]
+          [(null? (cdr matcher)) (folder proc seed node (car matcher) #f)]
+          [else (folder (^[node seed] (rec node (cdr matcher) seed))
+                        seed node (car matcher) #t)]))
+  (define (rec* node matcher seed)
+    (fold (cut rec* <> matcher <>)
+          (rec node matcher seed)
+          (folder cons '() node #/^[^.].*$/ #t)))
+  (let1 p (glob-prepare-pattern pattern separator prefix)
+    (rec (car p) (cdr p) seed)))
+
+(define (glob-prepare-pattern pattern separator prefix)
+  (define (f comp)
+    (cond [(equal? comp "") 'dir?]    ; pattern ends with '/'
+          [(equal? comp "**") '**]
+          [else (glob-component->regexp comp)]))
+  (let1 comps (string-split pattern separator)
+    (if (equal? (car comps) "")
+      (cons #t (map f (cdr comps)))
+      (cons prefix (map f comps)))))
+
+;; */*.{c,scm} -> '(*/*.c */*.scm)
+;;
+;; NB: we first expand the braces to separate patterns.  This is how
+;; zsh and tcsh handles {...}.  However, it is not good in terms of
+;; performance, since the common prefix are searched mulitple times.
+;; Hopefully we'll put some optimization here, making single traversal
+;; for the common prefix.
+;;
+;; The treatment of backslashes is tricky.
+;;
+(define (glob-expand-braces pattern seed)
+  (define (parse str pres level)
+    (let loop ([str str]
+               [segs pres])
+      (cond
+       [(rxmatch #/[{}]/ str) =>
+        (^m
+          (cond [(equal? (m 0) "{")
+                 (receive (ins post) (parse (m'after) '("") (+ level 1))
+                   (loop post
+                         (fold (^[seg seed]
+                                 (fold (^[in seed]
+                                         (cons (string-append seg in) seed))
+                                       seed ins))
+                               '()
+                               (map (cute string-append <> (m'before)) segs))))]
+                [(= level 0)
+                 (error "extra closing curly-brace in glob pattern:" pattern)]
+                [else         ; closing curly-brace
+                 (values (fold expand '()
+                               (map (cute string-append <> (m'before)) segs))
+                         (m'after))]))]
+       [(= level 0) (values (map (cute string-append <> str) segs) #f)]
+       [else (error "unclosed curly-brace in glob pattern:" pattern)])))
+  (define (expand pat seed)
+    (let1 segs (string-split pat #\,)
+      (if (null? seed) segs (append segs seed))))
+  (if (string-scan pattern #\{)
+    (append (values-ref (parse pattern '("") 0) 0) seed)
+    (cons pattern seed)))
+
+(define (glob-component->regexp pattern) ; "**" is already excluded
+  (define n read-char)
+  (define nd '(comp . #[.]))
+  (define ra '(rep 0 #f any))
+  (regexp-compile
+   (regexp-optimize
+    (with-input-from-string pattern
+      (^[]
+        (define (element0 ch ct)
+          (case ch
+            [(#\*) (element0* (n) ct)]
+            [(#\?) `(,nd ,@(element1 (n) ct))]
+            [else (element1 ch ct)]))
+        (define (element0* ch ct)
+          (case ch
+            [(#\*) (element0* (n) ct)]
+            [(#\?) `(,nd ,ra ,@(element1 (n) ct))]
+            [(#\.) `(,nd ,ra #\. ,@(element1 (n) ct))]
+            [else `((rep 0 1 (seq ,nd ,ra))
+                    ,@(element1 ch ct))]))
+        (define (element1 ch ct)
+          (cond [(eof-object? ch) '(eol)]
+                [(eqv? ch #\*) `(,ra ,@(element1* (n) ct))]
+                [(eqv? ch #\?) `(any ,@(element1 (n) ct))]
+                [(eqv? ch #\[)
+                 (case (peek-char)
+                   ;; we have to treat [!...] as [^...]
+                   [(#\!) (n)
+                    (let1 cs (read-char-set (current-input-port))
+                      (cons (char-set-complement! cs) (element1 (n) ct)))]
+                   [else
+                    (let1 cs (read-char-set (current-input-port))
+                      (cons cs (element1 (n) ct)))])]
+                [else (cons ch (element1 (n) ct))]))
+        (define (element1* ch ct)
+          (case ch
+            [(#\*) (element1* (n) ct)]
+            [else  (element1 ch ct)]))
+        `(0 #f bol ,@(element0 (n) '())))))))
+
+;; if rx is just test perfect match, e.g. #/^string$/, returns
+;; string portion.
+(define (fixed-regexp? rx)
+  (let1 ast (regexp-ast rx)
+    (and (> (length ast) 4)
+         (eq? (caddr ast) 'bol)
+         (let loop ([cs (cdddr ast)] [r '()])
+           (cond [(null? (cdr cs))
+                  (and (eq? (car cs) 'eol) (list->string (reverse r)))]
+                 [(char? (car cs)) (loop (cdr cs) (cons (car cs) r))]
+                 [else #f])))))
+
+(define (make-glob-fs-fold :key (root-path #f) (current-path #f))
+  ;; NB: We don't use cond-expand, for precompilation may be done on
+  ;; different architecture.
+  (let1 separ (if (#/mingw/ (gauche-architecture)) "\\" "/")
+    (define (ensure-dirname s)
+      (and s
+           (or (and-let* ([len (string-length s)]
+                          [ (> len 0) ]
+                          [ (not (eqv? (string-ref s (- len 1))
+                                       (string-ref separ 0))) ])
+                 (string-append s separ))
+               s)))
+    (define root-path/    (ensure-dirname root-path))
+    (define current-path/ (ensure-dirname current-path))
+    (^[proc seed node regexp non-leaf?]
+      (let1 prefix (case node
+                     [(#t) (or root-path/ separ)]
+                     [(#f) (or current-path/ "")]
+                     [else (string-append node separ)])
+        ;; NB: we can't use filter, for it is not built-in.
+        ;; also we can't use build-path from the same reason.
+        ;; We treat fixed-regexp specially, since it allows
+        ;; us not to search the directory---sometimes the directory
+        ;; has 'x' permission but not 'r' permission, and it would be
+        ;; unreasonable if we fail to go down the path even if we know
+        ;; the exact name.
+        (cond [(eq? regexp 'dir?) (proc prefix seed)]
+              [(fixed-regexp? regexp)
+               => (^s (let1 full (string-append prefix s)
+                        (if (and (file-exists? full)
+                                 (or (not non-leaf?)
+                                     (file-is-directory? full)))
+                          (proc full seed)
+                          seed)))]
+              [else
+               (fold (^[child seed]
+                       (or (and-let* ([ (regexp child) ]
+                                      [full (string-append prefix child)]
+                                      [ (or (not non-leaf?)
+                                            (file-is-directory? full)) ])
+                             (proc full seed))
+                           seed))
+                     seed
+                     (sys-readdir (case node
+                                    [(#t) (or root-path/ "/")]
+                                    [(#f) (or current-path/ ".")]
+                                    [else node])))])))
+    ))
+
+(define glob-fs-folder (make-glob-fs-fold))
+
 ;;;
 ;;; Windows-specific utility
 ;;;
 
 (inline-stub
- (when "defined(GAUCHE_WINDOWS)"
+ (.when (defined "GAUCHE_WINDOWS")
    ;; Windows HANDLE wrapper
    ;; We use foreign pointer for HANDLE.  If the type of handle is known at
    ;; the creation time, it is attached as the foreign pointer attribute
@@ -1519,8 +1744,7 @@
 ;; http://github.com/k-takata/ptycheck.
 (with-module gauche.internal
   (define (%sys-mintty? port-or-fd)
-    (and-let* ([ (global-variable-bound? (find-module 'gauche)
-                                         'sys-win-pipe-name) ]
+    (and-let* ([ (module-binds? 'gauche 'sys-win-pipe-name) ]
                [n (sys-win-pipe-name port-or-fd)])
       (boolean (#/^\\msys-[\da-f]+-pty\d+-(to|from)-master.*$/ n))))
   )
@@ -1571,10 +1795,21 @@
 ;; The correct way is to get {N consecutive backslashes + double-quote},
 ;; then replace it to {2N+1 consecutive backslashes + double-quote}.
 ;;
-(define (%sys-escape-windows-command-line s)
+;; The flag BATFILEP is true when the command to be executed is a batch
+;; file.  In that case, the command line is parsed differently and the rule
+;; is so twisted that it is virtually impossible to make it right.
+;; For now, we reject arguments that contain 'unsafe' characters.
+;; Cf. https://nvd.nist.gov/vuln/detail/CVE-2024-3566
+(define (%sys-escape-windows-command-line s batfilep)
   (cond [(not (string? s))
          (%sys-escape-windows-command-line (write-to-string s))]
         [(equal? s "") "\"\""]
+        [batfilep
+         (when (#/[()%!^<>&|\"]/ s)
+           (errorf "It is unsafe to pass argument ~s to BAT or CMD file." s))
+         (if (string-scan s #\space)
+           (string-append "\"" s "\"")
+           s)]
         [(#/[&<>\[\]{}^=\;!\'+,`~\s]/ s)
          ($ string-append "\""
             ($ regexp-replace-all #/(\\*)\"/ s
