@@ -1,7 +1,7 @@
 ;;;
 ;;; libtype.scm - type-related stuff
 ;;;
-;;;   Copyright (c) 2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2021-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -31,7 +31,7 @@
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;
 
-;; This must be the first form to prevents generation of *.sci file
+;; This must be the first form to prevent generation of *.sci file
 (declare)
 
 ;; In Gauche, types are a data structure that appears in both compile-time
@@ -96,11 +96,11 @@
 
 
 (inline-stub
- (declcode
-  (.include "gauche/priv/classP.h")
-  (.include "gauche/priv/nativeP.h")
-  (.include "gauche/priv/memoP.h")
-  (.include "gauche/priv/typeP.h"))
+ (.include "gauche/priv/configP.h"
+           "gauche/priv/classP.h"
+           "gauche/priv/nativeP.h"
+           "gauche/priv/memoP.h"
+           "gauche/priv/typeP.h")
 
  ;; Metaclass: <type-constructor-meta>
  ;;   Its instance is ScmTypeConstructor.  Provides the following slots.
@@ -152,7 +152,7 @@
  ;; initialization until the type is used.
  (define-ctype ScmDescriptiveType
    ::(.struct ScmDescriptiveTypeRec
-              (hdr::ScmInstance
+              (SCM_INSTANCE_HEADER::||
                name::ScmObj
                constructorArgs::ScmObj)))
 
@@ -198,18 +198,15 @@
  (define-cproc %dump-type-table () ::<void> ; for debug
    (Scm__MemoTableDump type-table SCM_CURERR))
 
- ;; TRANSIENT: for size and alignemnt slots, we'll change them to size_t once
- ;; 0.9.11 is released.  (0.9.10's precompiler doesn't support <size_t>
- ;; stub types.
  (define-ctype ScmNativeType
    ::(.struct ScmNativeTypeRec
-              (hdr::ScmInstance
+              (SCM_INSTANCE_HEADER::||
                name::ScmObj
                c-of-type::(.function (obj) ::int *)
                super::ScmObj
                c-type-name::(const char *)
-               size::u_long
-               alignment::u_long)))
+               size::size_t
+               alignment::size_t)))
 
  (define-cclass <native-type> :base :private :no-meta
    "ScmNativeType*" "Scm_NativeTypeClass"
@@ -217,8 +214,8 @@
    ((name)
     (super)
     (c-type-name :type <const-cstring>)
-    (size :type <ulong>)
-    (alignment :type <ulong>))
+    (size :type <size_t>)
+    (alignment :type <size_t>))
    (printer (Scm_Printf port "#<native-type %S>" (-> (SCM_NATIVE_TYPE obj) name))))
  )
 
@@ -229,6 +226,16 @@
   (unless (slot-bound? c 'supertype?)
     (slot-set! c 'supertype? (^[type sub] #f))))
 
+;;;
+;;; type? obj
+;;;
+
+;; Returns obj if it's either a class metaobject, a descriptive type,
+;; a native type, or a proxy type.  It's the same as (is-a? obj <type>)
+;; but this is more concise and clear.
+(define-cproc type? (obj) ::<boolean> :constant
+  (return (SCM_ISA obj SCM_CLASS_TYPE)))
+
 ;;;
 ;;; of-type? obj type
 ;;;
@@ -510,20 +517,24 @@
 
 ;; Internal API - called from procedure-type (libproc)
 ;; Compute #<^ ...> type from the information available in the procedure.
+;; Once this info is computed, it is cached in PROC.
 (define (compute-procedure-type proc)
   (define (%procedure-type proc)
-    (if-let1 clinfo (case-lambda-info proc)
+    (if-let1 clinfo (case-lambda-decompose proc)
       (construct-type </> (map (^v (%procedure-type (caddr v))) clinfo))
-      (let1 top (%class->proxy <top>)
-        (construct-type <^>
-                        `(,@(make-list (~ proc'required) top)
-                          ,@(if (~ proc'optional) '(*) '())
-                          -> *)))))
+      (or (and-let* ([ (closure? proc) ]
+                     [code (closure-code proc)])
+            ((with-module gauche.internal compiled-code-type) code))
+          ;; Fallback - if we don't have detailed type info, just use
+          ;; # of arguments.
+          (let1 top (%class->proxy <top>)
+            (construct-procedure-type (make-list (~ proc'required) top)
+                                      (~ proc'optional)
+                                      '*)))))
   (define (%method-type meth)
-    (construct-type <^>
-                    `(,@(map %class->proxy (~ meth'specializers))
-                      ,@(if (~ meth'optional) '(*) '())
-                      -> *)))
+    (construct-procedure-type (map %class->proxy (~ meth'specializers))
+                              (~ meth'optional)
+                              '*))
   (define (%generic-type gf)
     (construct-type </> (map %method-type (~ gf'methods))))
 
@@ -552,6 +563,16 @@
     (Scm_Error "Class %S doesn't have a known global binding and can't be used \
                 in a type expression." klass)))
 
+;; Utility.  Called from compiler, too.
+(define (construct-procedure-type argtypes ; (<List> <type>)
+                                  has-optional? ; <boolean>
+                                  rettypes) ; (<List> <type>) or '*
+  (let1 rts (if (eq? rettypes '*) '(*) rettypes)
+    ($ construct-type <^>
+       (if has-optional?
+         `(,@argtypes * -> ,@rts)
+         `(,@argtypes -> ,@rts)))))
+
 (define-class <^> (<descriptive-type>)
   ((arguments :init-keyword :arguments)    ; <Tuple>
    (results   :init-keyword :results))     ; <Tuple>
@@ -743,6 +764,31 @@
   :validator validate-Vector
   :subtype? (make-subtype-Seq <vector>))
 
+;;;
+;;; Class: <Assortment>
+;;;   A type consists of a set of concrete objects.
+;;;   If it has only one member, you can think of it as a singleton type.
+;;;   If it has multiple members, it's a union of singleton types.
+;;;   The members (instances) are sorted by default-comparator so that
+;;;   equivalence is easily tested.
+;;;
+
+(define-class <Assortment> (<descriptive-type>)
+  ((instances :init-keyword :instance))
+  :metaclass <type-constructor-meta>
+  :initializer (^[type args]
+                 (define objs (map unwrap-syntax args))
+                 (slot-set! type 'name
+                            (string->symbol
+                             (string-append
+                              "<Assortment " (x->string objs) ">")))
+                 (slot-set! type 'instances (sort objs)))
+  :deconstructor (^[type] (list (~ type'instances)))
+  :validator (^[type obj] (boolean (memv obj (~ type'instances))))
+  :subtype? (^[type super] (every (^[obj] (of-type? obj super))
+                                  (~ type'instances)))
+  :supertype? (^[type sub] #f))
+
 ;;;
 ;;; Types for bridging Scheme and C
 ;;;
@@ -875,24 +921,6 @@
        (return (not oor)))
      (return (and (SCM_INTP obj) (>= (SCM_INT_VALUE obj) 0)))))
 
- (define-cvar SCM_SIZE_MAX :static)
- (define-cvar SCM_SSIZE_MAX :static)
- (define-cvar SCM_SSIZE_MIN :static)
- (define-cvar SCM_PTRDIFF_MAX :static)
- (define-cvar SCM_PTRDIFF_MIN :static)
- (define-cfn native_sizetP (obj) ::int :static
-   (return (and (SCM_INTEGERP obj)
-                (>= (Scm_Sign obj) 0)
-                (<= (Scm_NumCmp obj SCM_SIZE_MAX) 0))))
- (define-cfn native_ssizetP (obj) ::int :static
-   (return (and (SCM_INTEGERP obj)
-                (>= (Scm_NumCmp obj SCM_SSIZE_MIN) 0)
-                (<= (Scm_NumCmp obj SCM_SSIZE_MAX) 0))))
- (define-cfn native_ptrdifftP (obj) ::int :static
-   (return (and (SCM_INTEGERP obj)
-                (>= (Scm_NumCmp obj SCM_PTRDIFF_MIN) 0)
-                (<= (Scm_NumCmp obj SCM_PTRDIFF_MAX) 0))))
-
  ;; we don't range-check flonums
  (define-cfn native_realP (obj) ::int :static
    (return (SCM_REALP obj)))
@@ -912,12 +940,6 @@
    (return (SCM_CLOSUREP obj)))
 
  (initcode
-  (set! SCM_SIZE_MAX (Scm_MakeIntegerU64 SIZE_MAX))
-  (set! SCM_SSIZE_MAX (Scm_MakeInteger64 SSIZE_MAX))
-  (set! SCM_SSIZE_MIN (Scm_MakeInteger64 (- (- SSIZE_MAX) 1)))
-  (set! SCM_PTRDIFF_MAX (Scm_MakeInteger64 PTRDIFF_MAX))
-  (set! SCM_PTRDIFF_MIN (Scm_MakeInteger64 (- (- PTRDIFF_MAX) 1)))
-
   (define-native-type <fixnum>  SCM_CLASS_INTEGER ScmSmallInt native_fixnumP)
   (define-native-type <short>   SCM_CLASS_INTEGER short native_shortP)
   (define-native-type <ushort>  SCM_CLASS_INTEGER u_short native_ushortP)
@@ -933,9 +955,12 @@
   (define-native-type <uint32>  SCM_CLASS_INTEGER uint32_t native_u32P)
   (define-native-type <int64>   SCM_CLASS_INTEGER int64_t native_s64P)
   (define-native-type <uint64>  SCM_CLASS_INTEGER uint64_t native_u64P)
-  (define-native-type <size_t>  SCM_CLASS_INTEGER size_t native_sizetP)
-  (define-native-type <ssize_t> SCM_CLASS_INTEGER ssize_t native_ssizetP)
-  (define-native-type <ptrdiff_t> SCM_CLASS_INTEGER ptrdiff_t native_ptrdifftP)
+  (define-native-type <size_t>  SCM_CLASS_INTEGER size_t Scm_IntegerFitsSizeP)
+  (define-native-type <ssize_t> SCM_CLASS_INTEGER ssize_t Scm_IntegerFitsSsizeP)
+  (define-native-type <ptrdiff_t> SCM_CLASS_INTEGER ptrdiff_t Scm_IntegerFitsPtrdiffP)
+  (define-native-type <off_t> SCM_CLASS_INTEGER off_t Scm_IntegerFitsOffsetP)
+  (define-native-type <intptr_t> SCM_CLASS_INTEGER intptr_t Scm_IntegerFitsIntptrP)
+  (define-native-type <uintptr_t> SCM_CLASS_INTEGER uintptr_t Scm_IntegerFitsUintptrP)
   (define-native-type <float>   SCM_CLASS_REAL float native_realP)
   (define-native-type <double>  SCM_CLASS_REAL double native_realP)
   (define-native-type <const-cstring> SCM_CLASS_STRING "const char*" native_cstrP)
@@ -949,18 +974,14 @@
 ;;; Make exported symbol visible from outside
 ;;;
 
-;; TRANSIENT: the inlinable flag is only necessary in 0.9.10 -> 0.9.11
-;; transition, for define-class doesn't create inlinable binding in 0.9.10 but
-;; it does in 0.9.11.
 (let ((xfer (with-module gauche.internal %transfer-bindings)))
   (xfer (current-module)
         (find-module 'gauche)
         '(<type-constructor-meta>
           <descriptive-type>
           <native-type>
-          <^> </> <?> <Tuple> <List> <Vector>
-          subtype? of-type?)
-        '(inlinable))
+          <^> </> <?> <Tuple> <List> <Vector> <Assortment>
+          type? subtype? of-type?))
   (xfer (current-module)
         (find-module 'gauche.internal)
         '(construct-type
@@ -970,4 +991,5 @@
           proxy-type-id
           ;; followings are called from procedure-type (libproc)
           reconstruct-procedure-type
-          compute-procedure-type)))
+          compute-procedure-type
+          construct-procedure-type)))
