@@ -1,7 +1,7 @@
 /*
  * load.c - load a program
  *
- *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,6 +33,7 @@
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
+#include "gauche/priv/configP.h"
 #include "gauche/port.h"
 #include "gauche/priv/builtin-syms.h"
 #include "gauche/priv/readerP.h"
@@ -503,7 +504,7 @@ static void unlock_dlobj(ScmDLObj *dlo)
 
 /* Find NAME in the looked-up entries.
    NAME must begin with '_'.
-   Assuming the caller holding the lock of OBJ. */
+   Assuming the caller holding the lock of DLO. */
 static ScmObj find_entry(ScmDLObj *dlo, ScmString *name)
 {
     ScmDictEntry *e = Scm_HashCoreSearch(&dlo->entries, (intptr_t)name,
@@ -512,6 +513,19 @@ static ScmObj find_entry(ScmDLObj *dlo, ScmString *name)
     else   return SCM_FALSE;
 }
 
+/* Register name => fptr entry in dlo.  Assuming the caller holding the
+   lock of DLO.  Returns a foreign pointer wrapping ptr. */
+static ScmObj add_entry(ScmDLObj *dlo, ScmString *name, void *ptr)
+{
+    ScmObj fptr = Scm_MakeForeignPointer(ldinfo.dlptr_class, ptr);
+    Scm_ForeignPointerAttrSet(SCM_FOREIGN_POINTER(fptr),
+                              SCM_SYM_NAME, SCM_OBJ(name));
+    ScmDictEntry *e = Scm_HashCoreSearch(&dlo->entries, (intptr_t)name,
+                                         SCM_DICT_CREATE);
+    (void)SCM_DICT_SET_VALUE(e, fptr);
+    return fptr;
+}
+
 /* lookup the symbol within DLO.
    NAME must begin with '_'.   We look up both with and without '_'.
    Assuming the caller holding the lock of OBJ. */
@@ -530,12 +544,7 @@ static ScmObj lookup_entry(ScmDLObj *dlo, ScmString *name)
                 return SCM_FALSE; /* not found */
             }
         }
-        fptr = Scm_MakeForeignPointer(ldinfo.dlptr_class, ptr);
-        Scm_ForeignPointerAttrSet(SCM_FOREIGN_POINTER(fptr),
-                                  SCM_SYM_NAME, SCM_OBJ(name));
-        ScmDictEntry *e = Scm_HashCoreSearch(&dlo->entries, (intptr_t)name,
-                                             SCM_DICT_CREATE);
-        (void)SCM_DICT_SET_VALUE(e, fptr);
+        fptr = add_entry(dlo, name, ptr);
     }
     return fptr;
 }
@@ -620,7 +629,8 @@ void Scm_RegisterPrelinked(ScmString *dsoname,
 
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.dso_mutex);
     for (int i=0; initfns[i] && initfn_names[i]; i++) {
-        lookup_entry(dlo, SCM_STRING(SCM_MAKE_STR_IMMUTABLE(initfn_names[i])));
+        add_entry(dlo, SCM_STRING(SCM_MAKE_STR_IMMUTABLE(initfn_names[i])),
+                  initfns[i]);
     }
     ldinfo.dso_prelinked = Scm_Cons(SCM_OBJ(dsoname), ldinfo.dso_prelinked);
     (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.dso_mutex);
@@ -1178,58 +1188,6 @@ ScmObj Scm_CurrentLoadHistory() { return PARAM_REF(Scm_VM(), load_history); }
 ScmObj Scm_CurrentLoadNext()    { return PARAM_REF(Scm_VM(), load_next); }
 ScmObj Scm_CurrentLoadPort()    { return PARAM_REF(Scm_VM(), load_port); }
 
-/*------------------------------------------------------------------
- * Compatibility stuff
- */
-
-#if GAUCHE_API_VERSION < 98
-/* TRANSIENT: Pre-0.9 Compatibility routine.  Kept for the binary compatibility.
-   Will be removed on 1.0 */
-void Scm__LoadFromPortCompat(ScmPort *port, int flags)
-{
-    Scm_LoadFromPort(port, flags|SCM_LOAD_PROPAGATE_ERROR, NULL);
-}
-
-/* TRANSIENT: Pre-0.9 Compatibility routine.  Kept for the binary compatibility.
-   Will be removed on 1.0 */
-int  Scm__LoadCompat(const char *file, int flags)
-{
-    return (0 == Scm_Load(file, flags|SCM_LOAD_PROPAGATE_ERROR, NULL));
-}
-
-/* TRANSIENT: Pre-0.9 Compatibility routine.  Kept for the binary compatibility.
-   Will be removed on 1.0 */
-ScmObj Scm__RequireCompat(ScmObj feature)
-{
-    Scm_Require(feature, SCM_LOAD_PROPAGATE_ERROR, NULL);
-    return SCM_TRUE;
-}
-
-/* TRANSIENT: This is entirely moved to Scheme (libeval.scm).  The entry is
-   kept only for the binary compatibility. */
-ScmObj Scm_VMLoadFromPort(ScmPort *port SCM_UNUSED,
-                          ScmObj next_paths SCM_UNUSED,
-                          ScmObj env SCM_UNUSED,
-                          int flags SCM_UNUSED)
-{
-    Scm_Error("[internal] Scm_VMLoadFromPort() is obsoleted; call load-from-port Scheme procedure.");
-    return SCM_UNDEFINED;
-}
-
-/* TRANSIENT: Kept for the binary compatibility; the feature
-   is in libeval.scm now. */
-void Scm__RecordLoadStart(ScmObj load_file_path SCM_UNUSED)
-{
-}
-
-/* TRANSIENT: Kept for the binary compatibility; not used anymore. */
-ScmObj Scm_LoadMainScript()
-{
-    return SCM_UNDEFINED;
-}
-#endif /*GAUCHE_API_VERSION < 98*/
-
-
 /*------------------------------------------------------------------
  * Initialization
  */
