@@ -1,7 +1,7 @@
 /*
  * main.c - interpreter main program
  *
- *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -32,6 +32,7 @@
  */
 
 #include "gauche.h"
+#include "gauche/priv/configP.h"
 
 #include <signal.h>
 #include <ctype.h>
@@ -95,17 +96,17 @@ void usage(int errorp)
             "Usage: gosh [-biqV][-I<path>][-A<path>][-u<module>][-m<module>][-l<file>][-L<file>][-e<expr>][-E<expr>][-p<type>][-F<feature>][-r<standard>][-f<flag>][--] [file]\n"
             "Options:\n"
             "  -V       Prints version and exits.\n"
-            "  -h       Show this message to stdout.\n"
+            "  -h       Shows this message to stdout.\n"
             "  -b       Batch mode.  Doesn't print prompts.  Supersedes -i.\n"
             "  -i       Interactive mode.  Forces to print prompts.\n"
             "  -q       Doesn't read the default initialization file.\n"
             "  -I<path> Adds <path> to the head of the load path list.\n"
             "  -A<path> Adds <path> to the tail of the load path list.\n"
-            "  -u<module> (use) load and import <module>\n"
+            "  -u<module> (use) loads and imports <module>\n"
             "  -l<file> Loads <file> before executing the script file or\n"
             "           entering repl.\n"
             "  -L<file> Like -l, but doesn't complain if <file> doesn't exist.\n"
-            "  -e<expr> Evaluate Scheme expression <expr> before executing\n"
+            "  -e<expr> Evaluates Scheme expression <expr> before executing\n"
             "           the script file or entering repl.\n"
             "  -E<expr> Similar to -e, but reads <expr> as if it is surrounded\n"
             "           by parenthesis.\n"
@@ -118,45 +119,53 @@ void usage(int errorp)
             "  -F<feature> Makes <feature> available in cond-expand forms\n"
             "           If <feature> begins with '-', it removes the feature (without\n"
             "           preceding minus) instead.\n"
-            "  -v<version> If <version> is not the running Gauche's version, but\n"
+            "  -v<version>\n"
+            "  -v:<version>\n"
+            "           If <version> is not the running Gauche's version, but\n"
             "           the specified version is installed in the system, execute\n"
             "           that version's gosh instead.  This is useful when you want\n"
             "           to invoke a specific version of Gauche.  <Version> must be\n"
-            "           0.9.6 or later.\n"
+            "           0.9.6 or later.  Without ':' before <version>, gosh aborts\n"
+            "           if the given version is not found; with ':', the invoked\n"
+            "           gosh runs (with warning) in that case.\n"
             "  -r<standard>  Starts gosh with the default environment defined\n"
             "           in RnRS, where n is determined by <standard>.  The following\n"
             "           values are supported as <standard>.\n"
             "      7               R7RS (R7RS-small)\n"
             "  -f<flag> Sets various flags\n"
             "      case-fold       uses case-insensitive reader (as in R5RS)\n"
-            "      include-verbose report while including files\n"
-            "      load-verbose    report while loading files\n"
-            "      no-inline       don't inline procedures & constants (combined\n"
+            "      include-verbose reports while including files\n"
+            "      load-verbose    reports while loading files\n"
+            "      no-inline       doesn't inline procedures & constants (combined\n"
             "                      no-inline-globals, no-inline-locals,\n"
             "                      no-inline-constants and no-inline-setters.)\n"
             "      no-inline-constants\n"
-            "                      don't inline constants.\n"
+            "                      doesn't inline constants.\n"
             "      no-inline-globals\n"
-            "                      don't inline global procedures.\n"
+            "                      doesn't inline global procedures.\n"
             "      no-inline-locals\n"
-            "                      don't inline local procedures.\n"
+            "                      doesn't inline local procedures.\n"
             "      no-inline-setters\n"
-            "                      don't inline setter procedures (srfi-17).\n"
+            "                      doesn't inline setter procedures (srfi-17).\n"
             "      no-lambda-lifting-pass\n"
-            "                      don't run lambda lifting pass.\n"
+            "                      doesn't run lambda lifting pass.\n"
             "      no-post-inline-pass\n"
-            "                      don't run post-inline optimization pass.\n"
-            "      no-source-info  don't preserve source information for debugging\n"
+            "                      doesn't run post-inline optimization pass.\n"
+            "      no-source-info  doesn't preserve source information for debugging\n"
             "      read-edit\n"
-            "                      enable input-editing mode, if terminal supports it.\n"
+            "                      enables input-editing mode, if terminal supports it.\n"
             "      no-read-edit\n"
-            "                      disable input-editing mode.\n"
+            "                      disables input-editing mode.\n"
             "      safe-string-cursors\n"
             "                      performs extra validation for use of string cursors\n"
             "      test            test mode, to run gosh inside the build tree\n"
             "                      without installing.\n"
             "      warn-legacy-syntax\n"
-            "                      print warning when legacy Gauche syntax is encountered\n"
+            "                      prints warning when legacy Gauche syntax is encountered\n"
+            "      warn-srfi-feature-id\n"
+            "                      prints warning when srfi-N is used as a feature id.\n"
+            "      no-warn-srfi-feature-id\n"
+            "                      doesn't print warning when srfi-N is used as a feature id.\n"
             "Environment variables:\n"
             "  GAUCHE_AVAILABLE_PROCESSORS\n"
             "      Value must be an integer.  If set, it overrides the number of\n"
@@ -245,19 +254,42 @@ void version(void)
     Scm_Exit(0);
 }
 
-void invoke_other_version(const char *version, int argc, char **argv)
+void maybe_invoke_other_version(const char *version, int argc, char **argv)
 {
+    int allow_fallback = FALSE;
+    if (*version == ':') {
+        allow_fallback = TRUE;
+        version++;
+    }
+    if (strcmp(version, GAUCHE_VERSION) == 0) return;
+
     static ScmObj invoke_other_version = SCM_UNDEFINED;
     SCM_BIND_PROC(invoke_other_version, "%invoke-other-version",
                   Scm_GaucheInternalModule());
+
     ScmEvalPacket epkt;
-    Scm_Apply(invoke_other_version,
-              SCM_LIST2(SCM_MAKE_STR_COPYING(version),
-                        Scm_CStringArrayToList((const char**)argv, argc, 0)),
-              &epkt);
-    /* %invoke-other-version won't return.  If we're here,
-       we even failed to call it. */
-    Scm_Panic("Failed to call %%invoke-other-version.  Installation problem?");
+    int r = Scm_Apply(invoke_other_version,
+                      SCM_LIST2(SCM_MAKE_STR_COPYING(version),
+                                Scm_CStringArrayToList((const char**)argv, argc, 0)),
+                      &epkt);
+    /* If %invoke-other-version returns normally, it couldn't find the
+       specified version.
+     */
+    if (r < 0) {
+        Scm_Panic("Failed to call %%invoke-other-version. "
+                  "Installation problem?");
+    }
+    SCM_ASSERT(r == 1);
+    if (!allow_fallback) {
+        Scm_Printf(SCM_CURERR,
+                   "No installed Gauche with version %s under %A.\n",
+                   version, epkt.results[0]);
+        Scm_Exit(1);
+    } else {
+        Scm_Warn("No installed Gauche with version %s under %A. "
+                 "Using the current version.",
+                 version, epkt.results[0]);
+    }
 }
 
 void further_options(const char *optarg)
@@ -307,6 +339,12 @@ void further_options(const char *optarg)
         /* standard input port is already initialized, so we should switch it */
         Scm_SetPortReaderLexicalMode(SCM_CURIN, SCM_INTERN("warn-legacy"));
     }
+    else if (strcmp(optarg, "warn-srfi-feature-id") == 0) {
+        SCM_VM_COMPILER_FLAG_CLEAR(vm, SCM_COMPILE_SRFI_FEATURE_ID);
+    }
+    else if (strcmp(optarg, "no-warn-srfi-feature-id") == 0) {
+        SCM_VM_COMPILER_FLAG_SET(vm, SCM_COMPILE_SRFI_FEATURE_ID);
+    }
     else if (strcmp(optarg, "test") == 0) {
         test_mode = TRUE;
     }
@@ -348,7 +386,9 @@ void further_options(const char *optarg)
                 "-fno-inline-constants, -fno-inline-setters, -fno-source-info, "
                 "-fno-post-inline-pass, -fno-lambda-lifting-pass, "
                 "-fread-edit, -fno-read-edit, "
-                "-fsafe-string-cursors, -fwarn-legacy-syntax, or -ftest\n");
+                "-fsafe-string-cursors, -fwarn-legacy-syntax, "
+                "-fwarn-srfi-feature-id, -fno-warn-srfi-feature-id, "
+                "or -ftest\n");
         exit(1);
     }
 }
@@ -405,10 +445,7 @@ int parse_options(int argc, char *argv[])
                                  SCM_MAKE_STR_COPYING(optarg), pre_cmds);
             break;
         case 'v':
-            if (strcmp(optarg, GAUCHE_VERSION) != 0) {
-                invoke_other_version(optarg, argc, argv);
-                /*NOTREACHED*/
-            }
+            maybe_invoke_other_version(optarg, argc, argv);
             break;
         case '-': break;
         case 'h': usage(FALSE); break;
@@ -550,6 +587,13 @@ void error_exit(ScmObj c)
     } else {
         Scm_Printf(SCM_CURERR, "gosh: %S: %A\n", Scm_ConditionTypeName(c), m);
     }
+
+    ScmObj print_additional_error_heading_proc = SCM_UNDEFINED;
+    SCM_BIND_PROC(print_additional_error_heading_proc,
+                  "print-additional-error-heading", Scm_GaucheModule());
+    Scm_Apply(print_additional_error_heading_proc,
+              SCM_LIST2(c, SCM_OBJ(SCM_CURERR)),
+              NULL);
     Scm_Exit(1);
 }
 
@@ -860,6 +904,7 @@ int main(int ac, char **av)
     } else {
         /* For REPL, we leave command-line to the default '(""). */
         Scm_InitCommandLine2(argc, (const char**)argv, SCM_COMMAND_LINE_OS);
+        Scm_InitCommandLine2(1, (const char*[]){""}, SCM_COMMAND_LINE_SCRIPT);
     }
 
     process_command_args(Scm_Reverse(pre_cmds));
