@@ -10,6 +10,7 @@
 ;;
 
 (use gauche.collection)
+(use gauche.charconv)
 (use gauche.sequence)
 (use gauche.dictionary)
 (use gauche.hook)
@@ -19,14 +20,9 @@
 (use gauche.generator)
 (use gauche.interpolate)
 (use gauche.process)
-(cond-expand
- [gauche-0.9.9 (use gauche.stringutil) (use gauche.portutil)]
- [else])
 (use gauche.threads)
-(cond-expand
- [gauche-0.9.10 (use srfi-1)]
- [else (use scheme.list)])
-(use srfi-13)
+(use scheme.list)
+(use srfi.13)
 (use file.util)
 (use text.tr)
 (use util.match)
