@@ -1,7 +1,7 @@
 /*
  * read.c - reader
  *
- *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -36,6 +36,7 @@
 #include <math.h>
 #define LIBGAUCHE_BODY
 #include "gauche.h"
+#include "gauche/priv/configP.h"
 #include "gauche/char_attr.h"
 #include "gauche/priv/portP.h"
 #include "gauche/priv/builtin-syms.h"
@@ -70,7 +71,8 @@ static ScmObj read_regexp(ScmPort *port);
 static ScmObj read_charset(ScmPort *port);
 static ScmObj read_sharp_comma(ScmPort *port, ScmReadContext *ctx);
 static ScmObj read_sharp_asterisk(ScmPort *port, ScmReadContext *ctx);
-static ScmObj process_sharp_comma(ScmPort *port, ScmObj key, ScmObj args,
+static ScmObj process_sharp_comma(ScmPort *port, int line,
+                                  ScmObj key, ScmObj args,
                                   ScmReadContext *ctx, int has_ref);
 static ScmObj read_shebang(ScmPort *port, ScmReadContext *ctx);
 static ScmObj read_num_prefixed(ScmPort *port, ScmChar ch, ScmReadContext *ctx);
@@ -405,6 +407,14 @@ inline static int char_word_case_fold(int c)
     return (c >= 0 && c < 128 && (ctypes[(unsigned char)c]&2));
 }
 
+/* For external API
+ * We may use flags for include_hash_sign in future.
+ */
+int Scm_CharWordConstituent(ScmChar ch, u_long flags SCM_UNUSED)
+{
+    return char_word_constituent((int)ch, FALSE);
+}
+
 /* R7RS 7.1.1 <delimiter> */
 static int char_is_delimiter(ScmChar ch)
 {
@@ -523,10 +533,13 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
                 /* #"..." - string interpolation  */
                 reject_in_r7(port, ctx, "#\"...\"");
                 Scm_UngetcUnsafe(c1, port);
+                int line = Scm_PortLine(port);
                 ScmObj form = read_item(port, ctx);
-                return process_sharp_comma(port,
+                return process_sharp_comma(port, line,
                                            SCM_SYM_STRING_INTERPOLATE,
-                                           SCM_LIST2(form, SCM_FALSE),
+                                           SCM_LIST4(form, SCM_FALSE,
+                                                     Scm_PortName(port),
+                                                     SCM_MAKE_INT(line)),
                                            ctx, FALSE);
             }
             case '/':
@@ -548,10 +561,13 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
             case '`': {
                 /* #`"..." - Legacy string interpolation syntax */
                 reject_in_r7(port, ctx, "#`\"...\"");
+                int line = Scm_PortLine(port);
                 ScmObj form = read_item(port, ctx);
-                return process_sharp_comma(port,
+                return process_sharp_comma(port, line,
                                            SCM_SYM_STRING_INTERPOLATE,
-                                           SCM_LIST2(form, SCM_TRUE),
+                                           SCM_LIST4(form, SCM_TRUE,
+                                                     Scm_PortName(port),
+                                                     SCM_MAKE_INT(line)),
                                            ctx, FALSE);
             }
             case '?': {
@@ -569,6 +585,21 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
                     ScmObj form = read_item(port, ctx);
                     return SCM_LIST2(SCM_SYM_DEBUG_FUNCALL, form);
                 }
+                case '?': {
+                    /* #??= or #??, - conditinal debug stub */
+                    ScmObj sym = SCM_FALSE;
+                    int c3 = Scm_GetcUnsafe(port);
+                    switch (c3) {
+                    case '=': sym = SCM_SYM_DEBUG_PRINT_CONDITIONALLY; break;
+                    case ',': sym = SCM_SYM_DEBUG_FUNCALL_CONDITIONALLY; break;
+                    default:
+                        Scm_ReadError(port, "unsupported #?-syntax: #??%C", c3);
+                        return SCM_UNDEFINED; /* dummy */
+                    }
+                    ScmObj test = read_item(port, ctx);
+                    ScmObj form = read_item(port, ctx);
+                    return SCM_LIST3(sym, test, form);
+                }
                 case '@': {
                     int c3 = Scm_GetcUnsafe(port);
                     switch (c3) {
@@ -1013,6 +1044,20 @@ static ScmObj read_quoted(ScmPort *port, ScmObj quoter, ScmReadContext *ctx)
  * String
  */
 
+/* Utility API to read string-literal like thingy.
+ * The caller already read and consumed the opening double-quote.
+ *
+ * FLAGS are ScmStringFlags.  Only SCM_STRING_INCOMPLETE is recognized.
+ */
+ScmObj Scm_ReadStringLiteral(ScmPort *port,
+                             ScmReadContext *ctx, /* can be NULL */
+                             u_long flags,
+                             ScmChar terminator SCM_UNUSED) /* reserved */
+{
+    int incompletep = flags & SCM_STRING_INCOMPLETE;
+    return read_string(port, incompletep, ctx);
+}
+
 /* Handling \xNN;, \uNNNN, \UNNNNNNNN escapes.  To make it easier
    to support both legacy \xN{2} syntax and new \xN{1,} syntax,
    we read-ahead as many hexdigits as possible first, then parse it,
@@ -1609,12 +1654,13 @@ static ScmObj read_sharp_comma(ScmPort *port, ScmReadContext *ctx)
     if (len <= 0) {
         Scm_ReadError(port, "bad #,-form: #,%S", form);
     }
-    ScmObj r = process_sharp_comma(port, SCM_CAR(form), SCM_CDR(form), ctx,
+    ScmObj r = process_sharp_comma(port, line, SCM_CAR(form), SCM_CDR(form), ctx,
                                    has_ref);
     return r;
 }
 
-static ScmObj process_sharp_comma(ScmPort *port, ScmObj key, ScmObj args,
+static ScmObj process_sharp_comma(ScmPort *port, int line,
+                                  ScmObj key, ScmObj args,
                                   ScmReadContext *ctx, int has_ref)
 {
     if (ctx->flags & RCTX_DISABLE_CTOR) return SCM_FALSE;
@@ -1625,6 +1671,11 @@ static ScmObj process_sharp_comma(ScmPort *port, ScmObj key, ScmObj args,
 
     if (!SCM_PAIRP(e)) Scm_ReadError(port, "unknown #,-key: %S", key);
     ScmObj r = Scm_ApplyRec(SCM_CAR(e), args);
+    if (SCM_PAIRP(r) && (ctx->flags & RCTX_SOURCE_INFO) && line >= 0) {
+        r = Scm_ExtendedCons(SCM_CAR(r), SCM_CDR(r));
+        Scm_PairAttrSet(SCM_PAIR(r), SCM_SYM_SOURCE_INFO,
+                        SCM_LIST2(Scm_PortName(port), SCM_MAKE_INT(line)));
+    }
     if (has_ref) ref_push(ctx, r, SCM_CDR(e));
     return r;
 }
