@@ -1,7 +1,7 @@
 ;;;
 ;;; vminsn.scm - Virtual machine instruction definition
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -53,10 +53,10 @@
 ;;;
 ;;;   <operand-type> - none : the insn doesn't take an operand.
 ;;;                    obj  : an ScmObj operand.
-;;;                    addr : an address the next pc points.
+;;;                    label : a label address the next pc points.
 ;;;                    code : an ScmCompiledCode operand.
 ;;;                    codes: a list of ScmCompiledCodes.
-;;;                    obj+addr : an ScmObj, followed by an address
+;;;                    obj+label : an ScmObj, followed by a label address
 ;;;
 ;;;   <combination>  - If this is a comibined insn, list the ingredients
 ;;;                    here.  It is used in multiple purposes:
@@ -420,7 +420,7 @@
 ;;  Prepare for a normal call.   Push a continuation that resumes
 ;;  execution from <cont>.
 ;;
-(define-insn PRE-CALL 1 addr #f
+(define-insn PRE-CALL 1 label #f
   (let* ([next::ScmWord*])
     (CHECK-STACK-PARANOIA CONT_FRAME_SIZE)
     (FETCH-LOCATION next)
@@ -430,7 +430,7 @@
     NEXT))
 
 ;; combined insn
-(define-insn PUSH-PRE-CALL 1 addr (PUSH PRE-CALL))
+(define-insn PUSH-PRE-CALL 1 label (PUSH PRE-CALL))
 
 ;; CHECK-STACK(size)
 ;;  Check for stack overflow
@@ -473,7 +473,7 @@
 ;; JUMP <addr>
 ;;  Jump to <addr>.
 ;;
-(define-insn JUMP      0 addr #f
+(define-insn JUMP      0 label #f
   (begin (FETCH-LOCATION PC) CHECK-INTR NEXT))
 
 ;; RET
@@ -571,7 +571,7 @@
 ;; LOCAL-ENV-JUMP(depth) <addr>
 ;;  Combination of LOCAL-ENV-SHIFT + JUMP.
 ;;  We can use this when none of the new environment needs boxing.
-(define-insn LOCAL-ENV-JUMP 1 addr #f
+(define-insn LOCAL-ENV-JUMP 1 label #f
   (begin
     (local_env_shift vm (SCM_VM_INSN_ARG code))
     (FETCH-LOCATION PC)
@@ -634,28 +634,28 @@
 ;;   Conditional branches.
 ;;   The combined operations leave the boolean value of the test result
 ;;   in VAL0.
-(define-insn BF      0 addr #f ($branch (SCM_CHECKED_FALSEP VAL0)))
-(define-insn BT      0 addr #f ($branch (not (SCM_CHECKED_FALSEP VAL0))))
-(define-insn BNEQ    0 addr #f ($w/argp z ($branch* (not (SCM_EQ VAL0 z)))))
-(define-insn BNEQV   0 addr #f ($w/argp z ($branch* (not (Scm_EqvP VAL0 z)))))
-(define-insn BNNULL  0 addr #f ($branch* (not (SCM_NULLP VAL0))))
+(define-insn BF      0 label #f ($branch (SCM_CHECKED_FALSEP VAL0)))
+(define-insn BT      0 label #f ($branch (not (SCM_CHECKED_FALSEP VAL0))))
+(define-insn BNEQ    0 label #f ($w/argp z ($branch* (not (SCM_EQ VAL0 z)))))
+(define-insn BNEQV   0 label #f ($w/argp z ($branch* (not (Scm_EqvP VAL0 z)))))
+(define-insn BNNULL  0 label #f ($branch* (not (SCM_NULLP VAL0))))
 
-(define-insn BNUMNE  0 addr #f (let* ((y VAL0))
+(define-insn BNUMNE  0 label #f (let* ((y VAL0))
                                  ($w/argp x ($branch* (not (Scm_NumEq x y))))))
-(define-insn BNLT    0 addr #f ($w/numcmp r <  ($branch* (not r))))
-(define-insn BNLE    0 addr #f ($w/numcmp r <= ($branch* (not r))))
-(define-insn BNGT    0 addr #f ($w/numcmp r >  ($branch* (not r))))
-(define-insn BNGE    0 addr #f ($w/numcmp r >= ($branch* (not r))))
+(define-insn BNLT    0 label #f ($w/numcmp r <  ($branch* (not r))))
+(define-insn BNLE    0 label #f ($w/numcmp r <= ($branch* (not r))))
+(define-insn BNGT    0 label #f ($w/numcmp r >  ($branch* (not r))))
+(define-insn BNGE    0 label #f ($w/numcmp r >= ($branch* (not r))))
 
 ;; Compare LREF(n,m) and VAL0 and branch.  This is not a simple combination
 ;; of LREF + BNLT etc. (which would compare stack top and LREF).  These insns
 ;; save one stack operation.  The compiler recognizes the pattern and
 ;; emits these.  See pass5/if-numcmp and pass5/if-numeq.
-(define-insn LREF-VAL0-BNUMNE 2 addr #f ($arg-source lref ($insn-body BNUMNE)))
-(define-insn LREF-VAL0-BNLT 2 addr #f ($arg-source lref ($insn-body BNLT)))
-(define-insn LREF-VAL0-BNLE 2 addr #f ($arg-source lref ($insn-body BNLE)))
-(define-insn LREF-VAL0-BNGT 2 addr #f ($arg-source lref ($insn-body BNGT)))
-(define-insn LREF-VAL0-BNGE 2 addr #f ($arg-source lref ($insn-body BNGE)))
+(define-insn LREF-VAL0-BNUMNE 2 label #f ($arg-source lref ($insn-body BNUMNE)))
+(define-insn LREF-VAL0-BNLT 2 label #f ($arg-source lref ($insn-body BNLT)))
+(define-insn LREF-VAL0-BNLE 2 label #f ($arg-source lref ($insn-body BNLE)))
+(define-insn LREF-VAL0-BNGT 2 label #f ($arg-source lref ($insn-body BNGT)))
+(define-insn LREF-VAL0-BNGE 2 label #f ($arg-source lref ($insn-body BNGE)))
 
 ;; BNUMNEI(i) <else-offset> ; combined CONSTI(i) + BNUMNE
 ;; BNEQC <else-offset>     ; branch if immediate constant is not eq? to VAL0
@@ -663,16 +663,16 @@
 ;;   NB: we tried other variations of constant op + branch combination,
 ;;       notably BNEQVI, BNUMNEF, BNLTF etc, but they did't show any
 ;;       improvement.
-(define-insn BNUMNEI     1 addr #f
+(define-insn BNUMNEI     1 label #f
   (let* ([imm::long (SCM_VM_INSN_ARG code)])
     ($w/argr v0
       ($type-check v0 SCM_NUMBERP "number")
       ($branch*
        (not (or (and (SCM_INTP v0)    (== (SCM_INT_VALUE v0) imm))
                 (and (SCM_FLONUMP v0) (== (SCM_FLONUM_VALUE v0) imm))))))))
-(define-insn BNEQC       0 obj+addr #f
+(define-insn BNEQC       0 obj+label #f
   (let* ([z]) (FETCH-OPERAND z) INCR-PC ($branch* (not (SCM_EQ VAL0 z)))))
-(define-insn BNEQVC      0 obj+addr #f
+(define-insn BNEQVC      0 obj+label #f
   (let* ([z]) (FETCH-OPERAND z) INCR-PC ($branch* (not (Scm_EqvP VAL0 z)))))
 
 ;; RF
@@ -727,7 +727,7 @@
 ;;  Like LET, this pushes the continuation frame to resume the
 ;;  operation from CONT-OFFSET.
 ;;
-(define-insn RECEIVE     2 addr #f
+(define-insn RECEIVE     2 label #f
   ($receive (set! size (+ CONT_FRAME_SIZE (ENV_SIZE (+ reqargs restarg))))
             (CHECK-STACK-PARANOIA size)
             (FETCH-LOCATION nextpc)
@@ -750,7 +750,7 @@
 ;;  dynamic-wind, the results of its body needs to be saved).
 ;;  This must be twined with VALUES-N, which reverses the effects, i.e.
 ;;  turn the values in the env frame into values.
-(define-insn RECEIVE-ALL 0 addr #f
+(define-insn RECEIVE-ALL 0 label #f
   (let* ([nextpc::ScmWord*])
     (CHECK-STACK-PARANOIA CONT_FRAME_SIZE)
     (FETCH-LOCATION nextpc)
@@ -790,6 +790,28 @@
   ($lset (SCM_VM_INSN_ARG0 code)        ; depth
          (SCM_VM_INSN_ARG1 code)))      ; offset
 
+;; XLSET(offset) depth
+;;   LSET, but used when depth and/or offset can't fit in the params field.
+(define-insn XLSET 1 obj #f
+  (let* ([dep_s])
+    (FETCH-OPERAND dep_s)
+    INCR-PC
+    ($lset (SCM_INT_VALUE dep_s)        ; depth
+           (SCM_VM_INSN_ARG code))))    ; offset
+
+;; ENV-SET(offset)
+;;  Mutate the top env's specified slot with VAL0
+;;  This is used with LOCAL-ENV-CLOSURES to initialize non-procedure
+;;  slots of the env.  We used to use LSET for this purpose, but now
+;;  LSET counts on that all mutable lvars are boxed, so we need a separete
+;;  insn that directly initialize the env frame.
+(define-insn ENV-SET 1 none #f
+  (let* ([off::int (SCM_VM_INSN_ARG code)])
+    (VM-ASSERT (> (-> ENV size) off))
+    (SCM_FLONUM_ENSURE_MEM VAL0)
+    (set! (ENV-DATA ENV off) VAL0)
+    NEXT))
+
 ;; GSET <location>
 ;;  LOCATION may be a symbol or gloc
 ;;
@@ -800,28 +822,8 @@
     (cond
      [(SCM_GLOCP loc) (SCM_GLOC_SET (SCM_GLOC loc) VAL0)]
      [(SCM_IDENTIFIERP loc)
-      ;; If runtime flag LIMIT_MODULE_MUTATION is set,
-      ;; we search only for the id's module, so that set! won't
-      ;; mutate bindings in the other module.
-      (let* ([id::ScmIdentifier* (Scm_OutermostIdentifier (SCM_IDENTIFIER loc))]
-             [name::ScmSymbol* (SCM_SYMBOL (-> id name))]
-             [limit::int
-              (SCM_VM_RUNTIME_FLAG_IS_SET vm SCM_LIMIT_MODULE_MUTATION)]
-             [gloc::ScmGloc*
-              (Scm_FindBinding (-> id module) name
-                               (?: limit SCM_BINDING_STAY_IN_MODULE 0))])
-        (when (== gloc NULL)
-          ;; Do search again for meaningful error message
-          (when limit
-            (set! gloc (Scm_FindBinding (-> id module) name 0))
-            (when (!= gloc NULL)
-              ($vm-err "can't mutate binding of %S, \
-                        which is in another module"
-                       (-> id name)))
-            ;; FALLTHROUGH
-            )
-          ($vm-err "symbol not defined: %S" loc))
-        (SCM_GLOC_SET gloc VAL0)
+      (let* ([gloc::ScmGloc* NULL])
+        (Scm_IdentifierGlobalSet (SCM_IDENTIFIER loc) VAL0 (& gloc))
         ;; memoize gloc.
         (set! (* PC) (SCM_WORD gloc)))]
      [else ($vm-err "GSET: can't be here")])
@@ -869,6 +871,15 @@
 
 (define-insn-lref* LREF-RET 0 none (LREF RET))
 
+;; XLREF(offset) depth
+;;   LREF, but used when depth and/or offset can't fit in the params field.
+(define-insn XLREF 1 obj #f
+  (let* ([dep_s])
+    (FETCH-OPERAND dep_s)
+    INCR-PC
+    ($lref (SCM_INT_VALUE dep_s)          ;depth
+           (SCM_VM_INSN_ARG code))))      ;offset
+
 ;; GREF <location>
 ;;  LOCATION may be a symbol or GLOC object.
 ;;  Retrieve global value in the current module.
@@ -894,7 +905,7 @@
 ;;  object.
 ;;
 (define-insn PROMISE  0 none #f
-  ($result (Scm_MakePromise FALSE VAL0)))
+  ($result (Scm_MakePromise SCM_PROMISE_UNFORCED VAL0)))
 
 ;; VALUES-APPLY(nargs) <args>
 ;;  This instruction only appears in the code generated dynamically
@@ -1013,7 +1024,7 @@
 
 (define-insn APPLY       1 none #f
   ;; this instruction will go away soon.  for now it only appears
-  ;; as the result of 'cond' with srfi-61 extension.
+  ;; as the result of 'cond' with SRFI-61 extension.
   (let* ([nargs::int (SCM_VM_INSN_ARG code)] [cp])
     (SCM_FLONUM_ENSURE_MEM VAL0)
     (while (> (pre-- nargs) 1)
@@ -1184,13 +1195,10 @@
 (define-insn UVEC-REF    1 none #f    ; uvector-ref
   (let* ([k VAL0]
          [utype::int (SCM_VM_INSN_ARG code)])
+    ;; Validation of utype and vec, and the range of k, is done in
+    ;; Scm_VMUVectorRef.
     ($w/argp vec
-      (unless (SCM_UVECTOR_SUBTYPE_P vec utype)
-        ($vm-err "%s required, but got %S" (Scm_UVectorTypeName utype) vec))
       ($type-check k SCM_INTP "fixnum")
-      (when (or (< (SCM_INT_VALUE k) 0)
-                (>= (SCM_INT_VALUE k) (SCM_UVECTOR_SIZE vec)))
-        ($vm-err "uvector-ref index out of range: %S" k))
       ($result (Scm_VMUVectorRef (SCM_UVECTOR vec) utype (SCM_INT_VALUE k)
                                  SCM_UNBOUND)))))
 
@@ -1382,11 +1390,11 @@
     ($result SCM_UNDEFINED)))
 
 (define-insn CURIN       0 none #f      ; current-input-port
-  ($result (SCM_OBJ (-> vm curin))))
+  ($result (SCM_OBJ SCM_CURIN)))
 (define-insn CUROUT      0 none #f      ; current-output-port
-  ($result (SCM_OBJ (-> vm curout))))
+  ($result (SCM_OBJ SCM_CUROUT)))
 (define-insn CURERR      0 none #f      ; current-error-port
-  ($result (SCM_OBJ (-> vm curerr))))
+  ($result (SCM_OBJ SCM_CURERR)))
 
 (define-insn SLOT-REF    0 none #f      ; slot-ref
   ($w/argp obj
@@ -1449,15 +1457,12 @@
     (POP-ARG before)
     (SCM_FLONUM_ENSURE_MEM before)
     (SCM_FLONUM_ENSURE_MEM after)
-    (set! (-> vm handlers)
-          (Scm_Cons (make_handler_entry before after args)
-                    (-> vm handlers)))
+    (push_dynamic_handlers vm before after args)
     NEXT))
 
 (define-insn POP-HANDLERS 0 none #f     ; pop dynamic handlers
   (begin
-    (VM-ASSERT (SCM_PAIRP (-> vm handlers)))
-    (set! (-> vm handlers) (SCM_CDR (-> vm handlers)))
+    (pop_dynamic_handlers vm)
     NEXT))
 
 ;;
@@ -1482,19 +1487,6 @@
                  (set! (ENV-DATA ENV off) (SCM_OBJ b)))))])
     NEXT))
 
-;; ENV-SET(offset)
-;;  Mutate the top env's specified slot with VAL0
-;;  This is used with LOCAL-ENV-CLOSURES to initialize non-procedure
-;;  slots of the env.  We used to use LSET for this purpose, but now
-;;  LSET counts on that all mutable lvars are boxed, so we need a separete
-;;  insn that directly initialize the env frame.
-(define-insn ENV-SET 1 none #f
-  (let* ([off::int (SCM_VM_INSN_ARG code)])
-    (VM-ASSERT (> (-> ENV size) off))
-    (SCM_FLONUM_ENSURE_MEM VAL0)
-    (set! (ENV-DATA ENV off) VAL0)
-    NEXT))
-
 ;; UNBOX
 ;;  VAL0 <- unbox(VAL0)
 (define-insn UNBOX 0 none #f
@@ -1513,22 +1505,45 @@
     (local_env_shift vm (SCM_VM_INSN_ARG code))
     NEXT))
 
-;; TRANSIENT: TODO: Move these up below LREF on 1.0 release
+;; EXTEND-DENV addr
+;; TAIL-EXTEND-DENV
+;;   Push new key-value pair on the current denv.
+;;   VAL0 holds the key, and stack top holds the value.
+;;   EXTEND-DENV is for non-tail calls, and a continuation is pushed
+;;   before adding key-value pair.
+(define-insn EXTEND-DENV 0 label #f
+  (let* ([arg] [next::ScmWord*])
+    (SCM_FLONUM_ENSURE_MEM VAL0)
+    (POP-ARG arg)
+    (FETCH-LOCATION next)
+    INCR_PC
+    (PUSH-CONT next)
+    (Scm_VMPushDynamicEnv arg VAL0)
+    NEXT))
 
-;; XLREF(offset) depth
-;;   LREF, but used when depth and/or offset can't fit in the params field.
-(define-insn XLREF 1 obj #f
-  (let* ([dep_s])
-    (FETCH-OPERAND dep_s)
-    INCR-PC
-    ($lref (SCM_INT_VALUE dep_s)          ;depth
-           (SCM_VM_INSN_ARG code))))      ;offset
+(define-insn TAIL-EXTEND-DENV 0 none #f
+  (let* ([arg])
+    (SCM_FLONUM_ENSURE_MEM VAL0)
+    (POP-ARG arg)
+    (Scm_VMPushDynamicEnv arg VAL0)
+    NEXT))
 
-;; XLSET(offset) depth
-;;   LSET, but used when depth and/or offset can't fit in the params field.
-(define-insn XLSET 1 obj #f
-  (let* ([dep_s])
-    (FETCH-OPERAND dep_s)
-    INCR-PC
-    ($lset (SCM_INT_VALUE dep_s)        ; depth
-           (SCM_VM_INSN_ARG code))))    ; offset
+;; EXPERIMENTAL
+;; XINSN info code-addr
+;;   'Extended instruction' - JIT compiled instruction handler.
+;;   The operand is an address of native code vector.
+(define-insn XINSN 0 obj+native #f
+  (.if (and SCM_TARGET_X86_64 (not GAUCHE_WINDOWS))
+    (let* ([jitcode::void*])
+      INCR_PC
+      (FETCH_LOCATION jitcode)
+      INCR_PC
+      (asm :volatile
+           "mov %[vm], %%r15; \
+            call *%[jitcode]"
+           ()
+           ((vm "r" vm)
+            (jitcode "r" jitcode))
+           ("r12" "r15"))
+      NEXT)
+    (Scm_Panic "XINSN instruction should never be seen on this platform.")))
