@@ -1,7 +1,7 @@
 /*
  * write.c - writer
  *
- *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2024  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,6 +33,7 @@
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
+#include "gauche/priv/configP.h"
 #include "gauche/writer.h"
 #include "gauche/priv/builtin-syms.h"
 #include "gauche/priv/macroP.h" /* PVREF stuff.  Will go in future */
@@ -48,8 +49,11 @@ static void write_ss(ScmObj obj, ScmPort *port, ScmWriteContext *ctx);
 static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx);
 static void write_object(ScmObj obj, ScmPort *out, ScmWriteContext *ctx);
 static ScmObj write_object_fallback(ScmObj *args, int nargs, ScmGeneric *gf);
+static int double_quote_closed_p(ScmString *s);
 SCM_DEFINE_GENERIC(Scm_GenericWriteObject, write_object_fallback, NULL);
 
+static void call_pprint(ScmObj obj, ScmPort *out);
+
 static const ScmWriteControls *defaultWriteControls;
 
 /*============================================================
@@ -128,6 +132,8 @@ ScmWriteControls *Scm_MakeWriteControls(const ScmWriteControls *proto)
         p->printBase = 10;
         p->printRadix = FALSE;
         p->printPretty = FALSE;
+        p->printIndent = 0;
+        p->stringLength = -1;
     }
     return p;
 }
@@ -353,6 +359,20 @@ static ScmObj write_object_fallback(ScmObj *args, int nargs,
                   Scm_ArrayToList(args, nargs));
     }
     ScmClass *klass = Scm_ClassOf(args[0]);
+    /* If any base class has print() pointer, we use it. */
+    for (ScmClass **cpa = klass->cpa; *cpa; cpa++) {
+        if ((*cpa)->print) {
+            /* NB: For historical reasons, the context passed to
+               write_object isn't passed down to the generic
+               function.  We'll address it later, but for now
+               we use the default context. */
+            ScmWriteContext ctx;
+            write_context_init(&ctx, SCM_WRITE_WRITE, 0, 0);
+            (*cpa)->print(args[0], SCM_PORT(args[1]), &ctx);
+            return SCM_TRUE;
+        }
+    }
+    /* Fallback printer */
     Scm_Printf(SCM_PORT(args[1]), "#<%A%s%p>",
                klass->name,
                (SCM_FALSEP(klass->redefined)? " " : ":redefined "),
@@ -665,7 +685,7 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
             if (wp->printLength == 0) {
                 /* in this case we don't print the elements at all, so we need
                    to treat this specially. */
-                Scm_PutzUnsafe("(...)", -1, port);
+                Scm_PutzUnsafe("(" SCM_WRITTEN_ELLIPSIS ")", -1, port);
                 if (st) st->currentLevel--;
                 goto next;
             }
@@ -682,7 +702,7 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
             if (wp->printLength == 0) {
                 /* in this case we don't print the elements at all, so we need
                    to treat this specially. */
-                Scm_PutzUnsafe("#(...)", -1, port);
+                Scm_PutzUnsafe("#(" SCM_WRITTEN_ELLIPSIS ")", -1, port);
                 if (st) st->currentLevel--;
                 goto next;
             }
@@ -714,7 +734,7 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
                     Scm_PutcUnsafe(')', port);
                     POP();
                 } else if (wp->printLength >= 0 && wp->printLength <= i) {
-                    Scm_PutzUnsafe(" ...)", -1, port);
+                    Scm_PutzUnsafe(" " SCM_WRITTEN_ELLIPSIS ")", -1, port);
                     POP();
                 } else {
                     Scm_PutcUnsafe(' ', port);
@@ -752,7 +772,7 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
                     }
                 } else if (wp->printLength >= 0 && wp->printLength <= count) {
                     /* print-length limit reached */
-                    Scm_PutzUnsafe(" ...)", -1, port);
+                    Scm_PutzUnsafe(" " SCM_WRITTEN_ELLIPSIS ")", -1, port);
                     POP();
                 } else if (ht && !SCM_EQ(Scm_HashTableRef(ht, v, SCM_MAKE_INT(1)), SCM_MAKE_INT(1)))  {
                     /* cdr part is shared */
@@ -840,9 +860,9 @@ void Scm_Format(ScmPort *out SCM_UNUSED,
  *
  *        %[width][.prec]A    - Same as %S, but use DISPLAY mode.
  *
- *        %C                  - Take ScmChar argument and outputs it.
+ *        %W                  - Call pprint.
  *
- *  Both functions return a number of characters written.
+ *        %C                  - Take ScmChar argument and outputs it.
  *
  *  A pound flag '#' for the S directive causes circular-safe output.
  *
@@ -858,6 +878,10 @@ void Scm_Format(ScmPort *out SCM_UNUSED,
  * called by Scm_WithPortLocking.  On some architecture, we can't pass
  * va_list type of argument in a closure packet easily.
  */
+/* NB: It is tempting to write the logic in Scheme, but we keep the main
+ * feature in C so that Scm_Printf is available to debug Scheme code.
+ * The exception is %W which need to call pprint.
+ */
 
 /* Pass 1.  Pop vararg and make a list of arguments.
  * NB: If we're "walking" pass, and the argument is a Lisp object,
@@ -917,7 +941,7 @@ static ScmObj SCM_NOINLINE vprintf_pass1(ScmPort *out,
                     SCM_APPEND1(h, t, Scm_MakeIntegerU((u_long)(intptr_t)val));
                     break;
                 }
-            case 'S': case 'A':
+            case 'S': case 'A': case 'W':
                 {
                     ScmObj o = va_arg(ap, ScmObj);
                     SCM_APPEND1(h, t, o);
@@ -1061,18 +1085,35 @@ static void vprintf_pass2(ScmPort *out, const char *fmt, ScmObj args)
                                 : ((c == 'A')
                                    ? SCM_WRITE_DISPLAY
                                    : SCM_WRITE_WRITE));
-                    int n = 0;
                     if (width <= 0) {
                         Scm_Write(val, SCM_OBJ(out), mode);
-                    } else {
+                    } if (prec <= 0) {
                         Scm_WriteLimited(val, SCM_OBJ(out), mode, width);
+                    } else {
+                        ScmObj buf = Scm_MakeOutputStringPort(TRUE);
+                        Scm_WriteLimited(val, buf, mode, width);
+                        ScmObj s = Scm_GetOutputString(SCM_PORT(buf), 0);
+                        Scm_PutsUnsafe(SCM_STRING(s), out);
+                        ScmSmallInt n = SCM_STRING_LENGTH(s);
+                        if (n > width) {
+                            if (double_quote_closed_p(SCM_STRING(s)))
+                                Scm_PutzUnsafe(" " SCM_WRITTEN_ELLIPSIS,
+                                               -1, out);
+                            else
+                                Scm_PutzUnsafe("\"" SCM_WRITTEN_ELLIPSIS,
+                                               -1, out);
+                        } else {
+                            for (; n < prec; n++) Scm_PutcUnsafe(' ', out);
+                        }
                     }
-                    if (n < 0 && prec > 0) {
-                        Scm_PutzUnsafe(" ...", -1, out);
-                    }
-                    if (n > 0) {
-                        for (; n < prec; n++) Scm_PutcUnsafe(' ', out);
-                    }
+                    break;
+                }
+            case 'W':
+                {
+                    SCM_ASSERT(SCM_PAIRP(args));
+                    ScmObj val = SCM_CAR(args);
+                    args = SCM_CDR(args);
+                    call_pprint(val, out);
                     break;
                 }
             case 'C':
@@ -1120,6 +1161,42 @@ static void vprintf_pass2(ScmPort *out, const char *fmt, ScmObj args)
     }
 }
 
+/* NB: libfmt has similar routine (quote-terminated?) in 'chop-and-out'.
+   Since Scm_Printf can be called during initialiation before Scheme runtime
+   is fully up, we need this implemented in C.  */
+static int double_quote_closed_p(ScmString *s)
+{
+    ScmSmallInt siz;
+    const char *p = Scm_GetStringContent(s, &siz, NULL, NULL);
+    int inside = FALSE;
+    int escaped = FALSE;
+    for (int i = 0; i < siz; i++, p++) {
+        if (!escaped) {
+            if (*p == '"') {
+                inside = !inside;
+            }
+            if (*p == '\\') {
+                escaped = TRUE;
+            } else {
+                escaped = FALSE;
+            }
+        } else {
+            escaped = FALSE;
+        }
+    }
+    return !inside;
+}
+
+static void call_pprint(ScmObj obj, ScmPort *out)
+{
+    /* Provisional.  We need to set proper indent if %W does not begin
+       at the beginning of line. */
+    ScmWriteControls *wc = Scm_MakeWriteControls(NULL);
+    wc->printPretty = TRUE;
+    wc->printWidth = 79;
+    Scm_WriteWithControls(obj, SCM_OBJ(out), SCM_WRITE_WRITE, wc);
+}
+
 /* Public APIs */
 
 void Scm_Vprintf(ScmPort *out, const char *fmt, va_list ap,
