@@ -417,7 +417,7 @@
 
 (define-syntax nest6 (syntax-rules ()
                        ((_ (?a ...) ...)
-                        (?a ... ...)))) ;srfi-149
+                        (?a ... ...)))) ;SRFI-149
 (test-macro "nest6" (a b c d e f g h i j)
             (nest6 (a b c d) (e f g) (h i) (j)))
 (test-macro "nest6" (a b c d e f g)
@@ -425,7 +425,7 @@
 
 (define-syntax nest7 (syntax-rules ()
                        ((_ (?a ...) ...)
-                        (?a ... ... z ?a ... ...)))) ;srfi-149
+                        (?a ... ... z ?a ... ...)))) ;SRFI-149
 (test-macro "nest7" (a b c d e f g h i j z a b c d e f g h i j)
             (nest7 (a b c d) (e f g) (h i) (j)))
 (test-macro "nest7" (a b c d e f g z a b c d e f g)
@@ -433,13 +433,13 @@
 
 (define-syntax nest8 (syntax-rules ()
                        ((_ ((?a ...) ...) ...)
-                        (?a ... ... ... z)))) ;srfi-149
+                        (?a ... ... ... z)))) ;SRFI-149
 (test-macro "nest8" (a b c d e f g h i j z)
             (nest8 ((a b c d) (e f g)) ((h i) (j))))
 (test-macro "nest8" (a b c d e f g h i j z)
             (nest8 ((a b c d) () (e f g)) () ((h i) () (j) ())))
 
-;; mixlevel is allowed by srfi-149
+;; mixlevel is allowed by SRFI-149
 (define-syntax mixlevel1 (syntax-rules ()
                            ((_ (?a ?b ...)) ((?a ?b) ...))))
 
@@ -1399,6 +1399,52 @@
           (push! (vector-ref a 1) 3)
           a)))
 
+(test "push-unique!" '(3 2 1)
+      (lambda ()
+        (let ((a '(1)))
+          (push-unique! a 1)
+          (push-unique! a 2)
+          (push-unique! a 3)
+          (push-unique! a 2)
+          (push-unique! a 1)
+          (push-unique! a 3)
+          a)))
+
+(test "push-unique!" '#((3 2 1) (3 1 2))
+      (lambda ()
+        (let ((a (vector '() '())))
+          (push-unique! (vector-ref a 0) 1)
+          (push-unique! (vector-ref a 0) 2)
+          (push-unique! (vector-ref a 0) 3)
+          (push-unique! (vector-ref a 0) 2)
+          (push-unique! (vector-ref a 0) 1)
+          (push-unique! (vector-ref a 0) 3)
+          (push-unique! (vector-ref a 1) 2)
+          (push-unique! (vector-ref a 1) 1)
+          (push-unique! (vector-ref a 1) 3)
+          (push-unique! (vector-ref a 1) 1)
+          a)))
+
+(test "push-unique!" '("a" "B" "c")
+      (lambda ()
+        (let ((a '("c")))
+          (push-unique! a "B" string-ci=?)
+          (push-unique! a "C" string-ci=?)
+          (push-unique! a "a" string-ci=?)
+          (push-unique! a "b" string-ci=?)
+          (push-unique! a "A" string-ci=?)
+          a)))
+
+(test "push-unique!" '(("a" "B" "c"))
+      (lambda ()
+        (let ((a (list '("c"))))
+          (push-unique! (car a) "B" string-ci=?)
+          (push-unique! (car a) "C" string-ci=?)
+          (push-unique! (car a) "a" string-ci=?)
+          (push-unique! (car a) "b" string-ci=?)
+          (push-unique! (car a) "A" string-ci=?)
+          a)))
+
 (test "pop!" '((2 3) . 1)
       (lambda ()
         (let* ((a (list 1 2 3))
@@ -1463,6 +1509,25 @@
           (dec! ((zz) x))
           (list x y))))
 
+(test "rotate!" '(2 1)
+      (lambda ()
+        (let ((a 1)
+              (b 2))
+          (rotate! a b)
+          (list a b))))
+(test "rotate!" '(2 3 1)
+      (lambda ()
+        (let ((a 1)
+              (b 2)
+              (c 3))
+          (rotate! a b c)
+          (list a b c))))
+(test "rotate!" '(2 3 1)
+      (lambda ()
+        (let ((vals (list 1 2 3)))
+          (rotate! (car vals) (cadr vals) (list-ref vals 2))
+          vals)))
+
 (test "dotimes" '(0 1 2 3 4 5 6 7 8 9)
       (lambda ()
         (let ((m '()))
@@ -1901,12 +1966,12 @@
 
 
 ;;----------------------------------------------------------------------
-;; srfi-147 begin
+;; SRFI-147 begin
 ;; (not yest supported)
 
-'(test-section "srfi-147 begin")
+'(test-section "SRFI-147 begin")
 
-'(test "srfi-147 begin (internal) 1"
+'(test "SRFI-147 begin (internal) 1"
       '(yes no)
       (lambda ()
         (define-syntax foo
@@ -1916,7 +1981,7 @@
         (list (foo 'yes 'no (zero? 0))
               (foo 'yes 'no (zero? 1)))))
 
-'(test "srfi-147 begin (internal) 2"
+'(test "SRFI-147 begin (internal) 2"
       11
       (lambda ()
         (let-syntax ([foo (syntax-rules ()
