@@ -272,6 +272,7 @@
 (define-module Oc
   (export ichi (rename shi yon) go)
   (define ichi 1)
+  (define ni 2) ; not exported
   (define shi 4)
   (define go (+ ichi shi))
   (define kyu (+ shi go))
@@ -289,6 +290,25 @@
                 (global-variable-ref m 'shi #f)
                 (global-variable-ref m 'kyu #f)))))
 
+(test "export-time renaming and defines?/exports? (0c)"
+      '((ichi #t #t) (ni #t #f) (shi #t #f) (yon #f #t) (go #t #t) (kyu #t #f) (ku #f #t))
+      (lambda ()
+        (let [(m (find-module 'Oc))]
+          (map (lambda (n)
+                 (list n
+                       (module-binds? m n)
+                       (module-exports? m n)))
+               '(ichi ni shi yon go kyu ku)))))
+(test "export-time renaming and defines?/exports? (0c-1)"
+      '((ichi #t #t) (ni #f #f) (shi #f #f) (yon #t #t) (go #t #t) (kyu #f #f) (ku #t #t))
+      (lambda ()
+        (let [(m (find-module 'Oc-1))]
+          (map (lambda (n)
+                 (list n
+                       (module-binds? m n)
+                       (module-exports? m n)))
+               '(ichi ni shi yon go kyu ku)))))
+
 (define-module Od
   (import (Oc :rename ((ku kokono)) :prefix mm:)))
 
@@ -464,7 +484,7 @@
 
 (define (mplbug-test mod var)
   (test* #"mpl search (~mod,~var)" #t
-         (global-variable-bound? (find-module mod) var)))
+         (module-binds? (find-module mod) var)))
 
 (mplbug-test 'mplbug-user1 'a)
 (mplbug-test 'mplbug-user1 'b)
@@ -644,8 +664,7 @@
                             set! syntax-rules else =>))
                    (missing '()))
           (cond [(null? names) missing]
-                [(global-variable-bound? 'null (car names))
-                 (loop (cdr names) missing)]
+                [(module-binds? 'null (car names)) (loop (cdr names) missing)]
                 [else (loop (cdr names) (cons (car names) missing))]))))
 
 (test "scheme module bindings" '()
@@ -694,8 +713,7 @@
                             write write-char zero?))
                    (missing '()))
           (cond [(null? names) missing]
-                [(global-variable-bound? 'scheme (car names))
-                 (loop (cdr names) missing)]
+                [(module-binds? 'scheme (car names)) (loop (cdr names) missing)]
                 [else (loop (cdr names) (cons (car names) missing))]))))
 
 ;;-------------------------------------------------------------------
