@@ -7,6 +7,8 @@
 (test-start "primitive syntax")
 
 ;; We use prim-test instead of test, for error-handler is not tested yet.
+;; NB: errsyn.scm has tests for invalid syntax.  It is run after
+;; error-handler and macro tests.
 
 ;;----------------------------------------------------------------
 (test-section "conditionals")
@@ -36,9 +38,9 @@
 (prim-test "cond" 2        (lambda ()  (cond (1 2) (else 5))))
 (prim-test "cond" 8        (lambda ()  (cond (#f 2) (1 8) (else 5))))
 (prim-test "cond" 3        (lambda ()  (cond (1 => (lambda (x) (+ x 2))) (else 8))))
-(prim-test "cond (srfi-61)" 1 (lambda () (cond (1 number? => values) (else 8))))
-(prim-test "cond (srfi-61)" 8 (lambda () (cond (1 string? => values) (else 8))))
-(prim-test "cond (srfi-61)" '(1 2)
+(prim-test "cond (SRFI-61)" 1 (lambda () (cond (1 number? => values) (else 8))))
+(prim-test "cond (SRFI-61)" 8 (lambda () (cond (1 string? => values) (else 8))))
+(prim-test "cond (SRFI-61)" '(1 2)
            (lambda () (cond ((values 1 2)
                              (lambda (x y) (and (= x 1) (= y 2)))
                              => list))))
@@ -48,9 +50,9 @@
 (prim-test "case" #t (lambda () (case 1 (() #f) ((1) #t))))
 (prim-test "case" #t (lambda () (case 1 (() #f) (else #t))))
 (prim-test "case" #t (lambda () (undefined? (case 1 (() #t)))))
-(prim-test "case (srfi-87)" 0 (lambda () (case (+ 2 3) ((1 3 5) 0) (else => values))))
-(prim-test "case (srfi-87)" 6 (lambda () (case (+ 2 3) ((1 3 5) => (cut + 1 <>)) (else => values))))
-(prim-test "case (srfi-87)" 5 (lambda () (case (+ 2 3) ((2 4 6) 0) (else => values))))
+(prim-test "case (SRFI-87)" 0 (lambda () (case (+ 2 3) ((1 3 5) 0) (else => values))))
+(prim-test "case (SRFI-87)" 6 (lambda () (case (+ 2 3) ((1 3 5) => (cut + 1 <>)) (else => values))))
+(prim-test "case (SRFI-87)" 5 (lambda () (case (+ 2 3) ((2 4 6) 0) (else => values))))
 
 ;;----------------------------------------------------------------
 (test-section "binding")
