@@ -6,7 +6,8 @@
 (use gauche.sequence)
 (use gauche.process)
 (use util.match)
-(use srfi-19)
+(use srfi.19)
+(use srfi.13)
 (test-start "rfc")
 
 ;; rfc.822 test is in ext/rfc
@@ -16,69 +17,126 @@
 (use rfc.base64)
 (test-module 'rfc.base64)
 
-(test* "encode" "" (base64-encode-string ""))
-(test* "encode" "YQ==" (base64-encode-string "a"))
-(test* "encode" "MA==" (base64-encode-string "0"))
-(test* "encode" "Cg==" (base64-encode-string "\n"))
-(test* "encode" "YTA=" (base64-encode-string "a0"))
-(test* "encode" "YTAK" (base64-encode-string "a0\n"))
-(test* "encode" "PQk0" (base64-encode-string "=\t4"))
-(test* "encode" "eTQ5YQ==" (base64-encode-string "y49a"))
-(test* "encode" "RWdqYWk=" (base64-encode-string "Egjai"))
-(test* "encode" "OTNiamFl" (base64-encode-string "93bjae"))
-(test* "encode" "QkFSMGVyOQ==" (base64-encode-string "BAR0er9"))
+;; NB: encode from/decode to uvector are tested in gauche.vport.
 
-(test* "encode w/ line width (default)"
+(test* "base64 encode" "" (base64-encode-message ""))
+(test* "base64 encode" "YQ==" (base64-encode-message "a"))
+(test* "base64 encode" "MA==" (base64-encode-message "0"))
+(test* "base64 encode" "Cg==" (base64-encode-message "\n"))
+(test* "base64 encode" "YTA=" (base64-encode-message "a0"))
+(test* "base64 encode" "YTAK" (base64-encode-message "a0\n"))
+(test* "base64 encode" "PQk0" (base64-encode-message "=\t4"))
+(test* "base64 encode" "eTQ5YQ==" (base64-encode-message "y49a"))
+(test* "base64 encode" "RWdqYWk=" (base64-encode-message "Egjai"))
+(test* "base64 encode" "OTNiamFl" (base64-encode-message "93bjae"))
+(test* "base64 encode" "QkFSMGVyOQ==" (base64-encode-message "BAR0er9"))
+
+(test* "base64 encode w/ line width (default)"
        "MDEyMzQ1Njc4OTAxMjM0NTY3ODkwMTIzNDU2Nzg5MDEyMzQ1Njc4OTAxMjM0NTY3ODkwMTIzNDU2\n"
-       (base64-encode-string "012345678901234567890123456789012345678901234567890123456"))
-(test* "encode w/ line width 10, e1"
+       (base64-encode-message "012345678901234567890123456789012345678901234567890123456"))
+(test* "base64 encode w/ line width 10, e1"
        "MDEyMzQ1Ng\n=="
-       (base64-encode-string "0123456" :line-width 10))
-(test* "encode w/ line width 11, e1"
+       (base64-encode-message "0123456" :line-width 10))
+(test* "base64 encode w/ line width 11, e1"
        "MDEyMzQ1Ng=\n="
-       (base64-encode-string "0123456" :line-width 11))
-(test* "encode w/ line width 12, e1"
+       (base64-encode-message "0123456" :line-width 11))
+(test* "base64 encode w/ line width 12, e1"
        "MDEyMzQ1Ng==\n"
-       (base64-encode-string "0123456" :line-width 12))
-(test* "encode w/ line width 11, e2"
+       (base64-encode-message "0123456" :line-width 12))
+(test* "base64 encode w/ line width 11, e2"
        "MDEyMzQ1Njc\n="
-       (base64-encode-string "01234567" :line-width 11))
-(test* "encode w/ line width 12, e2"
+       (base64-encode-message "01234567" :line-width 11))
+(test* "base64 encode w/ line width 12, e2"
        "MDEyMzQ1Njc=\n"
-       (base64-encode-string "01234567" :line-width 12))
-(test* "encode w/ line width 4"
+       (base64-encode-message "01234567" :line-width 12))
+(test* "base64 encode w/ line width 4"
        "MDEy\nMzQ=\n"
-       (base64-encode-string "01234" :line-width 4))
-(test* "encode w/ line width 3"
+       (base64-encode-message "01234" :line-width 4))
+(test* "base64 encode w/ line width 3"
        "MDE\nyMz\nQ="
-       (base64-encode-string "01234" :line-width 3))
-(test* "encode w/ line width 2"
+       (base64-encode-message "01234" :line-width 3))
+(test* "base64 encode w/ line width 2"
        "MD\nEy\nMz\nQ=\n"
-       (base64-encode-string "01234" :line-width 2))
-(test* "encode w/ line width 1"
+       (base64-encode-message "01234" :line-width 2))
+(test* "base64 encode w/ line width 1"
        "M\nD\nE\ny\nM\nz\nQ\n=\n"
-       (base64-encode-string "01234" :line-width 1))
-(test* "encode w/ line width 0"
+       (base64-encode-message "01234" :line-width 1))
+(test* "base64 encode w/ line width 0"
        "MDEyMzQ="
-       (base64-encode-string "01234" :line-width 0))
+       (base64-encode-message "01234" :line-width 0))
 
-(test* "decode" "" (base64-decode-string ""))
-(test* "decode" "a" (base64-decode-string "YQ=="))
-(test* "decode" "a" (base64-decode-string "YQ="))
-(test* "decode" "a" (base64-decode-string "YQ"))
-(test* "decode" "a0" (base64-decode-string "YTA="))
-(test* "decode" "a0" (base64-decode-string "YTA"))
-(test* "decode" "a0\n" (base64-decode-string "YTAK"))
-(test* "decode" "y49a" (base64-decode-string "eTQ5YQ=="))
-(test* "decode" "Egjai" (base64-decode-string "RWdqYWk="))
-(test* "decode" "93bjae" (base64-decode-string "OTNiamFl"))
-(test* "decode" "BAR0er9" (base64-decode-string "QkFSMGVyOQ=="))
-(test* "decode" "BAR0er9" (base64-decode-string "QkFS\r\nMGVyOQ\r\n=="))
+(test* "base64 decode" "" (base64-decode-string-to <string> ""))
+(test* "base64 decode" "a" (base64-decode-string-to <string> "YQ=="))
+(test* "base64 decode" "a" (base64-decode-string-to <string> "YQ="))
+(test* "base64 decode" "a" (base64-decode-string-to <string> "YQ"))
+(test* "base64 decode" "a0" (base64-decode-string-to <string> "YTA="))
+(test* "base64 decode" "a0" (base64-decode-string-to <string> "YTA"))
+(test* "base64 decode" "a0\n" (base64-decode-string-to <string> "YTAK"))
+(test* "base64 decode" "y49a" (base64-decode-string-to <string> "eTQ5YQ=="))
+(test* "base64 decode" "Egjai" (base64-decode-string-to <string> "RWdqYWk="))
+(test* "base64 decode" "93bjae" (base64-decode-string-to <string> "OTNiamFl"))
+(test* "base64 decode" "BAR0er9" (base64-decode-string-to <string> "QkFSMGVyOQ=="))
+(test* "base64 decode" "BAR0er9" (base64-decode-string-to <string> "QkFS\r\nMGVyOQ\r\n=="))
 
-(test* "standard encode" "YTA+YTA/" (base64-encode-string "a0>a0?"))
-(test* "standard decode" "a0>a0?" (base64-decode-string "YTA+YTA/"))
-(test* "url-safe encode" "YTA-YTA_" (base64-encode-string "a0>a0?" :url-safe #t))
-(test* "url-safe decode" "a0>a0?" (base64-decode-string "YTA-YTA_" :url-safe #t))
+(test* "base64 standard encode" "YTA+YTA/" (base64-encode-message "a0>a0?"))
+(test* "base64 standard decode" "a0>a0?" (base64-decode-string-to <string> "YTA+YTA/"))
+(test* "base64 url-safe encode" "YTA-YTA_" (base64-encode-message "a0>a0?" :url-safe #t))
+(test* "base64 url-safe decode" "a0>a0?" (base64-decode-string-to <string> "YTA-YTA_" :url-safe #t))
+
+(test* "base64 standard encode (digits)" "YTA+YTA/" (base64-encode-message "a0>a0?" :digits "+/"))
+(test* "base64 standard decode (digits)" "a0>a0?" (base64-decode-string-to <string> "YTA+YTA/" :digits "+/"))
+(test* "base64 url-safe encode (digits)" "YTA-YTA_" (base64-encode-message "a0>a0?" :digits "-_"))
+(test* "base64 url-safe decode (digits)" "a0>a0?" (base64-decode-string-to <string> "YTA-YTA_" :digits "-_"))
+(test* "base64 weird encode (digits)" "YTA.YTA?" (base64-encode-message "a0>a0?" :digits ".?"))
+(test* "base64 weird decode (digits)" "a0>a0?" (base64-decode-string-to <string> "YTA.YTA?" :digits ".?"))
+
+(test* "base64 omit-padding" "YQ" (base64-encode-message "a" :omit-padding #t))
+(test* "base64 omit-padding" "YTA" (base64-encode-message "a0" :omit-padding #t))
+
+;; Test data from RFC4648
+(define (test-base32 orig encoded encoded-hex)
+  (define (test-1 name encoded encoder decoder)
+    (define orig-bv (string->u8vector orig))
+    (define enc-no-pads (string-trim-right encoded #\=))
+    (test* #"~name encode" encoded (encoder orig))
+    (test* #"~name decode" orig (decoder <string> encoded))
+    (test* #"~name encode (bv)" encoded (encoder orig-bv))
+    (test* #"~name decode (bv)" orig-bv (decoder <u8vector> encoded))
+    (test* #"~name encode (no pads)" enc-no-pads (encoder orig :omit-padding #t))
+    (test* #"~name decode (no pads)" orig (decoder <string> enc-no-pads))
+    (when (string-index encoded #\=)
+      (test* #"~name decode (no pads, error)" (test-error)
+             (decoder <string> enc-no-pads :strict #t))))
+
+  (test-1 "base32" encoded base32-encode-message base32-decode-string-to)
+  (test-1 "base32hex" encoded-hex base32hex-encode-message base32hex-decode-string-to)
+  )
+
+(let ((data '(;orig     base32              base32hex
+              (""       ""                  "")
+              ("f"      "MY======"          "CO======")
+              ("fo"     "MZXQ===="          "CPNG====")
+              ("foo"    "MZXW6==="          "CPNMU===")
+              ("foob"   "MZXW6YQ="          "CPNMUOG=")
+              ("fooba"  "MZXW6YTB"          "CPNMUOJ1")
+              ("foobar" "MZXW6YTBOI======"  "CPNMUOJ1E8======"))))
+  (for-each (cut apply test-base32 <>) data))
+
+(define (test-base16 orig encoded)
+  (define orig-bv (string->u8vector orig))
+  (test* "base16 encode" encoded (base16-encode-message orig))
+  (test* "base16 decode" orig (base16-decode-string-to <string> encoded))
+  (test* "base16 encode (bv)" encoded (base16-encode-message orig-bv))
+  (test* "base16 decode (bv)" orig-bv (base16-decode-string-to <u8vector> encoded)))
+
+(let ((data '((""       "")
+              ("f"      "66")
+              ("fo"     "666F")
+              ("foo"    "666F6F")
+              ("foob"   "666F6F62")
+              ("fooba"  "666F6F6261")
+              ("foobar" "666F6F626172"))))
+  (for-each (cut apply test-base16 <>) data))
 
 ;;--------------------------------------------------------------------
 (test-section "rfc.quoted-printable")
@@ -86,48 +144,48 @@
 (test-module 'rfc.quoted-printable)
 
 (test* "encode" "abcd=0Cefg"
-       (quoted-printable-encode-string "abcd\x0cefg"))
+       (quoted-printable-encode-message "abcd\x0cefg"))
 (test* "encode"
        "abcd\r\nefg"
-       (quoted-printable-encode-string "abcd\r\nefg"))
+       (quoted-printable-encode-message "abcd\r\nefg"))
 (test* "encode (tab/space at eol)"
        "abcd=09\r\nefg=20\r\n"
-       (quoted-printable-encode-string "abcd\t\r\nefg \r\n"))
+       (quoted-printable-encode-message "abcd\t\r\nefg \r\n"))
 (test* "encode (soft line break)"
        "0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abc=\r\ndefghij0123456789abcdefghij"
-       (quoted-printable-encode-string "0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij"))
+       (quoted-printable-encode-message "0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij"))
 (test* "encode (soft line break w/line-width)"
        "0123456789abcdefg=\r\nhij0123456789abcd=\r\nefghij"
-       (quoted-printable-encode-string
+       (quoted-printable-encode-message
         "0123456789abcdefghij0123456789abcdefghij"
         :line-width 20))
 (test* "encode (soft line break w/line-width)"
        "0123456789abcdef=3D=\r\nghij0123456789a=3D=\r\n=3Dbcdefghij"
-       (quoted-printable-encode-string
+       (quoted-printable-encode-message
         "0123456789abcdef=ghij0123456789a==bcdefghij"
         :line-width 20))
 (test* "encode (soft line break w/line-width lower bound)"
        "a=\r\n=3F=\r\nb"
-       (quoted-printable-encode-string "a?b" :line-width 4))
+       (quoted-printable-encode-message "a?b" :line-width 4))
 (test* "encode (no line break)"
        "0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij"
-       (quoted-printable-encode-string "0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij"
+       (quoted-printable-encode-message "0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij"
                                        :line-width #f))
 (test* "encode (hard line break)"
        "a\r\nb\r\nc\r\n"
-       (quoted-printable-encode-string "a\rb\nc\r\n"))
+       (quoted-printable-encode-message "a\rb\nc\r\n"))
 (test* "encode (binary)"
        "a=0Db=0Ac=0D=0A"
-       (quoted-printable-encode-string "a\rb\nc\r\n" :binary #t))
+       (quoted-printable-encode-message "a\rb\nc\r\n" :binary #t))
 
 (test* "decode" "\x01\x08abcde=\r\n"
-       (quoted-printable-decode-string "=01=08abc=64=65=3D\r\n"))
+       (quoted-printable-decode-string-to <string> "=01=08abc=64=65=3D\r\n"))
 (test* "decode (soft line break)"
        "Now's the time for all folk to come to the aid of their country."
-       (quoted-printable-decode-string "Now's the time =\r\nfor all folk to come=   \r\n to the aid of their country."))
+       (quoted-printable-decode-string-to <string> "Now's the time =\r\nfor all folk to come=   \r\n to the aid of their country."))
 (test* "decode (robustness)"
        "foo=1qr =  j\r\n"
-       (quoted-printable-decode-string "foo=1qr =  j\r\n="))
+       (quoted-printable-decode-string-to <string> "foo=1qr =  j\r\n="))
 
 
 ;;--------------------------------------------------------------------
@@ -135,7 +193,7 @@
 (use rfc.cookie)
 (test-module 'rfc.cookie)
 (use scheme.list)
-(use srfi-19)
+(use srfi.19)
 
 (test* "parse, old" '(("foo" "bar")
                       ("aaa" "bbb" :path "/a/b" :domain "a.b.com")
@@ -175,18 +233,13 @@
      :discard #f :secure #t :comment "ZzzZzz, OooOoo"
      :comment-url "http://foo.com/hogehoge")))
 
-(test* "cookie, old"
-       '("guest-id=foo123;Domain=foo.com;Path=/abc;Expires=Sun, 09-Sep-2001 01:46:40 GMT"
-         "guest-account=87975348;Domain=zzz.com;Path=/zzz;Secure")
-       (construct-cookie-string *cookie-spec* 0))
-
-(test* "cookie, new"
-       '("guest-id=foo123;Domain=foo.com;Path=/abc;Max-Age=864000;Discard;Comment=hogehoge;CommentURL=\"http://foo.com/hogehoge\";Port=\"80, 8080\";Version=1"
+(test* "cookie"
+       '("guest-id=foo123;Domain=foo.com;Path=/abc;Expires=Sun, 09-Sep-2001 01:46:40 GMT;Max-Age=864000;Discard;Comment=hogehoge;CommentURL=\"http://foo.com/hogehoge\";Port=\"80, 8080\";Version=1"
          "guest-account=87975348;Domain=zzz.com;Path=/zzz;Secure;Comment=\"ZzzZzz, OooOoo\";CommentURL=\"http://foo.com/hogehoge\"")
-       (construct-cookie-string *cookie-spec* 1))
+       (construct-cookie-string *cookie-spec*))
 
-;; test for formatting srfi-19 time/date
-(test* "cookie, old, srfi-19 date"
+;; test for formatting SRFI-19 time/date
+(test* "cookie, SRFI-19 date"
        '("foo=bar;Expires=Sun, 09-Sep-2001 01:46:40 GMT"
          "foo=baz;Expires=Sun, 09-Sep-2001 01:46:40 GMT")
        (construct-cookie-string
@@ -448,9 +501,7 @@
       [d1 "data:application/octed-stream;base64,IUAjJCVeJiooKV9hYmM="])
   (test* "data uri decode->encode (text)" d0
          (receive (ct data) (uri-decompose-data d0)
-           (cond-expand
-            [gauche.ces.utf8 (uri-compose-data data)]
-            [else (uri-compose-data data :content-type ct)])))
+           (uri-compose-data data)))
   (test* "data uri decode->encode (binary)" d1
          (receive (ct data) (uri-decompose-data d1)
            (uri-compose-data data :content-type ct)))
@@ -810,8 +861,8 @@
 
 (use rfc.uuid)
 (test-module 'rfc.uuid)
-(use srfi-13)
-(use srfi-27)
+(use srfi.13)
+(use srfi.27)
 
 (let1 u "f81d4fae-7dec-11d0-a765-00a0c91e6bf6"
   (test* "uuid-parse & write 1" u
@@ -822,6 +873,11 @@
          (uuid->string (parse-uuid #"{~u}")))
   (test* "uuid-parse & write 4" u
          (uuid->string (parse-uuid (string-delete #\- u))))
+
+  (test* "uuid-parse error case" (test-error)
+         (parse-uuid "kaboom"))
+  (test* "uuid-parse error case (#f)" #f
+         (parse-uuid "kaboom" :if-invalid #f))
   )
 
 (let* ([u1 (uuid1)]
