@@ -2,10 +2,11 @@
 
 (use gauche.test)
 (use gauche.process)
-(use srfi-13)
+(use srfi.13)
 (use file.util)
 (use file.filter)
 (use gauche.config)
+(use gauche.threads)
 
 (test-start "utility scripts")
 
@@ -82,7 +83,7 @@
              (process-output->string '("./gosh" "-ftest" "test.o")))
          (delete-files "test.o")))
 
-(test* "r7rs + srfi-22 by interpreter name" "baz bar"
+(test* "r7rs + SRFI-22 by interpreter name" "baz bar"
        (unwind-protect
            (begin
              (delete-files "test.o" "scheme-r7rs")
@@ -169,6 +170,7 @@
                      (dolist [line (port->string-list in)]
                        (display ($ regexp-replace-all* line
                                    #/@@author@@/ ""
+                                   #/@@license@@/ ""
                                    #/@@/ ""
                                    #/\(cf-output-default\)/
                                    "(cf-define 'HAVE_STDIO_H \"1\")\n\
@@ -180,8 +182,6 @@
                    :output outfile))
     (filter-copy (build-path extdir "package-templates" "configure")
                  "test.o/configure")
-    (filter-copy (build-path extdir "package-templates" "configure-compat")
-                 "test.o/configure-compat")
     (filter-copy (build-path extdir "package-templates" "package.scm")
                  "test.o/package.scm")
     (filter-copy (build-path extdir "package-templates" "Makefile.in")
@@ -213,14 +213,14 @@
   (test* "Makefile substitution" '()
          (and (file-exists? "test.o/Makefile")
               (filter #/@\w+@/ (file->string-list "test.o/Makefile"))))
-  (test* "VERSION generation" "1.0\n"
+  (test* "VERSION generation" "0.1\n"
          (and (file-exists? "test.o/VERSION")
               (file->string "test.o/VERSION")))
   (test* "srcdir etc."
          '("srcdir = ." "top_srcdir = ../" "builddir = ." "top_builddir = ../")
          (file->string-list "test.o/src/Makefile"))
 
-  (test* "configure --version" "package configure 1.0"
+  (test* "configure --version" "package configure 0.1"
          (read-line
           (process-output
            (run-with-parent-directory-in-paths
@@ -278,6 +278,7 @@
                      (dolist [line (port->string-list in)]
                        (display (regexp-replace-all* line
                                                      #/@@author@@/ ""
+                                                     #/@@license@@/ ""
                                                      #/@@/ "")
                                 out)
                        (newline out)))
@@ -506,8 +507,8 @@
     (if in-place?
       `(../../gosh -q -I../../../src -I../../../lib
                    ../run compile
-                   --verbose test test.c testlib.stub)
-      `(,gauche-package compile --verbose test test.c testlib.stub)))
+                   --verbose test test.c testlib.scm)
+      `(,gauche-package compile --verbose test test.c testlib.scm)))
   (define test-command
     (if in-place?
       `(../../gosh -ftest -I. ./test.scm)
@@ -543,7 +544,7 @@
               '("configure" "Makefile.in" "package.scm"
                 "test.scm" "test/tester.scm")
               '("configure" "Makefile.in" "package.scm"
-                "test.c" "test.h" "test.scm" "testlib.stub"
+                "test.c" "test.h" "test.scm" "testlib.scm"
                 "test/tester.scm")))
 
   (test-log "Running ~a" compile-command)
@@ -578,7 +579,7 @@
 (define (do-precomp! files extra-options)
   (do-process! `("../../src/gosh" "-ftest"
                  ,#"-I~|*top-srcdir*|/test/test-precomp"
-                 ,(build-path *top-srcdir* "src/precomp")
+                 ,(build-path *top-srcdir* "lib/tools/precomp")
                  "--strip-prefix"
                  ,(fix-path (build-path *top-srcdir* "test/test-precomp"))
                  ,@extra-options
@@ -611,6 +612,7 @@
        (with-output-to-file "test.o/t.scm"
          (^[]
            (write '(add-load-path "."))
+           (write '(use gauche.threads)) ; some tests need this
            (write '(load libname))
            (write '(write expr))
            (write '(exit 0))))
@@ -641,7 +643,7 @@
   )
 
 (add-load-path "test.o")
-(use srfi-42)
+(use srfi.42)
 (use scheme.vector :only (vector-every))
 
 (define (precomp-test-2)
@@ -659,7 +661,7 @@
           [(number? x)
            (and (number? y)
                 (cond [(nan? x) (nan? y)]
-                      [(-zero? x) (-zero? y)]
+                      [(negative-zero? x) (negative-zero? y)]
                       [else (= x y)]))]
           [else (equal? x y)]))
 
@@ -779,20 +781,34 @@
             ((global-variable-ref 'types-test 'foo) "ng"))))
   )
 
+(define (precomp-test-4)
+  (test* "running precomp 4" #t (do-precomp! '("literal-mutex.scm") '("-e")))
+  (test* "compile 4" #t (do-compile! "literal-mutex" '("literal-mutex.c")))
+
+  (test* "run-once" '(1 1 1 1 1)
+         (dynload-and-eval
+          "literal-mutex"
+          (let* ([foo (module-binding-ref 'literal-mutex 'foo)]
+                 [thrs (map (^_ (make-thread foo)) (iota 5))])
+            (for-each thread-start! thrs)
+            (map thread-join! thrs))))
+  )
+
 (wrap-with-test-directory precomp-test-1 '("test.o"))
 (wrap-with-test-directory precomp-test-2 '("test.o"))
 (wrap-with-test-directory precomp-test-3 '("test.o"))
+(wrap-with-test-directory precomp-test-4 '("test.o"))
 
 ;;=======================================================================
 (test-section "build-standalone")
 
 (test-script
- (build-path (or (sys-getenv "top_srcdir") "..") "src" "build-standalone"))
+ (build-path (or (sys-getenv "top_srcdir") "..") "lib" "tools" "build-standalone"))
 
 (define (run-build-static opts args)
   (do-process `("../../src/gosh" "-ftest"
+                ,(build-path *top-srcdir* "lib/tools/build-standalone")
                 ,#"-I=~|*top-srcdir*|/test/test-static"
-                ,(build-path *top-srcdir* "src/build-standalone")
                 ,#"--header-dir=~|*top-srcdir*|/src"
                 ,#"--header-dir=~|*top-builddir*|/src"
                 ,#"--header-dir=~|*top-srcdir*|/gc/include"
@@ -810,7 +826,7 @@
                              "foo/bar.scm"
                              "foo/bar-impl.scm"
                              "foo/baz.scm")))
-  (test* "static link executable" "ARGS: (A b CdE)"
+  (test* "static link executable" "ARGS: #(A b CdE)"
          (process-output->string '(test.o/staticmain A b CdE)))
   )
 
