@@ -133,7 +133,7 @@
 ;;-------------------------------------------------------------------
 (test-section "diff")
 (use text.diff)
-(use srfi-13)
+(use srfi.13)
 (test-module 'text.diff)
 
 (define diff-a "foo
@@ -227,7 +227,7 @@ fuga
 ;;-------------------------------------------------------------------
 (test-section "edn")
 (use text.edn)
-(use srfi-13)
+(use srfi.13)
 (use gauche.uvector)
 (test-module 'text.edn)
 
@@ -321,7 +321,7 @@ fuga
 ;;-------------------------------------------------------------------
 (test-section "html-lite")
 (use text.html-lite)
-(use srfi-13)
+(use srfi.13)
 (test-module 'text.html-lite)
 
 (test* "html-escape-string"
@@ -355,7 +355,7 @@ fuga
        (map string-trim-both
             (string-split (html-doctype :type :xhtml-1.0) #\newline)))
 
-(use srfi-13)
+(use srfi.13)
 
 (let ()
   ;; NB: avoid using tree->string, for we haven't tested it yet.
@@ -382,6 +382,226 @@ fuga
          (flatten (html:img :src "foo" :alt "bar baz")))
   )
 
+;;-------------------------------------------------------------------
+(test-section "info")
+(use text.info)
+(test-module 'text.info)
+
+;;-------------------------------------------------------------------
+(test-section "multicolumn")
+(use text.multicolumn)
+(test-module 'text.multicolumn)
+
+(let ([data '("alike" "ancient" "attend" "blood" "both"
+              "break" "bury" "but" "children's" "civil" "continuance"
+              "could" "death" "death-mark'd" "dignity" "do" "ears"
+              "end" "fair" "fatal" "fearful" "foes" "forth" "from"
+              "grudge" "hands" "here" "hours" "households" "if"
+              "in" "is" "lay" "life" "loins" "love" "lovers" "makes"
+              "mend" "misadventured" "miss" "mutiny" "new" "nought"
+              "now" "of" "our" "overthrows" "pair" "parents"
+              "passage" "patient" "piteous" "rage" "remove" "scene"
+              "shall" "stage" "star-cross'd" "strife" "strive" "take"
+              "the" "their" "these" "to" "toil" "traffic" "two" "unclean"
+              "verona" "we" "what" "where" "which" "whose" "with" "you")])
+  (define (t expect . args)
+    (test*/diff (write-to-string `(display-multicolumn data ,@args))
+                expect
+                (with-output-to-string
+                  (cut apply display-multicolumn data args))))
+
+  (t '(
+       "alike          fearful        miss           strive"
+       "ancient        foes           mutiny         take"
+       "attend         forth          new            the"
+       "blood          from           nought         their"
+       "both           grudge         now            these"
+       "break          hands          of             to"
+       "bury           here           our            toil"
+       "but            hours          overthrows     traffic"
+       "children's     households     pair           two"
+       "civil          if             parents        unclean"
+       "continuance    in             passage        verona"
+       "could          is             patient        we"
+       "death          lay            piteous        what"
+       "death-mark'd   life           rage           where"
+       "dignity        loins          remove         which"
+       "do             love           scene          whose"
+       "ears           lovers         shall          with"
+       "end            makes          stage          you"
+       "fair           mend           star-cross'd"
+       "fatal          misadventured  strife"))
+
+  (t '(
+       "alike                    here                     piteous"
+       "ancient                  hours                    rage"
+       "attend                   households               remove"
+       "blood                    if                       scene"
+       "both                     in                       shall"
+       "break                    is                       stage"
+       "bury                     lay                      star-cross'd"
+       "but                      life                     strife"
+       "children's               loins                    strive"
+       "civil                    love                     take"
+       "continuance              lovers                   the"
+       "could                    makes                    their"
+       "death                    mend                     these"
+       "death-mark'd             misadventured            to"
+       "dignity                  miss                     toil"
+       "do                       mutiny                   traffic"
+       "ears                     new                      two"
+       "end                      nought                   unclean"
+       "fair                     now                      verona"
+       "fatal                    of                       we"
+       "fearful                  our                      what"
+       "foes                     overthrows               where"
+       "forth                    pair                     which"
+       "from                     parents                  whose"
+       "grudge                   passage                  with"
+       "hands                    patient                  you")
+      :minimum-width 25)
+
+  (t '("alike          ears           lay            pair           these"
+       "ancient        end            life           parents        to"
+       "attend         fair           loins          passage        toil"
+       "blood          fatal          love           patient        traffic"
+       "both           fearful        lovers         piteous        two"
+       "break          foes           makes          rage           unclean"
+       "bury           forth          mend           remove         verona"
+       "but            from           misadventured  scene          we"
+       "children's     grudge         miss           shall          what"
+       "civil          hands          mutiny         stage          where"
+       "continuance    here           new            star-cross'd   which"
+       "could          hours          nought         strife         whose"
+       "death          households     now            strive         with"
+       "death-mark'd   if             of             take           you"
+       "dignity        in             our            the"
+       "do             is             overthrows     their")
+     :max-columns 7)
+
+  (t '("alike          ancient        attend         blood"
+       "both           break          bury           but"
+       "children's     civil          continuance    could"
+       "death          death-mark'd   dignity        do"
+       "ears           end            fair           fatal"
+       "fearful        foes           forth          from"
+       "grudge         hands          here           hours"
+       "households     if             in             is"
+       "lay            life           loins          love"
+       "lovers         makes          mend           misadventured"
+       "miss           mutiny         new            nought"
+       "now            of             our            overthrows"
+       "pair           parents        passage        patient"
+       "piteous        rage           remove         scene"
+       "shall          stage          star-cross'd   strife"
+       "strive         take           the            their"
+       "these          to             toil           traffic"
+       "two            unclean        verona         we"
+       "what           where          which          whose"
+       "with           you")
+     :order 'row)
+
+  (t '(
+       "alike          here           piteous"
+       "ancient        hours          rage"
+       "attend         households     remove"
+       "blood          if             scene"
+       "both           in             shall"
+       "break          is             stage"
+       "bury           lay            star-cross'd"
+       "but            life           strife"
+       "children's     loins          strive"
+       "civil          love           take"
+       "continuance    lovers         the"
+       "could          makes          their"
+       "death          mend           these"
+       "death-mark'd   misadventured  to"
+       "dignity        miss           toil"
+       "do             mutiny         traffic"
+       "ears           new            two"
+       "end            nought         unclean"
+       "fair           now            verona"
+       "fatal          of             we"
+       "fearful        our            what"
+       "foes           overthrows     where"
+       "forth          pair           which"
+       "from           parents        whose"
+       "grudge         passage        with"
+       "hands          patient        you")
+     :width 50)
+
+  (t '(
+       "     alike          here           piteous"
+       "     ancient        hours          rage"
+       "     attend         households     remove"
+       "     blood          if             scene"
+       "     both           in             shall"
+       "     break          is             stage"
+       "     bury           lay            star-cross'd"
+       "     but            life           strife"
+       "     children's     loins          strive"
+       "     civil          love           take"
+       "     continuance    lovers         the"
+       "     could          makes          their"
+       "     death          mend           these"
+       "     death-mark'd   misadventured  to"
+       "     dignity        miss           toil"
+       "     do             mutiny         traffic"
+       "     ears           new            two"
+       "     end            nought         unclean"
+       "     fair           now            verona"
+       "     fatal          of             we"
+       "     fearful        our            what"
+       "     foes           overthrows     where"
+       "     forth          pair           which"
+       "     from           parents        whose"
+       "     grudge         passage        with"
+       "     hands          patient        you")
+     :width 50 :indent 5)
+
+  (t '(
+       "     alike          misadventured"
+       "     ancient        miss"
+       "     attend         mutiny"
+       "     blood          new"
+       "     both           nought"
+       "     break          now"
+       "     bury           of"
+       "     but            our"
+       "     children's     overthrows"
+       "     civil          pair"
+       "     continuance    parents"
+       "     could          passage"
+       "     death          patient"
+       "     death-mark'd   piteous"
+       "     dignity        rage"
+       "     do             remove"
+       "     ears           scene"
+       "     end            shall"
+       "     fair           stage"
+       "     fatal          star-cross'd"
+       "     fearful        strife"
+       "     foes           strive"
+       "     forth          take"
+       "     from           the"
+       "     grudge         their"
+       "     hands          these"
+       "     here           to"
+       "     hours          toil"
+       "     households     traffic"
+       "     if             two"
+       "     in             unclean"
+       "     is             verona"
+       "     lay            we"
+       "     life           what"
+       "     loins          where"
+       "     love           which"
+       "     lovers         whose"
+       "     makes          with"
+       "     mend           you")
+     :width 45 :indent 5)
+  )
+
 ;;-------------------------------------------------------------------
 (test-section "parse")
 (use text.parse)
@@ -622,6 +842,36 @@ fuga
 (use text.pager)
 (test-module 'text.pager)
 
+;;-------------------------------------------------------------------
+(test-section "segmented-match")
+(use text.segmented-match)
+(test-module 'text.segmented-match)
+
+(let ([data '(;; pattern (success-word ...) (fail-word ...)
+              ("a-b-c"
+               ("a-b-c" "a-b-c-d" "a-b-c-d-e" "a-b-c-"
+                "abc-bcd-cde" "abc-bcd-cde-def")
+               ("a-b" "ab-c" "abc-cde" "a-b-"))
+              ("abc"
+               ("abc" "abcd" "abc-abc" "abc-")
+               ("ab" "-abc" "a-bc"))
+              ("a--b"
+               ("a--b" "aaa--bbb")
+               ("a-b")))])
+  (dolist [pattest data]
+    (let ([pattern (car pattest)])
+      (dolist [succ (cadr pattest)]
+        (test* `(segmented-prefix ,pattern ,succ) #t
+               (segmented-prefix? pattern succ)))
+      (dolist [fail (caddr pattest)]
+        (test* `(segmented-prefix ,pattern ,fail) #f
+               (segmented-prefix? pattern fail)))
+      ))
+  )
+
+(test* "segmented match (alternative separator)" #t
+       (segmented-prefix? "/u/b/g" "/usr/bin/gosh" #\/))
+
 ;;-------------------------------------------------------------------
 (test-section "template")
 (use text.template)
